@@ -54,14 +54,37 @@ let execvp_command cmd args handler =
       let sock = handler in_read out_write in
       sock, id
 
-let fork_and_exec cmd args = 
+let fork_and_exec cmd ?vars args = 
+  let env =
+    match vars with
+      | None -> None
+      | Some vars ->
+	  (* convert environment to an alist *)
+	  let env = List.map (fun kev -> 
+	    try
+	      let len = String.length kev in
+	      let equal_sign = String.index kev '=' in
+		(String.sub kev 0 equal_sign, 
+		 String.sub kev (equal_sign + 1) (len - 1 - equal_sign))
+	    with Not_found -> (* May that happen ? *)
+	      (kev, "")
+	  ) (Array.to_list (Unix.environment ())) in
+	  (* update parent's environment with vars alist *)
+	  let env = List.fold_left (fun acc ((k, v) as assoc) ->
+	    assoc :: List.remove_assoc k acc
+	  ) env vars in
+	  Some (Array.of_list (List.map (fun (k, v) -> k ^ "=" ^ v) env)) in
   match Unix.fork() with
     0 -> begin
         try
           match Unix.fork() with
             0 -> begin
                 try
-                  Unix.execv cmd args;
+		  match env with
+		    | None ->
+			Unix.execv cmd args;
+		    | Some env ->
+			Unix.execve cmd args env;
                   exit 0
                 with e -> 
                     lprintf "Exception %s while starting file_completed_cmd\n" (Printexc2.to_string e); 
@@ -85,35 +108,15 @@ let max_filedescs = (max_all_sockets - max_sockets) / 2
 
 let chroot = Unix.chroot  
 
-
 let write = Unix.write
-  
-let detach_daemon () =
-  try
-    let pid =  Unix.fork () in
-    if pid < 0 then failwith "Error in fork";
-    if pid > 0 then exit 0;
-    let sid = Unix.setsid () in
-
-(* Changed in 2.5.27 *)
-    begin
-      let stdin_new = Unix.openfile "/dev/null" [O_RDONLY] 0o444 in
-      Unix.dup2 stdin_new Unix.stdin;
-      Unix.close stdin_new;
-    end;
-    
-    begin
-      let stdout_new = Unix.openfile "/dev/null" [O_WRONLY] 0o666 in
-      Unix.dup2 stdout_new Unix.stdout;
-      Unix.dup2 stdout_new Unix.stderr;
-      Unix.close stdout_new;
-    end;
-    
-    Printf2.detach ();
-        
-  with e ->
-      lprintf "Exception %s in detach_daemon\n"
-        (Printexc2.to_string e); 
-      exit 2
 
 let set_nonblock = Unix.set_nonblock
+
+external glibc_version : unit -> string = "glibc_version"
+
+let glibc_version_num () =
+  begin
+    try
+      let s = Printf.sprintf "%s" (glibc_version ()) in s
+    with e -> ""
+  end
