@@ -30,6 +30,121 @@ open CommonOptions
 open CommonTypes
 open CommonFile
 
+(* prints a new logline with date, module and starts newline *)
+let lprintf_nl () =
+  lprintf "%s[cCO] "
+    (log_time ()); lprintf_nl2
+      
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[cCO] "
+    (log_time ()); lprintf
+
+(*************************************************************************)
+(*                                                                       *)
+(*                         FILES                                         *)
+(*                                                                       *)
+(*************************************************************************)
+
+module FileOption = struct
+    
+    let value_to_state v =
+      match v with
+      | StringValue "Paused" -> FilePaused
+      | StringValue "Downloading" -> FileDownloading
+      | StringValue "Downloaded" -> FileDownloaded
+      | _ -> raise Not_found
+    
+    let state_to_value s = 
+      match s with
+      | FilePaused | FileAborted _ -> StringValue "Paused"
+      | FileDownloaded -> StringValue "Downloaded"
+      | _ -> StringValue "Downloading"
+    
+    
+    let value_to_file is_done v =
+      match v with
+        Options.Module assocs ->
+          let get_value name conv = conv (List.assoc name assocs) in
+          let network = try get_value "file_network" value_to_string
+            with _ -> "Donkey" in
+          let network = 
+            try network_find_by_name network with e ->
+                lprintf_nl ()
+		  "Error %s for network %s while parsing file %s"
+		    (Printexc2.to_string e)
+		    network
+		    (get_value "file_filename" value_to_string);
+		lprintf_nl () "This core is lacking support for network %s, exiting" network;
+                exit_properly 70
+          in
+          let file_state = 
+            try
+              get_value "file_state" value_to_state 
+            with _ -> FileDownloading
+          in          
+          let file_size = try
+              value_to_int64 (List.assoc "file_size" assocs) 
+            with _ -> Int64.zero
+          in
+          
+          let file = network_file_of_option network file_size 
+              file_state assocs in
+          let priority = try get_value "file_priority" value_to_int 
+            with _ -> 0 in
+          
+          let impl = as_file_impl file in
+          (try
+              impl.impl_file_age <- 
+                normalize_time (get_value "file_age" value_to_int)
+            with _ -> ());
+          set_file_state file file_state;       
+          if !verbose then
+          (match file_state with
+              FileDownloading -> lprintf_nl () "New downloading file";
+            | FileDownloaded -> lprintf_nl () "New downloaded file";
+            | _ -> lprintf_nl () "New file with other state"
+          );
+          
+          (try
+              set_file_best_name file
+              (get_value "file_filename" value_to_string)
+            with _ -> ());
+          set_file_priority file priority;
+          file
+      | _ -> assert false
+          
+    let file_to_value file =
+        let netname = string_to_value (file_network file).network_name in
+      let impl = as_file_impl file in
+      Options.Module (
+        ("file_network", netname) ::
+        ("file_size", int64_to_value (file_size file)) ::
+        ("file_priority", int_to_value (file_priority file)) ::
+        ("file_state", state_to_value (file_state file)) ::
+        ("file_filename", string_to_value (file_best_name file)) ::
+        ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
+          (file_to_option file)
+        )
+          
+    let t is_done =
+      define_option_class "File" (value_to_file is_done) file_to_value
+    ;;
+  end
+
+
+let swarmers_section = file_section files_ini [] ""
+let files_section = file_section files_ini [] "" 
+  
+let done_files = 
+  define_option files_section ["done_files"] 
+    "The files whose download is finished" (
+    listiter_option (FileOption.t true)) []
+  
+let files = 
+  define_option files_section ["files"] 
+    "The files currently being downloaded" (
+    listiter_option (FileOption.t false)) []
 
 (*************************************************************************)
 (*                                                                       *)
@@ -44,17 +159,18 @@ let value_to_tag =
           IntValue i -> Uint64 i
         | _ -> String (value_to_string v2)
       in
-      { tag_name = name; tag_value = value })
+      { tag_name = field_of_string name; tag_value = value })
   
 let tag_to_value = 
   tuple2_to_value (fun tag ->
-      string_to_value tag.tag_name, 
+      string_to_value (string_of_field tag.tag_name), 
       match tag.tag_value with
         Uint64 i -> int64_to_value i
       | String s -> string_to_value s
       | Addr _ -> assert false
       | Fint64 i -> int64_to_value i
       | Uint16 n | Uint8 n -> int_to_value n
+      | Pair (n1,n2) -> assert false
   )
   
 module ResultOption = struct 
@@ -90,12 +206,12 @@ module ResultOption = struct
       | _ -> assert false
     
     let result_to_value rs =
-      let r = CommonResult.get_result rs in
+      let r = CommonResult.IndexedResults.get_result rs in
       
       let tags = ref [] in
       List.iter (fun tag ->
           match tag.tag_name with
-            "availability" | "completesources" -> ()
+            Field_Availability | Field_Completesources -> ()
           | _ -> tags := tag :: !tags;
       ) r.result_tags;
       
@@ -150,7 +266,7 @@ module ServerOption = struct
           in
           let network = 
             try network_find_by_name network with e ->
-                lprintf "Network %s not supported\n" network;
+                lprintf_nl () "Network %s not supported" network;
                 raise e
               in
           let server = network_server_of_option network assocs in
@@ -481,37 +597,47 @@ module SharingOption = struct
               try conv (List.assoc name assocs) 
               with _ -> default 
             in
-            let sharing_recursive = get_value "recursive"
-                value_to_bool false 
+            let get_bool_value name =  get_value name value_to_bool false in
+            let sharing_recursive = get_bool_value "recursive" in
+            let sharing_minsize = get_value "minsize" value_to_int64 zero 
             in
-            let sharing_minsize = get_value "minsize"
-                value_to_int64 zero 
-            in
-            let sharing_maxsize = get_value "maxsize"
-                value_to_int64 Int64.max_int 
+            let sharing_maxsize = get_value "maxsize" value_to_int64
+              Int64.max_int 
             in
             let sharing_extensions = get_value "extensions"
                 (value_to_list value_to_string) [] 
             in
+            let sharing_directories = get_bool_value "directories" in
+            let sharing_incoming = get_bool_value "incoming" in
             {
-             sharing_recursive = sharing_recursive; 
-             sharing_minsize = sharing_minsize; 
-             sharing_maxsize = sharing_maxsize; 
-             sharing_extensions = sharing_extensions; 
+              sharing_directories = sharing_directories;
+              sharing_incoming = sharing_incoming;
+              sharing_recursive = sharing_recursive; 
+              sharing_minsize = sharing_minsize; 
+              sharing_maxsize = sharing_maxsize; 
+              sharing_extensions = sharing_extensions; 
             }
           end
       | _ -> assert false
     
     let sharing_to_value s =
-      let list = [
-          "recursive", bool_to_value s.sharing_recursive;
+      let assocs = [
           "extensions", 
           list_to_value string_to_value s.sharing_extensions;
           "minsize", int64_to_value s.sharing_minsize;
           "maxsize", int64_to_value s.sharing_maxsize;
         ]
       in
-      Options.Module list
+      let add_bool_value name v assocs =
+        if v then
+          (name , bool_to_value v) :: assocs
+        else assocs
+      in
+      let assocs = add_bool_value "directories" s.sharing_directories assocs in
+      let assocs = add_bool_value "incoming" s.sharing_incoming assocs in
+      let assocs = add_bool_value "recursive" s.sharing_recursive assocs in
+
+      Options.Module assocs
       
     let t =
       define_option_class "Sharing" value_to_sharing
@@ -520,11 +646,40 @@ module SharingOption = struct
   end
 
 let sharing_only_directory = {
+    sharing_incoming = false;
+    sharing_directories = false;
+    sharing_extensions = [];
+    sharing_recursive = false;
+    sharing_minsize = Int64.one;
+    sharing_maxsize = Int64.max_int;
+  }  
+  
+let sharing_incoming_directories = {
+    sharing_incoming = true;
+    sharing_directories = true;
+    sharing_extensions = [];
+    sharing_recursive = false;
+    sharing_minsize = Int64.one;
+    sharing_maxsize = Int64.max_int;
+  }  
+  
+let sharing_incoming_files = {
+    sharing_incoming = true;
+    sharing_directories = false;
+    sharing_extensions = [];
+    sharing_recursive = false;
+    sharing_minsize = Int64.one;
+    sharing_maxsize = Int64.max_int;
+  }
+
+let sharing_directories = {
+      sharing_incoming = false;
+      sharing_directories = true;
       sharing_extensions = [];
       sharing_recursive = false;
-      sharing_minsize = Int64.of_int 1;
+      sharing_minsize = Int64.one;
       sharing_maxsize = Int64.max_int;
-    }  
+    }
   
 let sharing_strategies = define_option searches_section
     ["customized_sharing"] ""
@@ -533,6 +688,9 @@ let sharing_strategies = define_option searches_section
 
 (* For mp3 sharers: recursively share all .mp3 files < 10 MB *)
     "mp3s", {
+      sharing_incoming = false;
+      sharing_directories = false;
+      
       sharing_extensions = [".mp3"];
       sharing_recursive = true;
       sharing_minsize = zero;
@@ -541,6 +699,8 @@ let sharing_strategies = define_option searches_section
 
 (* For video sharers: recursively share all .avi files > 500 MB *)
     "avis", {
+      sharing_incoming = false;
+      sharing_directories = false;
       sharing_extensions = [".avi"];
       sharing_recursive = true;
       sharing_minsize = megabytes 500;
@@ -548,16 +708,53 @@ let sharing_strategies = define_option searches_section
     };
 
     "all_files", {
+      sharing_incoming = false;
+      sharing_directories = false;
       sharing_extensions = [];
       sharing_recursive = true;
-      sharing_minsize = Int64.of_int 1;
+      sharing_minsize = Int64.one;
       sharing_maxsize = Int64.max_int;
     };
     
+    "incoming_files",  sharing_incoming_files;
+    
+    "incoming_directories", sharing_incoming_directories;
+    
 (* For incoming directory, share all files in the directory (not recursive) *)
     "only_directory", sharing_only_directory;
+    
+    "directories", sharing_directories;
   ]
 
+let _ =
+  option_hook sharing_strategies (fun _ ->
+      
+      if not (List.exists 
+            (fun (_,s) -> s.sharing_incoming && s.sharing_directories) 
+          !!sharing_strategies) then
+        sharing_strategies =:= 
+          ("incoming_directories", sharing_incoming_directories) :: 
+        !!sharing_strategies;
+      
+      if not (List.exists 
+            (fun (_,s) -> s.sharing_incoming && not s.sharing_directories) 
+          !!sharing_strategies) then
+        sharing_strategies =:= 
+          ("incoming_files", sharing_incoming_files) :: !!sharing_strategies;
+      
+  )
+
+let sharing_strategies name =
+  match name with
+  | "incoming_files" -> sharing_incoming_files
+  | "incoming_directories" -> sharing_incoming_directories
+  | "only_directory" -> sharing_only_directory
+  | "directories" -> sharing_directories
+  | _ ->
+      try
+        List.assoc name !!sharing_strategies
+      with _ -> sharing_only_directory
+  
 (*************************************************************************)
 (*                                                                       *)
 (*                         SHARED                                        *)
@@ -584,16 +781,12 @@ module SharedDirectoryOption = struct
             in
             let shdir_strategy = get_value_safe "strategy"
                 value_to_string "only_directory"
-	    in
-	    let shdir_upload_lifetime = get_value_safe "upload_lifetime" value_to_int 90 in
-	    let shdir_chunks_percent = get_value_safe "chunks_percent" value_to_int 100 in
+            in
             {
               shdir_dirname = shdir_dirname; 
               shdir_strategy = shdir_strategy; 
               shdir_networks = shdir_networks; 
               shdir_priority = shdir_priority; 
-	      shdir_upload_lifetime = shdir_upload_lifetime;
-	      shdir_chunks_percent = 100;
             }
           end
       
@@ -603,9 +796,7 @@ module SharedDirectoryOption = struct
             shdir_dirname = value_to_filename dir; 
             shdir_strategy = "only_directory"; 
             shdir_networks = []; 
-            shdir_priority = value_to_int prio;
-	    shdir_upload_lifetime = 90;
-	    shdir_chunks_percent = 100;
+            shdir_priority = value_to_int prio; 
           }          
       | v -> 
           let dir = value_to_string v in
@@ -614,9 +805,7 @@ module SharedDirectoryOption = struct
             shdir_dirname = dir; 
             shdir_strategy = "only_directory"; 
             shdir_networks = []; 
-            shdir_priority = prio;
-	    shdir_upload_lifetime = 90;
-	    shdir_chunks_percent = 100;
+            shdir_priority = prio; 
           }
           
     let shared_directory_to_value s =
@@ -627,8 +816,6 @@ module SharedDirectoryOption = struct
           "strategy", 
           string_to_value  s.shdir_strategy;
           "priority", int_to_value s.shdir_priority;
-	  "upload_lifetime", int_to_value s.shdir_upload_lifetime;
-	  "chunks_percent", int_to_value s.shdir_chunks_percent;
         ]
       in
       Options.Module list
@@ -642,162 +829,106 @@ module SharedDirectoryOption = struct
   
 let shared_directories = 
   define_option CommonOptions.path_section ["shared_directories" ] 
-  "Directories where files will be shared"
+  "                Incoming and shared directories.
+   At least two entries have to be present here, one with strategy
+   incoming_files and one with strategy incoming_directories.
+   Both entries can point to the same directory.
+   If one of the two strategies is missing, MLDonkey will create a default
+   directory with its entry here.
+   Finished BT multifile downloads are committed to the first directory
+   with strategy incoming_directories. Other downloads are committed
+   to the first directory with the strategy incoming_files.
+   If more than one directory has one of the incoming_* strategies
+   it will be ignored on commit, but they are shared nonetheless.
+   Other strategies can be found in searches.ini, section customized_sharing."
     (list_option SharedDirectoryOption.t) 
   [
     {
-      shdir_dirname = (Filename.concat file_basedir "incoming");
+      shdir_dirname = "shared";
       shdir_priority = 0;
       shdir_networks = [];
       shdir_strategy = "all_files";
-      shdir_upload_lifetime = 90;
-      shdir_chunks_percent = 100;
+    };
+    {
+      shdir_dirname = "incoming/files";
+      shdir_priority = 0;
+      shdir_networks = [];
+      shdir_strategy = "incoming_files";
+    };
+    {
+      shdir_dirname = "incoming/directories";
+      shdir_priority = 0;
+      shdir_networks = [];
+      shdir_strategy = "incoming_directories";
     }
   ]
 
-module SharedDirectoryIndexOption = struct
-    let index_to_shared_directory v =
-      let i = value_to_int v in
-      try 
-        List.nth !!shared_directories i
-      with Failure "nth" ->
-      try
-        List.hd !!shared_directories
-      with Failure "hd" ->
-        restore_default shared_directories;
-	List.hd !!shared_directories
-    
-    let shared_directory_to_index sh =
-      let res = ref 0 in
-      ignore (List.fold_left (fun i s ->
-        if s.shdir_dirname = sh.shdir_dirname then
-	  res := i;
-	i + 1
-      ) 0 !!shared_directories);
-      int_to_value !res
+
+let search_incoming_files () =
+  try
+    List.find (fun s -> s.shdir_strategy = "incoming_files") 
+    !!shared_directories
+  with Not_found ->
+        let dirname = Filename.concat "incoming" "files" in
+        let s = {
+        shdir_dirname = dirname;
+        shdir_priority = 0;
+        shdir_networks = [];
+        shdir_strategy = "incoming_files";
+          }
+        in
+        shared_directories =:= s :: !!shared_directories;
+        s
+
+let incoming_files () =
+  let dir = search_incoming_files () in
+    Unix2.safe_mkdir dir.shdir_dirname;
+    Unix2.can_write_to_directory dir.shdir_dirname;
+    dir
+
+let search_incoming_directories () =
+  try
+    List.find (fun s -> s.shdir_strategy = "incoming_directories") 
+    !!shared_directories
+  with Not_found ->
+      let dirname = Filename.concat "incoming" "directories" in
+      let s = {
+          shdir_dirname = dirname;
+          shdir_priority = 0;
+          shdir_networks = [];
+          shdir_strategy = "incoming_directories";
+        }
+      in
+      shared_directories =:= s :: !!shared_directories;
+      s
+
+let incoming_directories () =
+  let dir = search_incoming_directories () in
+    Unix2.safe_mkdir dir.shdir_dirname;
+    Unix2.can_write_to_directory dir.shdir_dirname;
+    dir
+
+let _ =
+(* Check the definition of the incoming_files and incoming_directories in
+shared_directories *)
+  let verification = ref false in
+  option_hook shared_directories (fun _ ->
+      if not !verification then begin
+          verification := true;
+          ignore (incoming_files ());
+          ignore (incoming_directories ());
+          verification := false
+        end
+  )
       
-     let t =
-      define_option_class "Shared Directory Index" index_to_shared_directory
-        shared_directory_to_index
-   
-  end
-  
-let default_incoming = define_option current_section ["default_incoming" ]
-  "Shared directory where files will be commited per default"
-  SharedDirectoryIndexOption.t
-  (List.hd !!shared_directories)
-
-(*************************************************************************)
-(*                                                                       *)
-(*                         FILES                                         *)
-(*                                                                       *)
-(*************************************************************************)
-
-module FileOption = struct
-
-    open SharedDirectoryIndexOption
     
-    let value_to_state v =
-      match v with
-      | StringValue "Paused" -> FilePaused
-      | StringValue "Downloading" -> FileDownloading
-      | StringValue "Downloaded" -> FileDownloaded
-      | _ -> raise Not_found
-    
-    let state_to_value s = 
-      match s with
-      | FilePaused | FileAborted _ -> StringValue "Paused"
-      | FileDownloaded -> StringValue "Downloaded"
-      | _ -> StringValue "Downloading"
-    
-      
-    let value_to_file is_done v =
-      match v with
-        Options.Module assocs ->
-          let get_value name conv = conv (List.assoc name assocs) in
-          let network = try get_value "file_network" value_to_string
-            with _ -> "Donkey" in
-          let network = network_find_by_name network in
-          let file_state = 
-            try
-              get_value "file_state" value_to_state 
-            with _ -> FileDownloading
-          in          
-          let file_size = try
-              value_to_int64 (List.assoc "file_size" assocs) 
-            with _ -> Int64.zero
-          in
-          
-          let file = network_file_of_option network file_size 
-              file_state assocs in
-          let priority = try get_value "file_priority" value_to_int 
-            with _ -> 0 in
-          
-          let impl = as_file_impl file in
-          (try
-              impl.impl_file_age <- 
-                normalize_time (get_value "file_age" value_to_int)
-            with _ -> ());
-          set_file_state file file_state;       
-          (match file_state with
-              FileDownloading -> lprintf "New downloading file\n";
-            | FileDownloaded -> lprintf "New downloaded file\n";
-            | _ -> lprintf "..........\n"
-          );
-          
-          (try
-              set_file_best_name file
-              (get_value "file_filename" value_to_string)
-            with _ -> ());
-          set_file_priority file priority;
-	  (try 
-	      set_file_dst_share file (Some 
-	        (get_value "file_dst_share" index_to_shared_directory))
-	   with _ -> set_file_dst_share file None);
-          file
-      | _ -> assert false
-          
-    let file_to_value file =
-        let netname = string_to_value (file_network file).network_name in
-      let impl = as_file_impl file in
-      Options.Module (
-        ("file_network", netname) ::
-        ("file_size", int64_to_value (file_size file)) ::
-        ("file_priority", int_to_value (file_priority file)) ::
-        ("file_state", state_to_value (file_state file)) ::
-        ("file_filename", string_to_value (file_best_name file)) ::
-        ("file_age", IntValue (Int64.of_int impl.impl_file_age)) ::
-	(match file_dst_share file with
-	  Some sh -> [ ("file_dst_share", shared_directory_to_index sh) ]
-	  | None  -> []) @
-        (file_to_option file)
-      )
-          
-    let t is_done =
-      define_option_class "File" (value_to_file is_done) file_to_value
-    ;;
-  end
-
-let files_section = file_section files_ini [] "" 
-let swarmers_section = file_section files_ini [] ""
-
-let done_files = 
-  define_option files_section ["done_files"] 
-    "The files whose download is finished" (
-    listiter_option (FileOption.t true)) []
-  
-let files = 
-  define_option files_section ["files"] 
-    "The files currently being downloaded" (
-    listiter_option (FileOption.t false)) []
-
 (*************************************************************************)
 (*                                                                       *)
 (*                         Functions                                     *)
 (*                                                                       *)
 (*************************************************************************)
 
-let load () = 
+let load () =
   Options.load files_ini;
   Options.load servers_ini;
   Options.load searches_ini;
@@ -808,10 +939,10 @@ let load () =
   ) !!known_uids;
   known_uids =:= [];
   Options.load friends_ini
-  
-let save () = 
+
+let save () =
   networks_iter (fun n -> network_save_complex_options n);
-  
+
   Options.save_with_help files_ini;
   Options.save_with_help searches_ini;
   Options.save_with_help friends_ini;
@@ -831,5 +962,213 @@ let save () =
         Options.save_with_help results_ini;
         results =:= [];
     end;
-  lprintf "Options correctly saved";
-  CommonGlobals.print_localtime ()
+  lprintf_nl () "Options correctly saved"
+
+let save_sources () =
+  networks_iter (fun n -> network_save_sources n);
+  lprintf_nl () "Sources correctly saved"
+
+open Zip
+
+let backup_zip archive files =
+  begin
+    let oc = Zip.open_out archive in
+    try
+        List.iter (fun file ->
+	  begin
+	    try
+              let s = Unix.stat file in
+		Zip.copy_file_to_entry file oc ~level:9 ~mtime:s.Unix.st_mtime file
+	    with e ->
+	      failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) file)
+	  end
+        ) files;
+      Zip.close_out oc
+    with e ->
+	Zip.close_out oc;
+	failwith (Printf.sprintf "Zip: error %s in %s" (Printexc2.to_string e) archive)
+  end
+
+open Tar
+
+let backup_tar archive files =
+  let failed_files = ref "" in
+  let otar = Tar.open_out ~compress:`Gzip archive in
+    List.iter (fun arg ->
+    begin
+      try
+	let ic = Pervasives.open_in_bin arg in
+	let stat = Unix.stat arg in
+	let size = stat.Unix.st_size in
+	if size > Sys.max_string_length then
+	  begin
+	    Tar.close_out otar;
+	    failwith (Printf.sprintf "Tar: file %s too big" arg)
+	  end;
+	let header = 
+	  { Tar.t_name = arg;
+	    t_mode = 0o644;
+            t_uid = stat.Unix.st_uid;
+            t_gid = stat.Unix.st_gid;
+            t_size = 0;
+            t_mtime = Int32.of_float stat.Unix.st_mtime;
+            t_chksum = 0;
+            t_typeflag = REGULAR;
+            t_linkname = "";
+	    t_format = POSIX_FORMAT;
+            t_uname = "";
+            t_gname = "";
+            t_devmajor = 0;
+            t_devminor = 0;
+            t_prefix = "";
+            t_gnu = None;}
+	in
+	let s = String.create size in
+	  Pervasives.really_input ic s 0 size;
+	  Pervasives.close_in ic;
+	  Tar.output otar header s
+	with e ->
+	  let error = (Printexc2.to_string e) in
+	    if error =
+	      "Gzip.Error(\"error during compression\")"
+	      && Autoconf.windows && arg = "fasttrack.ini" then
+              (* for whatever reason this error is raised on Windows,
+                 but fasttrack.ini is stored correctly *)
+	      if !verbose_hidden_errors then
+	        lprintf_nl () "Tar: Windows specific pseudo error %s in %s" error arg
+	      else ()
+	    else
+	      begin
+		if !failed_files = "" then
+	          failed_files := arg
+		else
+    		  failed_files := Printf.sprintf "%s  %s" !failed_files arg;
+		  lprintf_nl () "Tar: error %s in %s" error arg
+              end
+    end
+    ) files;
+    Tar.close_out otar;
+    if !failed_files <> "" then
+      failwith (Printf.sprintf "Tar: error backing up %s" !failed_files)
+
+let backup_options () =
+  let counter = ref 1 in
+  let backup_prefix = "backup-" in
+  let old_backups = List.rev (List.sort (fun o -> compare o)
+    (List.filter (fun o -> (
+	String.lowercase (Filename2.extension o) = ".tar.gz"
+	|| String.lowercase (Filename2.extension o) = ".zip")
+	  && String.sub o 0 (String.length backup_prefix) = backup_prefix)
+    (Unix2.list_directory "old_config")))
+  in
+  List.iter (fun s ->
+    incr counter;
+    if !counter > !!backup_options_generations then
+        Sys.remove (Filename.concat "old_config" s)
+    ) old_backups;
+  let format =
+    begin
+      match !!backup_options_format with
+        "zip" -> ".zip"
+      | _ -> ".tar.gz"
+    end
+  in
+  begin
+    try
+    let archive = Filename.concat "old_config" (backup_prefix ^ Date.reverse (Unix.time ()) ^ format) in
+      let files = List.sort (fun o -> compare o) (List.filter (fun o ->
+	  String.lowercase (Filename2.last_extension o) = ".ini"
+	  && o <> "file_sources.ini") 
+	    (Unix2.list_directory file_basedir)) in
+      begin
+        match (Filename2.last_extension archive) with
+          ".zip" -> backup_zip archive files
+        | _ -> backup_tar archive files
+      end
+    with e -> lprintf_nl () "Exception %s while options backup" (Printexc2.to_string e); raise e
+  end;
+  lprintf_nl () "Options backup as %s correctly saved" format
+             
+let buildinfo () =
+  (
+        "MLNet Multi-Network p2p client version " ^ Autoconf.current_version
+      ^ (if Autoconf.scm_version <> "" then "\nSCM version info: " ^ Autoconf.scm_version else "")
+      ^ "\nNetworks: " ^ !networks_string
+      ^ "\nOcaml version: " ^ Sys.ocaml_version
+      ^ "\nBuild on: " ^ Autoconf.build_system ^ " (" ^ Unix2.endianness () ^ ")"
+      ^ (if Autoconf.glibc_version = "" then "" 
+          else
+            let real_glibc_version = MlUnix.glibc_version_num () in
+            if real_glibc_version = "" || real_glibc_version = Autoconf.glibc_version 
+              then " with glibc " ^ Autoconf.glibc_version
+            else 
+              Printf.sprintf " (Warning: glibc version mismatch, %s present on your system, MlDonkey was compiled with %s)"
+              real_glibc_version Autoconf.glibc_version)  
+      ^ (if Autoconf.configure_arguments <> "" then
+           "\nConfigure arguments: " ^ Autoconf.configure_arguments else "")
+      ^ (if !patches_string <> "" then "\n" ^ !patches_string else "")
+      ^ "\nFeatures:"
+          ^ (if BasicSocket.has_threads () then " threads" else " no-threads")
+          ^ (let s = Zlib.zlib_version_num () in Printf.sprintf " zlib%s" (if s <> "" then "-" ^ s else ""))
+          ^ (if Autoconf.has_bzip2 then
+	       begin
+	         let s =
+	           (* catch the exception in the case Bzip2.bzlib_version_num returns an empty string *)
+	           try
+	             List.hd(String2.split_simplify
+		   (Misc2.bzlib_version_num ()) ',') 
+		   with e -> ""
+		 in
+	         Printf.sprintf " bzip2%s" (if s <> "" then "-" ^ s else "")
+	       end
+	     else " no-bzip2")
+          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && Autoconf.has_gd_jpg then
+	       Printf.sprintf " gd(jpg/png%s)"
+	         (let s = CommonGraphics.png_version_num () in
+		    if s <> "" then "-" ^ s else "")
+	     else "")
+          ^ (if Autoconf.has_gd && Autoconf.has_gd_png && not Autoconf.has_gd_jpg then
+	       Printf.sprintf " gd(png%s)"
+	         (let s = CommonGraphics.png_version_num () in
+		    if s <> "" then "-" ^ s else "")
+	     else "")
+	  ^ (if Autoconf.has_gd && not Autoconf.has_gd_png && Autoconf.has_gd_jpg then " gd(jpg)" else "")
+	  ^ (if not Autoconf.has_gd then " no-gd" else "")
+          ^ (if Autoconf.has_iconv then " iconv" else " no-iconv")
+          ^ (if Autoconf.check_bounds then " check-bounds" else " no-check-bounds")
+  )
+  
+let runinfo () =
+  (
+        "Enabled Networks: " 
+      ^   (if Autoconf.donkey = "yes" && !!enable_donkey then " Donkey" else "")
+      ^   (if Autoconf.donkey = "yes" && !!enable_overnet then " Overnet" else "")
+      ^   (if Autoconf.donkey = "yes" && !!enable_kademlia then " Kademlia" else "")             
+      ^   (if Autoconf.bittorrent = "yes" && !!enable_bittorrent then " BitTorrent" else "")
+      ^   (if Autoconf.fasttrack = "yes" && !!enable_fasttrack then " Fasttrack" else "")
+      ^   (if Autoconf.gnutella = "yes" && !!enable_gnutella then " Gnutella" else "")
+      ^   (if Autoconf.gnutella2 = "yes" && !!enable_gnutella2 then " G2" else "")
+      ^   (if Autoconf.filetp = "yes" && !!enable_fileTP then " FileTP" else "")
+      ^  "\nServer usage: " ^ (if !!enable_servers then "enabled" else "disabled (you are not able to connect to ED2K Servers)")
+      ^ (let uname = Unix32.uname () in
+          if uname <> "" then Printf.sprintf "\nSystem info: %s" uname else "")
+      ^ "\nLanguage: " ^ Charset.default_language
+      ^ " - locale: " ^ Charset.locstr
+      ^ "\n max_string_length: " ^ string_of_int Sys.max_string_length
+      ^ " - word_size: " ^ string_of_int Sys.word_size
+      ^ " - max_array_length: " ^ string_of_int Sys.max_array_length
+      ^ "\n max file descriptors: " ^ string_of_int (Unix2.c_getdtablesize ())
+      ^ " - max useable file size: " ^ 
+	    (match Unix2.c_sizeofoff_t () with
+	     | 4 -> "2GB"
+
+	     |  _ -> Printf.sprintf "2^%d-1 bits (do the maths ;-p)" ((Unix2.c_sizeofoff_t () *8)-1)
+
+(*
+	     | _ -> Printf.sprintf "%s"
+	       ((*size_of_int64*)(string_of_float(((2. ** (float_of_int((Unix2.c_sizeofoff_t () * 8)-1))) -. 1.)/. 8.)))
+*)
+
+	     )
+  )    
+  
\ No newline at end of file
