@@ -19,6 +19,10 @@
 
 val load : unit -> unit
 val save : unit -> unit
+val save_sources : unit -> unit
+val backup_options : unit -> unit
+val buildinfo : unit -> string
+val runinfo : unit -> string
   
 val done_files :  CommonTypes.file list Options.option_record
 val files :  CommonTypes.file list Options.option_record
@@ -31,13 +35,14 @@ val customized_queries : unit ->
   (string * CommonTypes.query_entry) list 
 val special_queries : (string * string) list Options.option_record
   
-val sharing_strategies :
-  (string * CommonTypes.sharing_strategy) list Options.option_record
+val sharing_strategies : string -> CommonTypes.sharing_strategy
 
 val shared_directories : 
   CommonTypes.shared_directory list Options.option_record
+  
+val incoming_files : unit -> CommonTypes.shared_directory
+val incoming_directories : unit -> CommonTypes.shared_directory
+  
 val sharing_only_directory : CommonTypes.sharing_strategy
 
 val swarmers_section : Options.options_section
-  
-  
