@@ -34,8 +34,8 @@ type 'a file_impl = {
   mutable impl_file_last_rate : float;
   mutable impl_file_best_name : string;
   mutable impl_file_priority : int;
-  mutable impl_file_last_seen : int;
-  mutable impl_file_dst_share : CommonTypes.shared_directory option;
+    mutable impl_file_last_seen : int;
+    mutable impl_file_probable_name : string option;
 } 
 and 'a file_ops = {
   mutable op_file_network : CommonTypes.network;
@@ -57,6 +57,7 @@ and 'a file_ops = {
     mutable op_file_print_sources_html : 'a -> Buffer.t -> unit;
     mutable op_file_files : ('a -> 'a file_impl -> CommonTypes.file list);    
     mutable op_file_debug : 'a -> string;
+    mutable op_file_proposed_filenames : 'a -> string list;
 } 
 val as_file : 'a file_impl -> CommonTypes.file
 val as_file_impl : CommonTypes.file -> 'a file_impl
@@ -99,7 +100,6 @@ val last : (int64 * int) list -> int64 * int
 val sample_timer : unit -> unit
 val file_download_rate : 'a file_impl -> float
 val add_file_downloaded : CommonTypes.file -> Int64.t -> unit
-val files_by_num : unit
 val file_downloaders : CommonTypes.file -> CommonTypes.ui_conn -> int -> bool
 val colored_chunks : int array -> string
 val file_print : CommonTypes.file -> CommonTypes.ui_conn -> unit
@@ -116,8 +116,6 @@ val set_file_last_seen : CommonTypes.file -> int -> unit
 val file_debug : CommonTypes.file -> string
 val set_file_comment : CommonTypes.file -> string -> unit
 val file_comment : CommonTypes.file -> string
-val file_dst_share : CommonTypes.file -> CommonTypes.shared_directory option
-val set_file_dst_share : CommonTypes.file -> CommonTypes.shared_directory option -> unit
   
 val recover_bytes : CommonTypes.file -> (int64 * int64) list
 val file_write : CommonTypes.file -> int64 -> string -> int -> int -> unit
@@ -127,4 +125,7 @@ val file_mtime : CommonTypes.file -> float
   val file_copy :
   CommonTypes.file -> CommonTypes.file -> int64 -> int64 -> int64 -> unit
 val file_files : CommonTypes.file -> CommonTypes.file list
-  
\ No newline at end of file
+val propose_filenames : unit -> unit
+val propose_filename : CommonTypes.file -> unit
+
+val forceable_download : CommonTypes.result_info list ref
