@@ -7,15 +7,18 @@ val networks_by_name : (string, CommonTypes.network) Hashtbl.t
 val networks_by_num : (int, CommonTypes.network) Hashtbl.t
 val networks_ops : (CommonTypes.network * CommonTypes.network) list ref
 val new_network :
-    string ->     CommonTypes.network_flag list -> (unit -> string) -> (unit -> string) -> CommonTypes.network
+    string ->  string ->   CommonTypes.network_flag list -> CommonTypes.network
 
 val check_network_implementations : unit -> unit
+val network_clean_exit : CommonTypes.network -> bool
 val network_connected_servers :
   CommonTypes.network -> CommonTypes.server list
 val network_is_enabled : CommonTypes.network -> bool
 val network_save_complex_options : CommonTypes.network -> unit
+val network_save_sources : CommonTypes.network -> unit
 val network_load_complex_options : CommonTypes.network -> unit
 val network_enable : CommonTypes.network -> unit
+val network_update_options : CommonTypes.network -> unit
 val network_disable : CommonTypes.network -> unit
 val network_recover_temp : CommonTypes.network -> unit
 val network_share : CommonTypes.network -> string -> string -> int64 -> unit
@@ -37,6 +40,7 @@ val networks_iter : (CommonTypes.network -> unit) -> unit
 val networks_iter_until_true : (CommonTypes.network -> bool) -> bool
 val networks_iter_all : (CommonTypes.network -> unit) -> unit
 val networks_iter_all_until_true : (CommonTypes.network -> bool) -> bool
+val networks_for_all : (CommonTypes.network -> bool) -> bool
 val network_find_by_name : string -> CommonTypes.network
 val network_find_by_num : int -> CommonTypes.network
 val network_commands : (string * string * CommonTypes.arg_kind * string) list ref
