@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Int64ops
 open BasicSocket
 open Printf2
 open CommonGlobals
@@ -31,22 +32,6 @@ let search_num = ref 0
 let searches_by_num = Hashtbl.create 1027
 
   
-let search_string q =
-  let rec iter q =
-    match q with
-      QAnd (q1, q2) -> Printf.sprintf "(%s) AND (%s)" (iter q1) (iter q2)
-    | QOr (q1, q2) ->  Printf.sprintf "(%s) OR (%s)" (iter q1) (iter q2)
-    | QAndNot (q1, q2) ->  Printf.sprintf "(%s) AND NOT (%s)" (iter q1) (iter q2)
-    | QHasWord s -> Printf.sprintf "CONTAINS[%s]" s
-    | QHasField (f,s) -> Printf.sprintf "[%s]CONTAINS[%s]" (string_of_field f) s
-    | QHasMinVal (f,v) -> Printf.sprintf "[%s]>%Ld" (string_of_field f) v
-    | QHasMaxVal (f,v) -> Printf.sprintf "[%s]<%Ld" (string_of_field f) v
-    | QNone ->
-	lprintf "QNone in query\n";
-	""
-  in
-  iter q
-  
 let new_search user s =
   incr search_num;
   let time = last_time () in
@@ -60,7 +45,7 @@ let new_search user s =
       search_nresults = 0;
       search_results = Intmap.empty;
       search_waiting = 0;
-      search_string = search_string s.GuiTypes.search_query;
+      search_string = CommonIndexing.string_of_query s.GuiTypes.search_query;
       search_closed = false;
       op_search_new_result_handlers = [];
       op_search_end_reply_handlers = [];
@@ -106,7 +91,7 @@ let search_of_args args =
         iter args  ((QHasMaxVal(Field_Size, maxsize)) :: q)
     | "-avail"  :: avail :: args ->
         let avail = Int64.of_string avail in
-        iter args ((QHasMinVal(Field_unknown "avail", avail)) :: q)
+        iter args ((QHasMinVal(Field_Availability, avail)) :: q)
     | "-uid"  :: uid :: args ->
         iter args ((QHasField(Field_Uid, uid)) :: q)
     | "-media"  :: filetype :: args ->
@@ -124,7 +109,7 @@ let search_of_args args =
     | "-album"  :: format :: args ->
         iter args ((QHasField(Field_Album, format)) :: q)
     | "-field"  :: field :: format :: args ->
-        iter args ((QHasField(Field_unknown field, format)) :: q)
+        iter args ((QHasField(Field_UNKNOWN field, format)) :: q)
     | "-network" :: name :: args ->
         net := (network_find_by_name name).network_num;
         iter args q
@@ -160,8 +145,17 @@ let search_of_args args =
 
   
 type englob_op = IN_NOOP | IN_AND | IN_OR
+
+let can_search = ref false
   
 let custom_query buf query =
+  can_search := false;
+  Hashtbl.iter (fun name net ->
+      try if net.op_network_is_enabled () && List.mem NetworkHasSearch net.network_flags then
+            can_search := true;
+      with _ -> ()
+  ) CommonNetwork.networks_by_name;
+  if !can_search then begin
   try
     let q = List.assoc query (CommonComplexOptions.customized_queries()) in
     Printf.bprintf buf "
@@ -281,7 +275,7 @@ let custom_query buf query =
           " label (if default = "" then "" else
             try
               let size = Int64.of_string default in
-              let size = Int64.div size (Int64.of_int 1048576) in
+              let size = size // 1048576L in
               Int64.to_string size
             with _ -> "")
       
@@ -305,7 +299,7 @@ let custom_query buf query =
           " label (if default = "" then "" else
             try
               let size = Int64.of_string default in
-              let size = Int64.div size (Int64.of_int 1048576) in
+              let size = size // 1048576L in
               Int64.to_string size
             with _ -> "")
       
@@ -521,7 +515,7 @@ let custom_query buf query =
     <option value=\"\"> --- </option>            
 ";    
     Hashtbl.iter (fun name net ->
-        try if net.op_network_is_enabled () then
+        try if net.op_network_is_enabled () && List.mem NetworkHasSearch net.network_flags then
             Printf.bprintf buf "
             <option value=\"%s\"> %s </option>            
             " name name
@@ -534,14 +528,48 @@ let custom_query buf query =
     
   with Not_found ->
       Printf.bprintf buf "No custom search %s" query
+  end
+  else
+    begin
+(*       html_mods_table_header buf "searchTable" "search" [];
+      html_mods_td buf [
+        ("", "srh", "No searchable networks enabled"); ];
+ *)
+      Buffer.add_string buf "<div class=\"results\">
+<table id=\"memstatsTable\" name=\"searchTable\" class=\"search\" cellspacing=0 cellpadding=0>
+<tr><td class=\"srh\" >No searchable networks enabled</td>";
 
+      Buffer.add_string buf "</tr></table></div>\n"
+    end
+(*   Printf.bprintf buf "
+<h3> No searchable networks enabled </h3>
+  "
+ *)
 let complex_search buf =
+  can_search := false;
+  Hashtbl.iter (fun name net ->
+      try if net.op_network_is_enabled () && List.mem NetworkHasSearch net.network_flags then
+            can_search := true;
+      with _ -> ()
+  ) CommonNetwork.networks_by_name;
+  Buffer.add_string buf "<div class=\"results\">
+<table id=\"memstatsTable\" name=\"searchTable\" class=\"search\" cellspacing=0 cellpadding=0>
+<tr><td class=\"srh\" >";
+
+
   Buffer.add_string  buf
     "
 <center>
 <h2> Complex Search </h2>
 </center>
+</td>
+    ";
 
+  if !can_search then begin
+
+  Buffer.add_string  buf
+    "
+<td>
 <form action=\"submit\">
 <table border=0>
 <tr>
@@ -708,20 +736,35 @@ Min bitrate
     <option value=\"\"> --- </option>            
 ";    
   Hashtbl.iter (fun name net ->
-      try if net.op_network_is_enabled () then
+      try if net.op_network_is_enabled () && List.mem NetworkHasSearch net.network_flags then
           Printf.bprintf buf "
             <option value=\"%s\"> %s </option>            
             " name name
       with _ -> ()
   ) CommonNetwork.networks_by_name;
   Printf.bprintf buf "
-      </select></td></tr>" ;
-  
-  
+      </select></td></tr></table></form></td>";
+  end;
+  if not !can_search then begin
+(*       html_mods_table_header buf "searchTable" "search" [];
+      html_mods_td buf [
+        ("", "srh", "No searchable networks enabled"); ];
+ *)
+      Printf.bprintf buf "<td><div class=\"results\">
+<table id=\"memstatsTable\" name=\"searchTable\" class=\"search\" cellspacing=0 cellpadding=0>
+<tr><td class=\"srh\" >No searchable networks enabled</td>";
+
+      Buffer.add_string buf "</tr></table></div></td>\n"
+    end;
+    (*
+  Printf.bprintf buf "
+<h3> No searchable networks enabled </h3>
+  ";
+  *)
   Buffer.add_string buf
     "
 </table>
-</form>
+</div>
 "
   
 let search_forget user s =
@@ -750,7 +793,7 @@ let andnot q1 q2 = QAndNot (q1,q2)
 let rec mftp_query_of_query_entry qe =
   match qe with
     Q_AND ([]) -> QNone
-  | Q_AND [_] -> lprintf "Q_AND [_]\n"; QNone
+  | Q_AND [_] -> lprintf_nl "Q_AND [_]"; QNone
   | Q_COMBO _ -> assert false
   | Q_AND (h :: q) ->
       List.fold_left
@@ -758,7 +801,7 @@ let rec mftp_query_of_query_entry qe =
 	(mftp_query_of_query_entry h)
 	q
 
-  | Q_OR [_] -> lprintf "Q_OR [_]\n"; QNone
+  | Q_OR [_] -> lprintf_nl "Q_OR [_]"; QNone
   | Q_OR ([]) -> QNone
   | Q_OR (h :: q) ->
       List.fold_left
@@ -835,7 +878,7 @@ let rec mftp_query_of_query_entry qe =
         try
           let bitrate =  Int64.of_string s
           in
-          QHasMinVal(Field_unknown "bitrate", bitrate)
+          QHasMinVal(Field_UNKNOWN "bitrate", bitrate)
         with _ -> QNone
       end
 
@@ -850,209 +893,14 @@ let rec mftp_query_of_query_entry qe =
                        SEARCH FILTERING
 
 *********************************************************************)
-      
-module Indexing = struct
-    let name_bit = 1 lsl 0
-(* "size" *)
-(* "bitrate" *)
-    let artist_bit = 1 lsl 1 (* tag "Artiste" *)
-    let title_bit = 1 lsl 2  (* tag "Title" *)
-    let album_bit = 1 lsl 3 (* tag "Album" *)
-    let media_bit = 1 lsl 4 (* "type" *)
-    let format_bit = 1 lsl 5 (* "format" *)
-    let uid_bit = 1 lsl 6  (* uid *)
-      
-    let index_result index_string r =
-      
-      List.iter (fun name ->
-          index_string name name_bit
-      ) r.result_names;
-      
-      List.iter (fun tag ->
-          match tag with
-          | { tag_name = "Artist"; tag_value = String s } -> 
-              index_string s artist_bit
-          | { tag_name = "Title"; tag_value = String s } -> 
-              index_string s title_bit
-          | { tag_name = "Album"; tag_value = String s } -> 
-              index_string s album_bit
-(* we could directly use the fields of r *)
-          | { tag_name = "format"; tag_value = String s } -> 
-              index_string s format_bit
-          | { tag_name = "type"; tag_value = String s } -> 
-              index_string  s media_bit
-          | { tag_name = "length" } -> ()
-          | { tag_value = String s } -> 
-              index_string s name_bit
-          | _ -> ()
-      ) r.result_tags;
 
-      List.iter (fun uid ->
-          index_string (match Uid.to_uid uid with
-              Ed2k _ -> "ed2k"
-            | Md5Ext _ -> "ft"
-            | TigerTree _ -> "ttr"
-            | Md5 _ -> "md5"
-            | Sha1 _ -> "sha1"
-            | Bitprint _ -> "bp"
-            | BTUrl _ -> "bt"
-          ) uid_bit
-      ) r.result_uids;
-      
-      if r.result_format <> "" then
-        index_string r.result_format format_bit;
-      if r.result_type <> "" then
-        index_string r.result_type media_bit
-    
-    
-    exception EmptyQuery
-(*
-      exception InvertQuery of CommonTypes.result_info Indexer.query
-    exception PassInvertQuery of CommonTypes.result_info Indexer.query
-*)
-    
-    let has_word s bit =
-      match String2.stem s with
-        [] -> raise EmptyQuery
-      | s :: tail -> 
-          List.fold_left (fun q s ->
-              Indexer.And (q, (Indexer.HasField (bit, s)))
-          ) (Indexer.HasField (bit, s)) tail
-    
-    let query_to_indexer doc_value q =
-      let rec iter q =
-        match q with
-        | QAnd (q1, q2) ->
-            (try
-                let e1 = iter q1 in
-                try
-                  let e2 = iter q2 in
-                  Indexer.And (e1,e2)
-                with 
-                  EmptyQuery -> e1
-(*                | InvertQuery e2 -> Indexer.AndNot(e1, e2) *)
-              with 
-                EmptyQuery -> iter q2
-(*
-                  | InvertQuery e1 ->
-                  try
-                    let e2 = iter q2 in
-                    Indexer.AndNot (e2,e1)
-                  with 
-| InvertQuery e2 -> raise (InvertQuery (Indexer.And(e1, e2)))
-  *)
-            )
-        | QOr  (q1, q2) ->
-            (try
-                let e1 = iter q1 in
-                try
-                  let e2 = iter q2 in
-                  Indexer.Or (e1,e2)
-                with _ -> e1
-              with _ -> iter q2
-            )
-        
-        | QAndNot (q1, q2) ->
-            (
-(*              try *)
-              let e1 = iter q1 in
-              try
-                let e2 = iter q2 in
-                Indexer.AndNot (e1,e2)
-              with 
-                EmptyQuery -> e1
-(*                | InvertQuery e2 -> Indexer.And (e1,e2) 
-              with EmptyQuery -> 
-                  try
-                    let e2 = iter q2 in
-                    raise (PassInvertQuery e2)
-                  with InvertQuery e2 -> e2
-                  | PassInvertQuery e2 -> raise (InvertQuery e2) *)
-            )
-        
-        | QHasWord s -> has_word s 0x7fffffff
-        | QHasField (f, s) ->
-            has_word s (
-              match f with
-                Field_Type -> media_bit 
-              | Field_Format -> format_bit
-              | Field_Title -> title_bit 
-              | Field_Artist -> artist_bit 
-              | Field_Album -> album_bit 
-              | Field_Uid -> uid_bit
-              | _ -> 0x7fffffff);
-        | QHasMinVal (f,size) ->
-            Indexer.Predicate
-              (if f = Field_Size then
-                (fun doc -> 
-                    let r = doc_value doc in
-                    r.result_size >= size)
-              else (fun doc -> true))
-        
-        | QHasMaxVal (f,size) ->
-            Indexer.Predicate (
-              if f = Field_Size then
-                (fun doc -> 
-                    let r = doc_value doc in
-                    r.result_size <= size)
-              else (fun doc -> true))
-        | QNone ->
-            failwith "query_to_indexer: QNone in query"
-      in
-      iter q
-  
+module Search = struct
+    let add_search_result s r = 
+      search_add_result_in s (find_result r.result_num)
   end
-
-
-module DocIndexer = Indexer2.FullMake(Document)
-  
-  
-module MakeIndex (FilterResult : sig end) = struct      
-    
-    open Indexing
-    open FilterResult
-    open Document
-    
-    let index = DocIndexer.create ()
-    
-    let index_string doc s fields =
-      let words = String2.stem s in
-      List.iter (fun s ->
-(*          lprintf "ADD %d: [%s] in index [field %d]\n"  
-            doc.result_num s fields;  *)
-          DocIndexer.add  index s doc fields
-      ) words 
       
-    let add rs = 
-      let r = get_result rs in
-      index_result (index_string rs.stored_result_index) r
-      
-    let find s = 
-      
-      let ss = s.search_query in
-      let req = query_to_indexer Document.doc_value ss in  
-      
-      let docs = DocIndexer.query index req in
-(*  lprintf "%d results\n" (List.length docs);  *)
-      Array.iter (fun doc ->
-          if DocIndexer.filtered doc then begin
-              lprintf "doc filtered\n"; 
-            end else
-          let r = Document.doc_value doc in
-(*    merge_result s doc.num; *)
-(*          lprintf "search_add_result: %d\n" r.result_num;  *)
-          search_add_result_in s (find_result r.result_num)
-      ) docs
-
-    let clear () =
-      DocIndexer.clear index
-
-    let stats () = DocIndexer.stats index
-      
-  end  
-
-module Filter = MakeIndex(struct end)
-module Local = MakeIndex(struct end)
+module Filter = IndexedResults.MakeIndex(Search)
+module Local = IndexedResults.MakeIndex(Search)
 
 let clean_local_search = ref 0
   
@@ -1106,4 +954,4 @@ let _ =
       ) searches_by_num;
       Printf.bprintf buf "  Memorized searches: %d\n" !counter;
       Printf.bprintf buf "  Memorized items: %d\n" !items;
-  )
\ No newline at end of file
+  )
