@@ -27,6 +27,7 @@ open Printf2
 open Md4
 
 open CommonOptions
+open CommonComplexOptions
 open CommonGlobals
 open Options
 open CommonTypes
@@ -34,7 +35,6 @@ open CommonTypes
 type 'a shared_impl = {
     impl_shared_fullname : string;
     impl_shared_codedname : string;
-    impl_shared_directory : shared_directory;
     mutable impl_shared_val : 'a;
     mutable impl_shared_update : int;
     mutable impl_shared_num : int;
@@ -60,7 +60,7 @@ let as_shared_impl  (shared : shared) =
 
 let shared_num s = (as_shared_impl s).impl_shared_num
   
-module H = Weak2.Make(struct
+module H = Weak.Make(struct
       type t = shared
       let hash file = Hashtbl.hash (shared_num file)
       
@@ -73,16 +73,20 @@ let shareds_by_num = H.create 1027
 let ni n m = 
   let s = Printf.sprintf "Shared.%s not implemented by %s" 
       m n.network_name in
-  lprintf "%s\n" s;
+  lprintf_nl "%s" s;
   s
   
 let fni  n m =  failwith (ni n m)
 let ni_ok n m = ignore (ni n m)
 
+let lprintf_nl () =
+  lprintf "%s[cShared]: "
+  (log_time ()); lprintf_nl2
+    
 let shared_calculate_total_bytes () =
   nshared_bytes := Int64.zero;
   H.iter (fun s ->
-    nshared_bytes := (Int64.add !nshared_bytes (as_shared_impl s).impl_shared_size)
+    nshared_bytes := !nshared_bytes ++ (as_shared_impl s).impl_shared_size
   ) shareds_by_num
     
 let shared_must_update shared =
@@ -101,10 +105,9 @@ let shared_must_update_downloaded shared =
 
 let update_shared_num impl =
   if impl.impl_shared_num = 0 then begin
-      if !verbose then begin 
-          lprintf "NEW SHARED %s/%s\n" impl.impl_shared_codedname
+      if !verbose_share then
+          lprintf_nl () "NEW SHARED %s/%s" impl.impl_shared_codedname
             impl.impl_shared_fullname; 
-        end; 
       incr shared_counter;
       impl.impl_shared_num <- !shared_counter;
       H.add shareds_by_num (as_shared impl);
@@ -121,36 +124,37 @@ let shared_remove impl =
   H.remove shareds_by_num (as_shared impl)
     
 let dirnames = Hashtbl.create 13
+let dirnames_prio = ref []
 let dirname_counter = ref 0
 
 let files_scanned = ref 0
 let files_scanned_size = ref zero
   
-let new_shared shdir filename fullname =
+let new_shared dirname prio filename fullname =
 (*  lprintf "XXXXXXX\ndirname %s \nfilename %s \nfullname %s\n"
     dirname filename fullname;  *)
   let fullname = Filename2.normalize fullname in
   let filename = Filename2.normalize filename in
   let dirname = try
-      Hashtbl.find dirnames shdir.shdir_dirname with _ ->
+      Hashtbl.find dirnames dirname with _ ->
         incr dirname_counter;
         let name = Printf.sprintf "shared%d" !dirname_counter in
-        Hashtbl.add dirnames shdir.shdir_dirname name;
+        Hashtbl.add dirnames dirname name;
+        dirnames_prio := (name, prio) :: !dirnames_prio;
         name in
   let codedname = Filename.concat dirname filename in
   if !verbose_share then
-    lprintf "\ndirname %s \nfilename %s \nfullname %s\ncodedname %s\n"
-      dirname filename fullname codedname; 
-
-  let size = Unix32.getsize fullname in
+    lprintf_nl () "sharing %s" fullname;
+  let size = Unix32.getsize fullname false in
   incr files_scanned;
   files_scanned_size := !files_scanned_size ++ size;
   if Unix2.is_directory fullname then begin
-    lprintf "new_shared: %s is directory! Skipped network.share\n" fullname;
+    if !verbose_share then
+      lprintf_nl () "new_shared: %s is directory! Skipped network.share" fullname;
   end
   else begin
   CommonNetwork.networks_iter (fun n -> 
-      CommonNetwork.network_share n shdir fullname codedname size);
+      CommonNetwork.network_share n fullname codedname size);
   shared_calculate_total_bytes ();
   end
 
@@ -165,15 +169,27 @@ let shared_fullname s =
 let shared_codedname s =
   let impl = as_shared_impl s in
   impl.impl_shared_codedname  
-
-let shared_shared_directory s =
-  let impl = as_shared_impl s in
-  impl.impl_shared_directory
   
 let shared_unshare s =
   let impl = as_shared_impl s in
+  shared_remove impl;
   try impl.impl_shared_ops.op_shared_unshare impl.impl_shared_val with _ -> ()
 
+let shared_dir = function
+  | None	-> ""
+  | Some sh	-> 
+    try
+      let impl = as_shared_impl sh in
+      let codedname = impl.impl_shared_codedname in
+      let idx = String.index codedname '/' in
+      String.sub codedname 0 idx
+    with Not_found -> ""
+		    
+let shared_prio impl =
+  try
+    List.assoc (shared_dir impl) !dirnames_prio
+  with Not_found -> 0
+
 let new_shared_ops network = {
     op_shared_unshare = (fun _ -> ni_ok network "shared_unshare");
     op_shared_info = (fun _ -> fni network "shared_info");
@@ -182,7 +198,6 @@ let new_shared_ops network = {
 let dummy_shared = {
     impl_shared_fullname = "";
     impl_shared_codedname = "";
-    impl_shared_directory = !!CommonComplexOptions.default_incoming;
     impl_shared_val = 0;
     impl_shared_update = 0;
     impl_shared_num = 0;
@@ -203,9 +218,6 @@ let shared_find num =
 let shared_iter f =
   H.iter f shareds_by_num
 
-let file_size filename =  Unix32.getsize filename
-let local_dirname = Sys.getcwd ()
-  
 (* Prevent sharing of temp directory to avoid sending incomplete files *)
 let can_share dirname =
   Filename2.normalize dirname <> Filename2.normalize 
@@ -217,13 +229,18 @@ let shared_add_directory shared_dir local_dir =
   waiting_directories := (shared_dir, local_dir) :: !waiting_directories
   
 let shared_scan_directory shared_dir local_dir =
-  let dirname = shared_dir.shdir_dirname in
-  let strategy = try
-      List.assoc shared_dir.shdir_strategy 
-        !!CommonComplexOptions.sharing_strategies
-    with _ -> 
-        CommonComplexOptions.sharing_only_directory    
+  let incoming_files_inode = 
+    ((Unix.stat ((CommonComplexOptions.incoming_files ()).shdir_dirname)).Unix.st_ino)
   in
+  let incoming_directories_inode = 
+    ((Unix.stat ((CommonComplexOptions.incoming_directories ()).shdir_dirname)).Unix.st_ino)
+  in
+  let temp_directory_inode =
+    ((Unix.stat !!temp_directory).Unix.st_ino)
+  in
+  let dirname = shared_dir.shdir_dirname in
+  let strategy = 
+    CommonComplexOptions.sharing_strategies shared_dir.shdir_strategy in
   let dirname = 
     if Filename.is_relative dirname then
       Filename.concat file_basedir dirname
@@ -233,19 +250,34 @@ let shared_scan_directory shared_dir local_dir =
   if can_share dirname then
     try
       let files = Unix2.list_directory full_dir in
-       if !verbose_share then lprintf "Sharing sub-directory %s\n" full_dir;
+      if !verbose_share then lprintf_nl () "Sharing sub-directory %s" full_dir; 
       List.iter (fun file ->
           if file <> "" && file.[0] <> '.' then
             let full_name = Filename.concat full_dir file in
             let local_name = Filename.concat local_dir file in
             try
               if Unix2.is_directory full_name then begin
-                  if strategy.sharing_recursive then
+	        if strategy.sharing_recursive then
+	          if Sys.os_type <> "Win32" then (* Win32 means MinGW here as it does not supports inode nums *)
+	          begin
+		    let inode = ((Unix.stat full_name).Unix.st_ino) in
+		      if inode = incoming_files_inode then
+		        if !verbose_share then lprintf_nl () "avoid sharing incoming_files %s" full_dir else ()
+		      else
+		        if inode = incoming_directories_inode then
+			  if !verbose_share then lprintf_nl () "avoid sharing incoming_directories %s" full_dir else ()
+			else
+			  if inode = temp_directory_inode then
+			    if !verbose_share then lprintf_nl () "avoid sharing temp %s" full_dir else ()
+			  else
+			    shared_add_directory shared_dir local_name
+                  end
+                  else
                     shared_add_directory shared_dir local_name
-                end
+		end
               else
               try
-                let size = file_size full_name in
+                let size = (Unix32.getsize full_name) false in
                 if size > strategy.sharing_minsize && 
                   size < strategy.sharing_maxsize &&
                   (strategy.sharing_extensions = [] ||
@@ -253,17 +285,18 @@ let shared_scan_directory shared_dir local_dir =
                         Filename2.last_extension full_name)) 
                     strategy.sharing_extensions)
                 then
-                  new_shared shared_dir local_name full_name
+                  new_shared dirname shared_dir.shdir_priority
+                    local_name full_name
               with e -> 
-                  lprintf "%s will not be shared (exception %s)\n"
+                  lprintf_nl () "%s will not be shared (exception %s)"
                     full_name (Printexc2.to_string e);
             with _ -> ()
       ) files
     with e -> 
-        lprintf "Exception %s while sharing %s\n"
+        lprintf_nl () "Exception %s while sharing %s"
            (Printexc2.to_string e) full_dir
   else 
-    lprintf "Cannot share %s\n" full_dir
+    lprintf_nl () "Cannot share %s" full_dir
 
 let _ = 
   BasicSocket.add_infinite_timer 1. (fun _ ->
@@ -280,7 +313,7 @@ let _ =
     
 let shared_add_directory shared_dir =
   if shared_dir.shdir_dirname <> "" then begin
-      if !verbose_share then lprintf "SHARING %s PRIO %d\n" shared_dir.shdir_dirname
+      if !verbose_share then lprintf_nl () "Sharing %s prio %d" shared_dir.shdir_dirname
         shared_dir.shdir_priority;
       shared_add_directory shared_dir ""
     end
@@ -289,15 +322,17 @@ let shared_add_directory shared_dir =
   
 let shared_check_files () =
   let list = ref [] in
+  (* check shared files, store removed files in !list *)
   H.iter (fun s ->
       let name = shared_fullname s in
       if not (Unix32.file_exists name) then list := s :: !list
   ) shareds_by_num;
+  (* unshare removed files *)
   List.iter (fun s -> shared_unshare s) !list;
   files_scanned_size := zero;
   files_scanned := 0;
   List.iter (fun s -> shared_add_directory s) 
-  !!  CommonComplexOptions.shared_directories;
+  !!CommonComplexOptions.shared_directories;
   shared_calculate_total_bytes ()
   
 let impl_shared_info impl =
@@ -309,7 +344,7 @@ let impl_shared_info impl =
     T.shared_size = impl.impl_shared_size;
     T.shared_uploaded = impl.impl_shared_uploaded;
     T.shared_requests = impl.impl_shared_requests; 
-    T.shared_id = Md4.null;
+    T.shared_uids = [];
   }
   
 let shared_info s =
@@ -321,15 +356,18 @@ let _ =
       let counter = ref 0 in
       H.iter (fun _ -> incr counter) shareds_by_num;
       Printf.bprintf buf "  shared: %d\n" !counter;
+      Printf.bprintf buf "  dirnames: %d\n" (Hashtbl.length dirnames);
   )
-
+  
 let com_shareds_by_num = shareds_by_num
 let shareds_by_num = ()
 
-open CommonComplexOptions
-
-let share_to_commit file =
-  match CommonFile.file_dst_share file with
-    Some sh -> sh
-    | None  -> !!default_incoming
-
+(* This won't fit nicely with priorities on upload.
+  Maybe the pririties could be given in another option,
+  and the association would remain independantly of the fact
+  that the directories are currently shared or not. *)
+  
+(*let _ = 
+  Options.set_string_wrappers shared_directories
+    Filepath.semipath_to_string
+    Filepath.string_to_semipath*)
