@@ -42,7 +42,7 @@ let buf = Buffer.create 1000
       
 let gui_send writer sock t = 
   try
-    Buffer.clear buf;
+    Buffer.reset buf;
     buf_int buf 0;
     writer buf t;
     let s = Buffer.contents buf in
@@ -72,6 +72,8 @@ let buf_array buf f list =
 (* if len <= 0xfffff, how would the gui know what to do when len == 0xffff? *)
 
 let buf_string buf s =
+  (* charset conversion could generate longer strings *)
+  let s = Charset.to_utf8 s in
   let len = String.length s in
   if len < 0xffff then begin
       buf_int16 buf len;
@@ -82,6 +84,9 @@ let buf_string buf s =
       Buffer.add_string buf s
     end 
 
+let buf_string_bin buf s =
+  buf_string buf s 
+    
 let buf_uid buf uid =
   buf_string buf (Uid.to_string uid)
 
@@ -118,10 +123,10 @@ let rec buf_query buf q =
   match q with
     Q_AND list ->
       buf_int8 buf 0;
-      buf_list buf buf_query  list
+      buf_list buf buf_query list
   | Q_OR list ->
       buf_int8 buf 1;
-      buf_list buf buf_query  list
+      buf_list buf buf_query list
   | Q_ANDNOT (q1,q2) ->
       buf_int8 buf 2;
       buf_query buf q1;
@@ -168,11 +173,11 @@ let rec buf_query buf q =
       buf_string buf s2
   | Q_HIDDEN list ->
       buf_int8 buf 13;
-      buf_list buf buf_query  list
+      buf_list buf buf_query list
   | Q_COMBO _ -> assert false
       
 let buf_tag buf t =
-  buf_string buf t.tag_name;
+  buf_string buf (string_of_field t.tag_name);
   match t.tag_value with
   | Uint64 s -> buf_int8 buf 0; buf_int64_32 buf s
   | Fint64 s -> buf_int8 buf 1; buf_int64_32 buf s
@@ -180,6 +185,7 @@ let buf_tag buf t =
   | Addr ip -> buf_int8 buf 3; buf_ip buf ip
   | Uint16 n -> buf_int8 buf 4; buf_int16 buf n
   | Uint8 n -> buf_int8 buf 5; buf_int8 buf n
+  | Pair (n1,n2) -> buf_int8 buf 6; buf_int64_32 buf n1; buf_int64_32 buf n2
       
 let buf_host_state proto buf t =
   if proto < 12 then
@@ -221,6 +227,12 @@ let buf_client_type buf t =
 let buf_bool buf b =
   buf_int8 buf (if b then 1 else 0)
       
+let buf_bool_option buf b =
+  buf_int8 buf
+  (match b with
+  | None -> 2
+  | Some b -> if b then 1 else 0)
+      
 let buf_result proto buf r =
   buf_int buf r.result_num;
   buf_int buf 0;
@@ -290,8 +302,58 @@ let buf_mp3 buf t =
   buf_string buf t.M.comment;
   buf_int buf t.M.tracknum;
   buf_int buf t.M.genre
-  
-let buf_format buf f =
+
+let buf_ogg_stream_type buf st =
+  match st with
+  OGG_VIDEO_STREAM  -> buf_int8 buf 0
+| OGG_AUDIO_STREAM  -> buf_int8 buf 1
+| OGG_INDEX_STREAM  -> buf_int8 buf 2
+| OGG_TEXT_STREAM   -> buf_int8 buf 3
+| OGG_VORBIS_STREAM -> buf_int8 buf 4
+| OGG_THEORA_STREAM -> buf_int8 buf 5
+
+let buf_vorbis_bitrate buf br =
+  match br with
+  Maximum_br r -> buf_int8 buf 0; buf_float buf r
+| Nominal_br r -> buf_int8 buf 1; buf_float buf r
+| Minimum_br r -> buf_int8 buf 2; buf_float buf r
+
+let buf_theora_cs buf cs =
+  match cs with
+  CSUndefined -> buf_int8 buf 0
+| CSRec470M   -> buf_int8 buf 1
+| CSRec470BG  -> buf_int8 buf 2
+
+let buf_ogg_stream_tag buf tag =
+  match tag with
+  Ogg_codec s                 -> buf_int8 buf 0; buf_string buf s
+| Ogg_bits_per_samples n      -> buf_int8 buf 1; buf_int buf n
+| Ogg_duration n              -> buf_int8 buf 2; buf_int buf n
+| Ogg_has_subtitle            -> buf_int8 buf 3
+| Ogg_has_index               -> buf_int8 buf 4
+| Ogg_audio_channels n        -> buf_int8 buf 5; buf_int buf n
+| Ogg_audio_sample_rate r     -> buf_int8 buf 6; buf_float buf r
+| Ogg_audio_blockalign n      -> buf_int8 buf 7; buf_int buf n
+| Ogg_audio_avgbytespersec r  -> buf_int8 buf 8; buf_float buf r
+| Ogg_vorbis_version r        -> buf_int8 buf 9; buf_float buf r
+| Ogg_vorbis_sample_rate r    -> buf_int8 buf 10; buf_float buf r
+| Ogg_vorbis_bitrates l       -> buf_int8 buf 11; buf_list buf buf_vorbis_bitrate l
+| Ogg_vorbis_blocksize_0 n    -> buf_int8 buf 12; buf_int buf n
+| Ogg_vorbis_blocksize_1 n    -> buf_int8 buf 13; buf_int buf n
+| Ogg_video_width r           -> buf_int8 buf 14; buf_float buf r
+| Ogg_video_height r          -> buf_int8 buf 15; buf_float buf r
+| Ogg_video_sample_rate r     -> buf_int8 buf 16; buf_float buf r
+| Ogg_aspect_ratio r          -> buf_int8 buf 17; buf_float buf r
+| Ogg_theora_cs cs            -> buf_int8 buf 18; buf_theora_cs buf cs
+| Ogg_theora_quality n        -> buf_int8 buf 19; buf_int buf n
+| Ogg_theora_avgbytespersec n -> buf_int8 buf 20; buf_int buf n
+
+let buf_ogg buf stream =
+  buf_int buf stream.stream_no;
+  buf_ogg_stream_type buf stream.stream_type;
+  buf_list buf buf_ogg_stream_tag stream.stream_tags
+
+let buf_format proto buf f =
   match f with
   | FormatUnknown | FormatNotComputed _ -> buf_int8 buf 0
   | FormatType (s1, s2) -> buf_int8 buf 1; 
@@ -305,7 +367,13 @@ let buf_format buf f =
   | MP3 (t, _) -> 
       buf_int8 buf 3;
       buf_mp3 buf t
-      
+  | OGG ogg_infos ->
+      if proto > 31
+        then begin
+          buf_int8 buf 4;
+          buf_list buf buf_ogg ogg_infos;
+        end else buf_int8 buf 0
+
 let buf_kind buf k =
   match k with
     Known_location (ip, port) -> 
@@ -355,7 +423,7 @@ let buf_partial_file proto buf f =
       buf_int8 buf 10;
       buf_list buf (fun buf (network, avail) ->
           buf_int buf network;
-          buf_string buf avail
+          buf_string_bin buf avail
       ) f.file_availability
     end;
   if f.file_fields.Fields_file_info.file_download_rate then begin
@@ -372,7 +440,7 @@ let buf_partial_file proto buf f =
     end;
   if f.file_fields.Fields_file_info.file_format then begin
       buf_int8 buf 14;
-      buf_format buf f.file_format;
+      buf_format proto buf f.file_format;
     end;
   if f.file_fields.Fields_file_info.file_name then begin
       buf_int8 buf 15;
@@ -424,7 +492,7 @@ let buf_file_field proto buf field =
       buf_int8 buf 10;
       buf_list buf (fun buf (network, avail) ->
           buf_int buf network;
-          buf_string buf avail
+          buf_string_bin buf avail
       ) x
   | Fields_file_info.File_download_rate x ->
       buf_int8 buf 11;
@@ -437,7 +505,7 @@ let buf_file_field proto buf field =
       buf_float_date proto buf x
   | Fields_file_info.File_format x ->
       buf_int8 buf 14;
-      buf_format buf x
+      buf_format proto buf x
   | Fields_file_info.File_name x ->
       buf_int8 buf 15;
       buf_string buf x
@@ -465,17 +533,17 @@ let buf_file proto buf f =
   if proto > 17 then
     buf_list buf (fun buf (network, avail) ->
         buf_int buf network;
-        buf_string buf avail
+        buf_string_bin buf avail
     ) f.file_availability
   else
-    buf_string buf (match f.file_availability with
+    buf_string_bin buf (match f.file_availability with
         [] -> ""
       | (_, av) :: _ -> av);
   buf_float buf f.file_download_rate;  
   buf_array buf (buf_float_date proto) f.file_chunks_age;  
   buf_float_date proto buf f.file_age;
 (* last, so that it can be safely discarded in partial implementations: *)
-  buf_format buf f.file_format;
+  buf_format proto buf f.file_format;
   if proto >= 8 then 
     buf_string buf f.file_name;
   if proto >= 9 then 
@@ -484,16 +552,32 @@ let buf_file proto buf f =
     if proto >= 12 then
       buf_int buf f.file_priority;
     if proto > 21 then
-      buf_string buf f.file_comment
+      buf_string buf f.file_comment;
+    if proto > 30 then
+      buf_list buf buf_uid f.file_uids;
+    if proto > 35 then
+      buf_list buf (fun buf (name, size) ->
+          buf_string buf name;
+          buf_int64 buf size
+      ) f.file_sub_files
       
-let buf_addr buf addr =
-  match addr with
+let buf_addr proto buf addr =
+  (match addr with
     Ip.AddrIp ip ->
       buf_int8 buf 0;
       buf_ip buf ip
   | Ip.AddrName s ->
       buf_int8 buf 1;
-      buf_string buf s
+      buf_string buf s);
+  if proto > 33 then begin
+    let is_blocked = 
+      try 
+        Ip_set.ip_blocked (Ip.ip_of_addr addr)
+      with _ ->
+        false
+    in
+    buf_bool buf is_blocked 
+  end
   
 let buf_server proto buf s =
   buf_int buf s.server_num;
@@ -501,7 +585,7 @@ let buf_server proto buf s =
   if proto < 2 then 
     buf_ip buf (Ip.ip_of_addr s.server_addr)
   else
-    buf_addr buf s.server_addr;    
+    buf_addr proto buf s.server_addr;    
   buf_int16 buf s.server_port;
   buf_int buf s.server_score;
   buf_list buf buf_tag s.server_tags;
@@ -512,7 +596,7 @@ let buf_server proto buf s =
   buf_string buf s.server_description;
   if proto > 28 then 
     buf_bool buf s.server_preferred
-  
+
 let buf_client proto buf c =
   buf_int buf c.client_num;
   buf_int buf c.client_network;
@@ -535,8 +619,15 @@ let buf_client proto buf c =
       if proto >= 20 then begin
           buf_int_date proto buf c.client_connect_time
       end;
-      if proto >= 21 then
-        buf_string buf c.client_emulemod;
+      if proto >= 21 then begin
+        buf_string buf c.client_emulemod
+      end;
+      if proto >= 33 then begin
+        buf_string buf c.client_release
+      end;
+      if proto >= 35 then begin
+        buf_bool_option buf c.client_sui_verified
+      end;
     end
     
 let buf_network proto buf n =
@@ -558,8 +649,8 @@ let buf_network proto buf n =
             | NetworkHasChat -> 5
             | NetworkHasSupernodes -> 6
             | NetworkHasUpload -> 7
+            | NetworkHasStats -> -1
             | UnknownNetworkFlag -> -1
-
           )
       ) n.network_netflags;
     end
@@ -588,9 +679,19 @@ let buf_shared_info proto buf s =
   buf_int64 buf s.shared_uploaded;
   buf_int buf s.shared_requests;
   if proto >= 10 then
-    buf_md4 buf s.shared_id
-    
-  
+    if proto < 31 then
+      (* only return the ed2k hash *)
+      let md4 = ref Md4.null in
+      List.iter
+        (fun s_uid ->
+          match Uid.to_uid s_uid with
+            | Ed2k hash -> md4 := hash
+            | _ -> ()
+        ) s.shared_uids;
+      buf_md4 buf !md4
+    else
+      buf_list buf buf_uid s.shared_uids
+
 (***************
 
        Encoding of messages from the Core to the GUI 
@@ -735,8 +836,9 @@ let rec to_gui (proto : int array) buf t =
           if proto < 14 then  45 else 54);
         buf_list buf (buf_file proto) list
     
-    | ConnectedServers list ->      buf_opcode buf 28;
+    | ConnectedServers list ->
         let proto = proto.(28) in
+        buf_opcode buf 28;
         buf_list buf (buf_server proto) list
     
     | Client_stats s -> 
@@ -770,7 +872,8 @@ let rec to_gui (proto : int array) buf t =
                 end
             end                
     
-    | Room_remove_user (room, user) ->       buf_opcode buf 32;
+    | Room_remove_user (room, user) ->
+        buf_opcode buf 32;
         buf_int buf room;
         buf_int buf user
     
@@ -779,16 +882,18 @@ let rec to_gui (proto : int array) buf t =
         buf_opcode buf (if proto < 10 then 33 else 48);
         buf_shared_info proto buf shared_info
     
-    | Shared_file_upload (num, upload,requests) ->    buf_opcode buf 34;
+    | Shared_file_upload (num, upload,requests) ->
+        buf_opcode buf 34;
         buf_int buf num;
         buf_int64 buf upload;
         buf_int buf requests
     | Shared_file_unshared num -> buf_opcode buf 35;
         buf_int buf num
     
-    | Add_section_option (section, o) -> buf_opcode buf 36;
+    | Add_section_option (section, o) ->
         let proto = proto.(36) in
         let module M = Options in
+        buf_opcode buf 36;
         buf_string buf section;
         let desc = if o.M.option_desc = "" 
           then o.M.option_name else o.M.option_desc in
@@ -807,9 +912,10 @@ let rec to_gui (proto : int array) buf t =
             | "Filename" -> 2
             | _ -> 0);
     
-    | Add_plugin_option (section, o) -> buf_opcode buf 38;
+    | Add_plugin_option (section, o) ->
         let proto = proto.(38) in
         let module M = Options in
+        buf_opcode buf 38;
         buf_string buf section;
         let desc = if o.M.option_desc = "" 
           then o.M.option_name else o.M.option_desc in
@@ -828,14 +934,15 @@ let rec to_gui (proto : int array) buf t =
             | "Filename" -> 2
             | _ -> 0);
     
-    | File_remove_source (n1,n2) -> buf_opcode buf 50;
+    | File_remove_source (n1,n2) ->
+        buf_opcode buf 50;
         buf_int buf n1; buf_int buf n2
     
     | File_update_availability (file_num, client_num, avail) -> 
         let proto = proto.(9) in
         if proto < 11 then raise UnsupportedGuiMessage;
         buf_opcode buf 9;
-        buf_int buf file_num; buf_int buf client_num; buf_string buf avail
+        buf_int buf file_num; buf_int buf client_num; buf_string_bin buf avail
     
     | CleanTables (clients, servers) ->      
         let proto = proto.(11) in
@@ -852,6 +959,10 @@ let rec to_gui (proto : int array) buf t =
         let proto = proto.(57) in
         buf_opcode buf 57; buf_search buf_string proto buf s
     
+    | Version s ->
+        let proto = proto.(58) in
+        buf_opcode buf 58; buf_string buf s
+    
     | GiftServerAttach _ -> assert false
     | GiftServerStats _ -> assert false
   with e ->
@@ -895,6 +1006,7 @@ let rec from_gui (proto : int array) buf t =
             buf_opcode buf 42;          
           end;
         buf_search buf_query proto buf search
+
     | Download_query (list, int, force) -> 
         let proto = proto.(50) in
         buf_opcode buf (if proto < 14 then 7 else 50);
@@ -1055,7 +1167,18 @@ protocol version. Do not send them ? *)
         
     | InterestedInSources interested ->
         buf_int16 buf 64; buf_bool buf interested
-        
+    | GetVersion -> buf_opcode buf 65; 
+
+    (* introduced with protocol 32 *)
+    | ServerRename (num, s) ->
+        buf_opcode buf 66;
+        buf_int buf num;
+        buf_string buf s
+    | ServerSetPreferred (num, preferred) ->
+        buf_opcode buf 67;
+        buf_int buf num;
+        buf_bool buf preferred
+
   with e ->
       lprintf "GuiEncoding.from_gui: Exception %s\n"
         (Printexc2.to_string e)
