@@ -142,6 +142,7 @@ type file_info = {
     mutable file_last_seen : int;
     mutable file_priority : int;
     mutable file_uids : Uid.t list;
+    mutable file_sub_files : (string * int64) list;
   }
   
 type user_info = {
@@ -160,6 +161,7 @@ type server_info = {
     
     mutable server_addr : Ip.addr;
     mutable server_port : int;
+    mutable server_realport : int;
     mutable server_score : int;
     mutable server_tags : CommonTypes.tag list;
     mutable server_nusers : int64;
@@ -206,10 +208,12 @@ type client_info = {
     mutable client_chat_port : int;
     mutable client_connect_time : int;
     mutable client_software : string;
+    mutable client_release : string;
     mutable client_emulemod : string;
     mutable client_downloaded : int64;
     mutable client_uploaded : int64;
     mutable client_upload : string option;
+    mutable client_sui_verified : bool option;
 (*  mutable client_sock_addr : string;  *)
   }
 
@@ -234,7 +238,7 @@ type shared_info = {
     mutable shared_size : int64;
     mutable shared_uploaded : int64;
     mutable shared_requests : int;
-    mutable shared_id : Md4.t;
+    mutable shared_uids : Uid.t list; (* net file UID *)
   }
   
   
@@ -320,5 +324,6 @@ let file_info_test =
     file_last_seen = BasicSocket.last_time ();
     file_priority = 0;
    file_uids = [];
+    file_sub_files = [];
  } 
   
