@@ -42,10 +42,10 @@ open CommonOptions
 open CommonGlobals
   
 module P = GuiProto
-  
-let charset_to_gui s = s
+let lprintf_nl () =
+  lprintf "%s[dIface] "
+    (log_time ()); lprintf_nl2
 
-let charset_from_gui s = s
 
 let binary_gui_send gui t =
   match gui.gui_sock with
@@ -53,7 +53,7 @@ let binary_gui_send gui t =
       Fifo.put core_gui_fifo t
   | Some sock ->
       try
-        GuiEncoding.gui_send (GuiEncoding.to_gui gui.gui_proto_to_gui_version charset_to_gui ) 
+        GuiEncoding.gui_send (GuiEncoding.to_gui gui.gui_proto_to_gui_version) 
         sock t
       with UnsupportedGuiMessage -> 
 (* the message is probably not supported by this GUI *)
@@ -80,7 +80,7 @@ let binary_result_handler gui num r =
 let gift_result_handler gui num r =
   gui.gui_id_counter <- gui.gui_id_counter + 1;
   gui_send gui (P.Search_result (num, gui.gui_id_counter, Some 
-      (get_result r))) 
+      (IndexedResults.get_result r))) 
           
 let gui_can_write gui =
   match gui.gui_sock with
@@ -163,8 +163,16 @@ let update_room_info room =
       impl.impl_room_update <- !gui_counter
     end
   
+let update_network_info n =
+  let update = 0 in
+  if update < !gui_counter then begin
+      with_guis (fun gui -> 
+          update_events gui update n.network_num (gui.gui_events.gui_networks)
+      );
+    end
+  
 let update_result_info r =
-  let r = get_result r in
+  let r = IndexedResults.get_result r in
 (*  let update = r.result_update in
   let result_num = r.result_num in
   if update < !gui_counter then begin *)
@@ -226,6 +234,9 @@ let send_event gui ev =
   | Console_message_event msg ->
       gui_send gui (P.Console msg)
 
+  | Network_info_event n ->
+      gui_send gui (P.Network_info (network_info n))
+
   | _ ->  lprintf "Event not treated\n"
     
 let send_update_file gui file_num update =
@@ -246,6 +257,11 @@ let send_update_user gui user_num update =
   let user_info = P.User_info (user_info user) in
   gui_send gui user_info
   
+let send_update_network gui network_num update =
+  let network = network_find_by_num network_num in
+  let network_info = P.Network_info (network_info network) in
+  gui_send gui network_info
+  
 let send_update_client gui client_num update =
   let client = client_find client_num in
   let impl = as_client_impl client in
@@ -275,7 +291,7 @@ let send_update_room gui room_num update =
 
 let send_update_result gui result_num update =
   let r = find_result result_num in
-  let result_info = P.Result_info (get_result r) in
+  let result_info = P.Result_info (IndexedResults.get_result r) in
   gui_send gui result_info  
   
   let send_update_shared gui shfile_num update =
@@ -329,6 +345,7 @@ let connecting_writer gui _ =
         (gui.gui_events.gui_rooms, send_update_room);
         (gui.gui_events.gui_results, send_update_result);
         (gui.gui_events.gui_shared_files, send_update_shared);      
+        (gui.gui_events.gui_networks, send_update_network);      
       ]
 
                     
@@ -372,10 +389,6 @@ let gui_initialize gui =
       gui_send gui (Network_info (network_info n)));
   gui_send gui (Console (DriverControlers.text_of_html !!motd_html));
   
-  networks_iter_all (fun n ->
-      gui_send gui (Network_info (network_info n)));
-  gui_send gui (Console (DriverControlers.text_of_html !!motd_html));
-  
   
   if gui.gui_poll then begin
       
@@ -390,6 +403,7 @@ let gui_initialize gui =
       gui.gui_users <- create_events ();
       gui.gui_results <- create_events ();
       gui.gui_shared_files <- create_events ();
+      gui.gui_networks <- create_events ();
     
     
     end else begin
@@ -407,7 +421,7 @@ let gui_initialize gui =
                 gui.gui_events.gui_new_events <-
                   (File_add_source_event (file,c))
                 :: gui.gui_events.gui_new_events
-            ) (file_active_sources file)
+            ) sources
       ) !!files;
       
       List.iter (fun file ->
@@ -453,7 +467,7 @@ let gui_initialize gui =
 (*
 lprintf "options for net %s\n" r.network_name; 
 *)
-              let prefix = r.network_prefix () in
+              let prefix = r.network_shortname ^ "-" in
               let args = simple_options prefix opfile in
 (*
 lprintf "Sending for %s\n" prefix; 
@@ -473,9 +487,18 @@ lprintf "Sending for %s\n" prefix;
           ) (strings_of_section_options "" s)
       ) (sections downloads_ini);
 
+(* Options panels defined in users.ini *)
+      List.iter (fun s ->
+          let section = section_name s in
+          List.iter (fun o ->
+              gui_send gui (
+                P.Add_section_option (section, o))
+          ) (strings_of_section_options "" s)
+      ) (sections users_ini);
+
 (* Options panels defined in each plugin *)
       networks_iter_all (fun r ->
-          let prefix = r.network_prefix () in
+          let prefix = r.network_shortname ^ "-" in
           List.iter (fun file ->
               
               List.iter (fun s ->
@@ -509,8 +532,7 @@ let gui_reader (gui: gui_record) t _ =
           (to_gui_last_opcode + 1) version;
         gui.gui_proto_from_gui_version <- Array.create 
           (from_gui_last_opcode + 1) version;
-        lprintf "Using protocol %d for communications with the GUI\n" 
-          version    
+        lprintf_nl () "GUI protocol %d" version    
     
     | P.GuiExtensions list ->
         List.iter (fun (ext, bool) ->
@@ -552,6 +574,7 @@ let gui_reader (gui: gui_record) t _ =
           | _ ->
               guis := gui :: !guis;
               gui.gui_auth <- true;
+              gui.gui_conn.conn_user <- find_ui_user user;
               
               if not gui.gui_initialized then 
                 gui_initialize gui;
@@ -573,7 +596,7 @@ let gui_reader (gui: gui_record) t _ =
           | P.Command cmd ->
               let o = gui.gui_conn in
               let buf = o.conn_buf in
-              Buffer.clear buf;
+              Buffer.reset buf; 
               Buffer.add_string buf "\n----------------------------------\n";
               Printf.bprintf buf "Eval command: %s\n\n" cmd;
               DriverControlers.eval (ref true) cmd o;
@@ -639,7 +662,7 @@ let gui_reader (gui: gui_record) t _ =
                     network_extend_search r s e)
           
           | P.KillServer -> 
-              exit_properly 0
+              CommonInteractive.clean_exit 0
           
           | P.Search_query s ->
               
@@ -651,7 +674,7 @@ let gui_reader (gui: gui_record) t _ =
               
               let user = gui.gui_conn.conn_user in
               let query = 
-                try CommonGlobals.simplify_query
+                try CommonIndexing.simplify_query
                     (CommonSearch.mftp_query_of_query_entry 
                       s.GuiTypes.search_query)
                 with Not_found ->
@@ -692,6 +715,10 @@ search.op_search_end_reply_handlers;
               networks_iter network_connect_servers
           
           | P.Url url ->
+              (* changed with the new header-check code from dllink, but i
+                 didn't care about the read from file functionality that
+                 perhaps worked before:
+              
               if not (networks_iter_until_true (fun n -> network_parse_url n url)) then
                 begin
                   let file = File.to_string url in
@@ -700,6 +727,45 @@ search.op_search_end_reply_handlers;
                       ignore (networks_iter_until_true (fun n -> network_parse_url n line))
                   ) lines
                 end
+              *)
+              let query_networks url = 
+                if not (networks_iter_until_true
+                    (fun n ->
+                       try
+                         network_parse_url n url
+                       with e ->
+                         lprintf "Exception %s for network %s\n"
+                           (Printexc2.to_string e) (n.network_name);
+                         false
+                    )) then
+                   lprintf "Unable to match URL\n"
+                else
+                   lprintf "done\n"
+              in
+              if (String2.starts_with url "http") then (
+                let u = Url.of_string url in
+                let module H = Http_client in
+                let r = {
+                  H.basic_request with
+                    H.req_url =  u;
+                    H.req_proxy = !CommonOptions.http_proxy;
+                    H.req_request = H.HEAD;
+                    H.req_user_agent = 
+                       Printf.sprintf "MLDonkey/%s" Autoconf.current_version;
+                } in
+                H.whead r 
+                  (fun headers ->
+                    (* Combine the list of header fields into one string *)
+                    let concat_headers = 
+                      (List.fold_right (fun (n, c) t -> n ^ ": " ^ c ^ "\n" ^ t) headers "")
+                    in
+                    ignore (query_networks concat_headers)
+                  );
+                lprintf "Parsing HTTP url..."
+                )
+              else
+                query_networks url
+          
           | P.GetUploaders ->
               gui_send gui (P.Uploaders
                   (List2.tail_map (fun c -> client_num c) 
@@ -978,6 +1044,9 @@ search.op_search_end_reply_handlers;
                   update_shared_info s;
               ) 
 
+          | P.GetVersion ->
+              gui_send gui (P.Version Autoconf.current_version)
+
           | P.GiftAttach _ ->
               gui_send gui (P.GiftServerAttach ("mldonkey", "1.1"))
           | P.GiftStats ->
@@ -988,7 +1057,15 @@ search.op_search_end_reply_handlers;
                   ) :: !list
               );
               gui_send gui (P.GiftServerStats !list)
-              
+
+          (* introduced with protocol 32 *)
+          | P.ServerRename (num, name) ->
+              let s = server_find num in
+              server_rename s name
+          | P.ServerSetPreferred (num, preferred) ->
+              let s = server_find num in
+              server_set_preferred s preferred
+
   with 
     Failure s ->
       gui_send gui (Console (Printf.sprintf "Failure: %s\n" s))
@@ -1011,6 +1088,7 @@ let gui_events () =
       gui_users = create_events ();
       gui_results = create_events ();
       gui_shared_files = create_events ();
+      gui_networks = create_events ();
     
   }
   
@@ -1053,7 +1131,7 @@ let gui_handler t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
-      lprintf "CONNECTION FROM GUI\n"; 
+      lprintf_nl () "GUI connection from %s" (Ip.to_string from_ip);
       if Ip.matches from_ip !!allowed_ips then 
         
         let module P = GuiProto in
@@ -1070,8 +1148,7 @@ let gui_handler t event =
         TcpBufferedSocket.set_reader sock (GuiDecoding.gui_cut_messages
             (fun opcode s ->
               try
-                let m = GuiDecoding.from_gui gui.gui_proto_from_gui_version
-                charset_from_gui opcode s in
+                let m = GuiDecoding.from_gui gui.gui_proto_from_gui_version opcode s in
                 gui_reader gui m sock;
               with GuiDecoding.FromGuiMessageNotImplemented -> ()
           ));
@@ -1083,7 +1160,7 @@ let gui_handler t event =
         (* sort GUIs in increasing order of their num *)
         
       else begin
-          lprintf "Connection from that IP %s not allowed\n"
+          lprintf_nl () "GUI connection from %s rejected (see allowed_ips setting)"
             (Ip.to_string from_ip);
           Unix.close s
         end
@@ -1093,7 +1170,7 @@ let gift_handler t event =
   match event with
     TcpServerSocket.CONNECTION (s, Unix.ADDR_INET (from_ip, from_port)) ->
       let from_ip = Ip.of_inet_addr from_ip in
-      lprintf "CONNECTION FROM GUI\n"; 
+      lprintf "Gift: Connection from %s\n" (Ip.to_string from_ip);
       if Ip.matches from_ip !!allowed_ips then 
         
         let module P = GuiProto in
@@ -1121,7 +1198,7 @@ let gift_handler t event =
         (* sort GUIs in increasing order of their num *)
         
       else begin
-          lprintf "Connection from that IP %s not allowed\n"
+          lprintf "Connection from IP %s not allowed\n"
             (Ip.to_string from_ip);
           Unix.close s
         end
@@ -1185,6 +1262,9 @@ let rec update_events list =
           | User_info_event u ->
               update_user_info u
           
+          | Network_info_event n ->
+              update_network_info n;
+
           | Client_new_file_event (c,_,r) ->
               update_client_info c;
               update_result_info r;
@@ -1275,8 +1355,17 @@ let install_hooks () =
               with _ -> ())
       ) 
   ) downloads_ini;
+  iter_file (fun o ->
+      option_hook o (fun _ ->
+          with_guis (fun gui ->
+              try
+                let oo = strings_of_option o in
+                gui_send gui (P.Options_info [oo])
+              with _ -> ())
+      ) 
+  ) users_ini;
   networks_iter_all (fun r ->
-      let prefix = r.network_prefix()  in
+      let prefix = r.network_shortname ^ "-"  in
       List.iter (fun opfile ->
           iter_file (fun o ->
                 option_hook o (fun _ ->
