@@ -57,7 +57,8 @@ module DP500(M: sig
           val telnet_bind_addr : Ip.t Options.option_record
           val dp500_port : int Options.option_record
         end
-        
+
+      val incoming_directory : unit -> string        
       val files : unit -> file list
         
     end) =
@@ -278,6 +279,9 @@ let exec_command telnet line sock =
           
           in
           
+          let mlnet_incoming_string = "MLNET INCOMING" in
+          let mlnet_incoming_string_len = String.length mlnet_incoming_string in
+          
           let mlnet_temp_string = "MLNET TEMP" in
           let mlnet_temp_string_len = String.length mlnet_temp_string in
           
@@ -286,11 +290,23 @@ let exec_command telnet line sock =
             | "VIDEO" -> 
 
                 if argument = "" then begin
+                    if incoming_directory () <> "" then
+                      write_string sock (Printf.sprintf "%s|%s|1|\r\n"
+                          mlnet_incoming_string mlnet_incoming_string
+                      );
+                    
                     if !!temp_directory <> "" then
                       write_string sock (Printf.sprintf "%s|%s|1|\r\n"
                           mlnet_temp_string mlnet_temp_string 
                       );
                     end;
+                if incoming_directory () <> "" &&
+                  String2.starts_with argument mlnet_incoming_string then
+                    list_directory (incoming_directory ())
+                      (String.sub argument mlnet_incoming_string_len
+                        (String.length argument - mlnet_incoming_string_len))
+                  
+                else
                 if !!temp_directory <> "" &&
                   String2.starts_with argument mlnet_temp_string then
                     !!temp_directory,
@@ -358,7 +374,7 @@ let exec_command telnet line sock =
       let file = correct_file file in
       if !verbose_dp500 then lprintf "Size of [%s]\n" file;
       let fd = Unix32.create_ro file in
-      let size = Unix32.getsize64 fd in
+      let size = Unix32.getsize64 fd false in
       write_string sock (Printf.sprintf "%015Ld" size)
     else
     if command = "GET" then
@@ -379,7 +395,7 @@ let exec_command telnet line sock =
             end;
           let pclink_buf, len_read = 
             if len_read = 0 then
-              let size = Unix32.getsize64 fd in
+              let size = Unix32.getsize64 fd false in
               let size = Int64.to_int size in
               String.create size, size
             else
