@@ -35,8 +35,9 @@ let (!!) = Options.(!!)
 
 let string_color_of_state state =
   match state with
-  | Connected_downloading -> gettext M.downloading, Some !!O.color_downloading 
-  | Connected (-1) -> gettext M.connected, Some !!O.color_connected 
+  | Connected_downloading _ -> gettext M.downloading, Some !!O.color_downloading 
+  | Connected (-1)
+  | Connected (-2) -> gettext M.connected, Some !!O.color_connected 
   | Connecting  -> gettext M.connecting, Some !!O.color_connecting
   | NewHost -> "NEW HOST", None
   | Connected_initiating -> gettext M.initiating, Some !!O.color_not_connected
@@ -129,6 +130,7 @@ class box columns friend_tab =
       
     method column_menu  i = 
       [
+        `I ("Autosize", fun _ -> self#wlist#columns_autosize ());
         `I ("Sort", self#resort_column i);
         `I ("Remove Column",
           (fun _ -> 
@@ -188,10 +190,10 @@ class box columns friend_tab =
       match col with
         Col_client_name -> shorten !!O.max_client_name_len f.client_name
       | Col_client_state -> fst (string_color_of_client friend_tab f)
-      | Col_client_type -> (match f.client_type with
-              FriendClient -> gettext M.friend
-            | ContactClient -> gettext M.contact
-            | NormalClient -> "Y")
+      | Col_client_type -> (let t = f.client_type in
+            if t land client_friend_tag <> 0 then gettext M.friend else
+            if t land client_contact_tag <> 0 then gettext M.contact else
+              "Y")
       | Col_client_network -> Gui_global.network_name f.client_network
       | Col_client_kind -> 
           match f.client_kind with
@@ -379,6 +381,13 @@ class box_list (client_info_box : GPack.box) friend_tab =
                               try
                                 let c = Hashtbl.find G.locations num in
                                 if Mi.is_connected c.client_state then incr G.nclocations;
+                                (c.client_state <- 
+                                   match c.client_state with
+                                       Connected_downloading n ->
+                                         if n = file.file_num 
+                                           then Connected_downloading n
+                                           else Connected (-1)
+                                     | _ -> c.client_state);    
                                 l := c :: !l
                               with _ -> 
                                   Gui_com.send (GuiProto.GetClient_info num)
