@@ -314,19 +314,7 @@ let max_last_seen = 100 * 24 * 3600
 let time_to_string time =
   if time = max_last_seen 
     then "--"
-    else begin
-  let days = time / 60 / 60 / 24 in
-  let rest = time - days * 60 * 60 * 24 in
-  let hours = rest / 60 / 60 in
-  let rest = rest - hours * 60 * 60 in
-  let minutes = rest / 60 in
-  let seconds = rest - minutes * 60 in
-    if days > 0
-    then Printf.sprintf " %dd " days
-    else if hours > 0
-    then Printf.sprintf " %d:%02d:%02d " hours minutes seconds
-    else Printf.sprintf " %d:%02d " minutes seconds
-    end
+    else Date.time_to_string_long time
 
 let max_eta = 1000.0 *. 60.0 *. 60.0 *. 24.0
     
