@@ -35,7 +35,7 @@ module D = Gui_downloads
 (*module Gui_rooms = Gui_rooms2*)
 let chmod_config () =
   let base_config = 
-    (Filename.concat CommonOptions.config_dir "mldonkey_newgui.ini")
+    (Filename.concat CommonOptions.home_dir "mldonkey_newgui.ini")
   in
   let save_config =
     base_config^".old"
@@ -459,8 +459,8 @@ let value_reader gui t =
 
     | GiftServerAttach _
     | GiftServerStats _ -> assert false
-
-    | Search s -> ()
+    | Version _
+    | Search _ -> ()
         
   with e ->
       lprintf "Exception %s in reader\n" (Printexc2.to_string e)
