@@ -33,43 +33,57 @@ open BTTypes
 open CommonTypes
 
 let is_enabled = ref false
-  
+
 let disable enabler () =
   if !enabler then begin
       is_enabled := false;
       enabler := false;
       List.iter (fun file ->
-          Hashtbl2.safe_iter (fun c -> disconnect_client c Closed_by_user) 
+          Hashtbl2.safe_iter (fun c -> disconnect_client c Closed_by_user)
           file.file_clients) !current_files;
       (match !listen_sock with None -> ()
-        | Some sock -> 
+        | Some sock ->
             listen_sock := None;
             TcpServerSocket.close sock Closed_by_user);
       BTTracker.stop_tracker ();
       if !!enable_bittorrent then enable_bittorrent =:= false
     end
-    
+
 let enable () =
-  lprintf "enable\n";
+  (* lprintf "enable\n"; *)
   if not !is_enabled then
     let enabler = ref true in
-    lprintf "enabling...\n";
-    Unix2.safe_mkdir "torrents";
-    Unix2.safe_mkdir downloads_directory;
-    Unix2.safe_mkdir tracked_directory;  
+    (* lprintf "enabling...\n"; *)
+    Unix2.safe_mkdir torrents_directory;
     Unix2.safe_mkdir seeded_directory;
+    Unix2.safe_mkdir tracked_directory;
+    Unix2.safe_mkdir downloads_directory;
+    Unix2.safe_mkdir old_directory;
+    Unix2.can_write_to_directory torrents_directory;
+    Unix2.can_write_to_directory seeded_directory;
+    Unix2.can_write_to_directory tracked_directory;
+    Unix2.can_write_to_directory downloads_directory;
+    Unix2.can_write_to_directory old_directory;
     is_enabled := true;
-    if !!tracker_port > 0 then (
-        try BTTracker.start_tracker () 
+    if !!BTTracker.tracker_port = !!client_port then
+      begin
+        lprint_newline();
+	lprintf_nl () "BT-client_port and BT-tracker_port can not be the same.";
+	lprintf_nl () "Change one of the settings and restart MLDonkey, exiting...\n";
+	Pervasives.exit 69
+      end;
+    if !!BTTracker.tracker_port > 0 then (
+        try BTTracker.start_tracker ()
         with e ->
             lprintf "Exception in BTTracker.start_tracker: %s\n"
               (Printexc2.to_string e));
-    add_infinite_timer 1200. (fun _ ->
+    add_session_timer enabler 300. (fun _ ->
         BTInteractive.share_files ();
-        BTTracker.clean_tracker_timer ());
+    );
     add_timer 10. BTInteractive.share_files;
+    add_session_timer enabler 600. BTInteractive.retry_all_ft;
     network.op_network_disable <- disable enabler;
-    
+
     if not !!enable_bittorrent then enable_bittorrent =:= true;
 (*
   List.iter (fun s ->
@@ -86,22 +100,20 @@ let enable () =
         current_files := file :: !current_files
   ) files_by_key;
 *)
-    
 
-    lprintf "XXXXXXXXXXXXXXX\n";
-    BTClients.recover_files ();  
+
+    BTClients.recover_files ();
     add_session_timer enabler 60.0 (fun timer ->
         BTClients.recover_files ();
     );
-    
+
     add_session_timer enabler 120.0 (fun timer ->
         BTClients.send_pings ();
     );
-    
-  
+
   add_session_timer enabler 10.0 (fun timer ->
       BTClients.recompute_uploaders());
-  
+
   CommonGlobals.do_at_exit ( fun _ ->
     List.iter (fun file ->
 		 BTClients.file_stop file
@@ -110,7 +122,7 @@ let enable () =
 
   BTClients.listen ();
   ()
-  
+
 let _ =
   network.op_network_is_enabled <- (fun _ -> !!CommonOptions.enable_bittorrent);
   option_hook enable_bittorrent (fun _ ->
@@ -119,8 +131,8 @@ let _ =
       else network_disable network);
 (*
   network.op_network_save_simple_options <- BTComplexOptions.save_config;
-  network.op_network_load_simple_options <- 
-    (fun _ -> 
+  network.op_network_load_simple_options <-
+    (fun _ ->
       try
         Options.load bittorrent_ini;
       with Sys_error _ ->
@@ -129,21 +141,20 @@ let _ =
   *)
   network.op_network_enable <- enable;
   network.network_config_file <- [bittorrent_ini];
+  check_client_uid ();
   network.op_network_info <- (fun n ->
-      { 
+      {
         network_netnum = network.network_num;
         network_config_filename = (match network.network_config_file with
             [] -> "" | opfile :: _ -> options_file_name opfile);
         network_netflags = network.network_flags;
         network_netname = network.network_name;
         network_enabled = network.op_network_is_enabled ();
-        network_uploaded = Int64.zero;
-        network_downloaded = Int64.zero;
+        network_uploaded = !bt_upload_counter;
+        network_downloaded = !bt_download_counter;
         network_connected = 0;
       });
-  CommonInteractive.register_gui_options_panel "BitTorrent" 
+  CommonInteractive.register_gui_options_panel "BitTorrent"
   gui_bittorrent_options_panel
-  
-  
+
 let main (toto: int) = ()
-  
\ No newline at end of file
