@@ -248,13 +248,13 @@ let _ =
         P.user_tags = (
           let list = if user.user_data > 1. then 
               [ 
-                { tag_name =  "link"; tag_value = String user.user_link };
-                { tag_name =  "shared"; tag_value = String (
+                { tag_name =  Field_UNKNOWN "link"; tag_value = String user.user_link };
+                { tag_name =  Field_UNKNOWN "shared"; tag_value = String (
                     Printf.sprintf "%12.0f" user.user_data) }
               ]          else []
           in
           if user.user_admin then
-            { tag_name =  "admin"; tag_value = String "admin" } :: list
+            { tag_name =  Field_UNKNOWN "admin"; tag_value = String "admin" } :: list
           else list
         );
             
@@ -315,8 +315,16 @@ let _ =
         P.file_last_seen = BasicSocket.last_time ();
         P.file_priority = file_priority (as_file file.file_file);
         P.file_uids = [];
+        P.file_sub_files = [];
       }    
   );
+  (*
+  file_ops.op_file_files <- (fun file impl -> 
+      match file.file_swarmer with
+        None -> [CommonFile.as_file impl]
+      | Some swarmer ->
+          Int64Swarmer.subfiles swarmer)
+; *)
   (*
   file_ops.op_file_save_as <- (fun file new_name  ->
       match file_state file with
@@ -361,6 +369,7 @@ let _ =
         P.client_chat_port = 0 ;
         P.client_connect_time = last_time ();
         P.client_software = "";
+        P.client_release = "";
         P.client_emulemod = "";
         P.client_downloaded = zero;
         P.client_uploaded = zero;
@@ -379,6 +388,8 @@ let _ =
 
 let _ =
   network.op_network_save_complex_options <- (fun _ -> ());
+  network.op_network_save_sources <- (fun _ -> ());
+  network.op_network_update_options <- (fun _ -> ());
   network.op_network_add_server <- (fun ip port ->
       as_server (new_server ip port).server_server
   )
