@@ -31,22 +31,19 @@ open CommonFile
 open CommonTypes
 open CommonOptions
 open CommonGlobals
+open CommonResult
   
 open DonkeyTypes
 open DonkeyOptions
 open DonkeyGlobals
 
-let shared_files_ini = create_options_file (
-    Filename.concat file_basedir "shared_files_new.ini")
+let shared_files_ini = create_options_file "shared_files_new.ini"
 
-let file_sources_ini = create_options_file (
-    Filename.concat file_basedir "file_sources.ini")
+let file_sources_ini = create_options_file "file_sources.ini"
   
-let stats_ini = create_options_file (
-    Filename.concat file_basedir "stats.ini")
+let stats_ini = create_options_file "stats.ini"
 
-let mod_stats_ini = create_options_file (
-    Filename.concat file_basedir "stats_mod.ini")
+let mod_stats_ini = create_options_file "stats_mod.ini"
 
 let shared_section = file_section shared_files_ini [] ""
 let stats_section = file_section stats_ini [] ""
@@ -64,34 +61,38 @@ I know this is stupid, but "give the people what they want"..
 
 *)
 
+let lprintf_nl () =
+  lprintf "%s[EDK] "
+  (log_time ()); lprintf_nl2
+
 let create_online_sig () =
-
+  
   let most_users = ref Int64.zero in
-	let server_name= ref "" in
-	let server_ip = ref "" in
-	let server_port = ref 0 in
-	List.iter (fun s -> 
-		if s.server_nusers > !most_users then begin 
-			server_name := s.server_name;
-			server_ip := (Ip.to_string s.server_ip);
-			server_port := s.server_port;
-			most_users := s.server_nusers;
-		end
-	) (connected_servers());
-
-	let oc = open_out (Filename.concat file_basedir "onlinesig.dat") in
-
+  let server_name= ref "" in
+  let server_ip = ref "" in
+  let server_port = ref 0 in
+  List.iter (fun s -> 
+      if s.server_nusers > !most_users then begin 
+          server_name := s.server_name;
+          server_ip := (Ip.to_string s.server_ip);
+          server_port := s.server_port;
+          most_users := s.server_nusers;
+        end
+  ) (connected_servers());
+  
+  let oc = open_out "onlinesig.dat" in
+  
   if !most_users = Int64.zero then
-		output_string oc ("0\n")
-	else 
-		output_string oc (Printf.sprintf "1|%s|%s|%d\n" !server_name !server_ip !server_port);
-	let dlkbs = (( (float_of_int !udp_download_rate) +. (float_of_int !control_download_rate)) /. 1024.0) in
-    let ulkbs = (( (float_of_int !udp_upload_rate) +. (float_of_int !control_upload_rate)) /. 1024.0) in
-	
-	output_string oc (Printf.sprintf "%.1f|%.1f|%d\n" dlkbs ulkbs 
-			(Intmap.length !CommonUploads.pending_slots_map));
-	close_out oc
-    
+    output_string oc ("0\n")
+  else 
+    output_string oc (Printf.sprintf "1|%s|%s|%d\n" !server_name !server_ip !server_port);
+  let dlkbs = (( (float_of_int !udp_download_rate) +. (float_of_int !control_download_rate)) /. 1024.0) in
+  let ulkbs = (( (float_of_int !udp_upload_rate) +. (float_of_int !control_upload_rate)) /. 1024.0) in
+  
+  output_string oc (Printf.sprintf "%.1f|%.1f|%d\n" dlkbs ulkbs 
+      (Intmap.length !CommonUploads.pending_slots_map));
+  close_out oc
+  
 (************ COMPLEX OPTIONS *****************)
   
 let value_to_addr v =
@@ -270,15 +271,50 @@ let value_to_file file_size file_state assocs =
     try conv (List.assoc name assocs) with _ -> []
   in
   
-  let file_md4_name = 
+  let file_md4 = 
     try
       get_value "file_md4" value_to_string
     with _ -> failwith "Bad file_md4"
   in
+  let file_diskname =
+    let filename =
+      try
+        get_value "file_diskname" value_to_string
+      with _ ->
+        let filename = Filename.concat !!temp_directory file_md4 in
+          lprintf_nl () "getting file_diskname from ini failed, testing for ed2k-temp-file %s" filename;
+        if Sys.file_exists filename then
+          filename
+        else
+          begin
+            let filename =
+              Filename.concat
+                !!temp_directory
+                ( string_of_uid ( Ed2k (Md4.of_string file_md4) ) )
+            in
+            lprintf_nl () "geting file_diskname from ini failed, testing for ed2k-temp-file %s"
+              filename;
+            if Sys.file_exists filename then
+              filename
+            else
+              Filename.concat
+                !!temp_directory
+                ( file_string_of_uid ( Ed2k (Md4.of_string file_md4) ) )
+          end
+    in
+    if not (Sys.file_exists filename) then
+      (* I think we should die here, to prevent any corruption. *)
+      lprintf_nl () "ERROR ED2K-TEMP-FILE %s DOES NOT EXIST, THIS WILL PERHAPS LEAD TO CORRUPTION IN THAT DOWNLOAD!"
+        filename;
+    if !verbose then lprintf_nl () "ed2k-temp-file %s used." filename;
+    filename
+  in
+
+  let filenames = List.map (fun name -> name, GuiTypes.noips()) 
+    (get_value_nil "file_filenames" (value_to_list value_to_string)) in
   
-  let file = DonkeyGlobals.new_file file_state (
-      Filename.concat !!temp_directory file_md4_name)
-    (Md4.of_string file_md4_name) file_size true in
+  let file = DonkeyGlobals.new_file file_diskname file_state
+    (Md4.of_string file_md4) file_size filenames true in
 
   (*
   (try 
@@ -291,17 +327,12 @@ let value_to_file file_size file_state assocs =
     with _ -> ()                );
 *)
   
-  file.file_filenames <-
-    List.map (fun name -> name, GuiTypes.noips()) 
-  (get_value_nil "file_filenames" (value_to_list value_to_string));
   
-  (*
   
   (try
-      set_file_best_name (as_file file.file_file)
+      set_file_best_name (as_file file)
       (get_value "file_filename" value_to_string)
     with _ -> update_best_name file);
-*)
   
 (*
   (try
@@ -319,7 +350,8 @@ let value_to_file file_size file_state assocs =
     with _ -> 
         lprintf "Could not load chunks states\n"; );
 *)
-  let md4s = get_value "file_md4s" (value_to_array value_to_md4) in
+  let md4s = try get_value "file_md4s" (value_to_array value_to_md4) 
+    with _ -> [||] in
   
   if md4s <> [||] then file.file_computed_md4s <- md4s;
   
@@ -377,6 +409,7 @@ let file_to_value file =
   let fields =
     [
       "file_md4", string_to_value (Md4.to_string file.file_md4);
+      "file_diskname", string_to_value file.file_diskname;
 (*      "file_all_chunks", string_to_value (string_of_chunks file);   *)
 (*      "file_absent_chunks", List
         (List.map (fun (i1,i2) -> 
@@ -485,22 +518,22 @@ module ModStatsOption = struct
             try conv (List.assoc name assocs) with _ -> []
           in
           { 
-            brand_mod_seen = value_to_int (List.assoc "mseen" assocs);
-            brand_mod_banned = value_to_int (List.assoc "mbanned" assocs);
-            brand_mod_filerequest = value_to_int (List.assoc "mfilereqs" assocs);
-            brand_mod_download = value_to_int64 (List.assoc "mdownload" assocs);
-            brand_mod_upload = value_to_int64 (List.assoc "mupload" assocs);
+            brand_seen = value_to_int (List.assoc "mseen" assocs);
+            brand_banned = value_to_int (List.assoc "mbanned" assocs);
+            brand_filerequest = value_to_int (List.assoc "mfilereqs" assocs);
+            brand_download = value_to_int64 (List.assoc "mdownload" assocs);
+            brand_upload = value_to_int64 (List.assoc "mupload" assocs);
           }
           
       | _ -> failwith "Options: not a mod_stat option"
           
     let stat_to_mod_value b =
       Options.Module [
-        "mseen", int_to_value b.brand_mod_seen;
-        "mbanned", int_to_value b.brand_mod_banned;
-        "mfilereqs", int_to_value b.brand_mod_filerequest;
-        "mdownload", int64_to_value b.brand_mod_download;
-        "mupload", int64_to_value b.brand_mod_upload;
+        "mseen", int_to_value b.brand_seen;
+        "mbanned", int_to_value b.brand_banned;
+        "mfilereqs", int_to_value b.brand_filerequest;
+        "mdownload", int64_to_value b.brand_download;
+        "mupload", int64_to_value b.brand_upload;
       ]
     
     
@@ -538,10 +571,6 @@ module UidOption = struct
     
   end
 
-let old_files = 
-  define_option donkey_section ["old_files"] 
-  "The files that were downloaded" (list_option Md4.option) []
-
 let brotherhood = 
   define_option donkey_section ["brotherhood"] 
     "The links between files being downloaded" (list_option 
@@ -577,15 +606,13 @@ let force_add_server ip port =
       s        
       
 let check_add_server ip port =
-  if !!update_server_list &&
-    Ip.valid ip && Ip.reachable ip &&
+  if Ip.usable ip &&
     not (is_black_address ip port) && port <> 4662 then
     force_add_server ip port
   else raise Not_found
 
 let safe_add_server ip port =
-  if !!update_server_list &&
-    Ip.valid ip && Ip.reachable ip &&
+  if Ip.usable ip &&
     not (is_black_address ip port) && port <> 4662 then
     try
       ignore (DonkeyGlobals.find_server ip port)
@@ -603,7 +630,7 @@ let remove_server ip port =
 let config_files_loaded = ref false  
       
 let load _ =
-  lprintf "LOADING SHARED FILES AND SOURCES\n"; 
+  if !verbose then lprintf_nl () "Loading shared files";
   (try
       Options.load shared_files_ini;
       Options.load stats_ini;
@@ -621,37 +648,37 @@ let new_stats_array () =
   
 let new_mod_stats_array () = 
   Array.init brand_mod_count (fun _ ->
-      { dummy_mod_stats with brand_mod_seen = 0 }
+      { dummy_stats with brand_seen = 0 }
   )
 
-let gstats_by_brand = define_option stats_section ["stats"] "" 
+let gstats_array = define_option stats_section ["stats"] "" 
     (array_option StatsOption.t) (new_stats_array ())
 
-let gstats_by_brand_mod = define_option mod_stats_section ["stats"] "" 
+let gstats_mod_array = define_option mod_stats_section ["stats"] "" 
     (array_option ModStatsOption.t) (new_mod_stats_array ())
 
 let _ =
-  option_hook gstats_by_brand (fun _ ->
-      let old_stats = !!gstats_by_brand in
+  option_hook gstats_array (fun _ ->
+      let old_stats = !!gstats_array in
       let old_len = Array.length old_stats in
       if old_len <> brand_count then
         let t = new_stats_array () in
         for i = 0 to old_len - 1 do
           t.(i) <- old_stats.(i)
         done;
-        gstats_by_brand =:= t
+        gstats_array =:= t
   )
   
 let _ =
-  option_hook gstats_by_brand_mod (fun _ ->
-      let old_mod_stats = !!gstats_by_brand_mod in
+  option_hook gstats_mod_array (fun _ ->
+      let old_mod_stats = !!gstats_mod_array in
       let old_mod_len = Array.length old_mod_stats in
       if old_mod_len <> brand_mod_count then
         let t = new_mod_stats_array () in
         for i = 0 to old_mod_len - 1 do
           t.(i) <- old_mod_stats.(i)
         done;
-        gstats_by_brand_mod =:= t
+        gstats_mod_array =:= t
   )
 
 let diff_time = ref 0
@@ -660,22 +687,22 @@ let sources_loaded = ref false  (* added 2.5.24 *)
 
 let save _ =
   if !config_files_loaded then begin
-(*  lprintf "SAVING SHARED FILES AND SOURCES\n"; *)
       Options.save_with_help shared_files_ini;
       guptime =:= !!guptime + (last_time () - start_time) - !diff_time;
       diff_time := (last_time () - start_time);
       Options.save_with_help stats_ini;
       Options.save_with_help mod_stats_ini;
-      create_online_sig ();
-    end;
+      create_online_sig ()
+    end
+
+let save_sources _ =
   if !sources_loaded then begin
       save_time =:= last_time ();
       let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
       Options.save_with_help file_sources_ini;
       cleaner ()
     end
-(*  lprintf "SAVED\n";  *)
-    
+
 let guptime () = !!guptime - !diff_time
   
 let load_sources () = 
@@ -702,6 +729,45 @@ let load_sources () =
       ) !!brotherhood
     with _ -> ())
 
+let check_result r tags =
+  if r.result_names = [] || r.result_size = Int64.zero then begin
+      if !verbose then begin
+          lprintf_n () "BAD RESULT:";
+          List.iter (fun tag ->
+              lprintf "[%s] = [%s]" (string_of_field tag.tag_name)
+                (string_of_tag_value tag.tag_value);
+              lprint_newline ();
+          ) tags;
+        end;
+      false
+    end
+  else true
+
+(* Inserted in complexOptions to be able to access old_files (ugly) *)
+let result_of_file md4 tags =
+  let rec r = {  dummy_result with
+      result_uids = [Uid.create (Ed2k md4)];
+      result_done = (List.mem md4 !!old_files) || (Hashtbl.mem files_by_md4 md4); 
+    } in
+  List.iter (fun tag ->
+      match tag with
+        { tag_name = Field_Filename; tag_value = String s } ->
+          r.result_names <- s :: r.result_names
+      | { tag_name = Field_Size; tag_value = Uint64 v } ->
+          r.result_size <- v;
+      | { tag_name = Field_Format; tag_value = String s } ->
+          r.result_tags <- tag :: r.result_tags;
+          r.result_format <- s
+      | { tag_name = Field_Type; tag_value = String s } ->
+          r.result_tags <- tag :: r.result_tags;
+          r.result_type <- s
+      | _ ->
+          r.result_tags <- tag :: r.result_tags
+  ) tags;
+  if check_result r tags then
+    let rs = update_result_num r in
+    Some rs
+  else None
     
   
 let _ =
@@ -717,6 +783,7 @@ let _ =
   
   network.op_network_load_complex_options <- load;
   network.op_network_save_complex_options <- save;
+  network.op_network_save_sources <- save_sources;
   
   let cleaner = DonkeySources.attach_sources_to_file file_sources_section in
   cleaner ()
