@@ -17,6 +17,7 @@
     Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
 
+open Int64ops
 open Printf2
 open Md4
 
@@ -29,6 +30,7 @@ open CommonFile
 open CommonUser
 open CommonSearch
 open CommonTypes
+open CommonInteractive
 open Options
 open BasicSocket
 open TcpBufferedSocket
@@ -46,7 +48,7 @@ open DonkeyStats
 
 let verbose_upload = false
       
-let msg_block_size_int = 10000
+let msg_block_size_int = 10240
 let msg_block_size = Int64.of_int msg_block_size_int
 let upload_buffer = String.create msg_block_size_int
 let max_msg_size = 15000
@@ -77,7 +79,6 @@ module NewUpload = struct
     let rec send_small_block c sock file begin_pos len_int = 
 (*      lprintf "send_small_block %d\n" len_int; *)
 (*      let len_int = Int32.to_int len in *)
-      CommonUploads.consume_bandwidth len_int;
       try
         if !verbose then begin
             lprintf "send_small_block(%s-%s) %Ld %d\n"
@@ -92,7 +93,7 @@ module NewUpload = struct
             M.BlocReq {  
               B.md4 = file.file_md4;
               B.start_pos = begin_pos;
-              B.end_pos = Int64.add begin_pos (Int64.of_int len_int);
+              B.end_pos = begin_pos ++ (Int64.of_int len_int);
               B.bloc_str = "";
               B.bloc_begin = 0;
               B.bloc_len = 0; 
@@ -105,12 +106,13 @@ module NewUpload = struct
         DonkeyProtoCom.new_string msg upload_buffer;
         Unix32.read (file_fd file) begin_pos upload_buffer slen len_int;
         let uploaded = Int64.of_int len_int in
-        count_upload c file uploaded;
+        count_upload c uploaded;
+	CommonUploads.consume_bandwidth len_int;
         (match file.file_shared with None -> ()
           | Some impl ->
               shared_must_update_downloaded (as_shared impl);
               impl.impl_shared_uploaded <- 
-                Int64.add impl.impl_shared_uploaded uploaded);
+                impl.impl_shared_uploaded ++ uploaded);
         if c.client_connected then
           printf_string "U[OUT]"
         else
@@ -119,18 +121,18 @@ module NewUpload = struct
         write_string sock upload_buffer;
         check_end_upload c sock
       with e -> 
-          lprintf "Exception %s in send_small_block\n" (Printexc2.to_string e)
+          if !verbose_hidden_errors then lprintf "Exception %s in send_small_block\n" (Printexc2.to_string e)
     
     let rec send_client_block c sock per_client =
 (*      lprintf "send_client_block\n"; *)
-      if per_client > 0 && CommonUploads.remaining_bandwidth () > 0 then
+      if per_client > 0 && CommonUploads.can_write_len sock max_msg_size then
         match c.client_upload with
         | Some ({ up_chunks = _ :: chunks } as up)  ->
             if up.up_file.file_shared = None then begin
 (* Is there a message to warn that a file is not shared anymore ? *)
                 c.client_upload <- None;
               end else
-            let max_len = Int64.sub up.up_end_chunk up.up_pos in
+            let max_len = up.up_end_chunk -- up.up_pos in
             let max_len = Int64.to_int max_len in
             let msg_block_size_int = mini msg_block_size_int per_client in
             if max_len <= msg_block_size_int then
@@ -158,11 +160,9 @@ module NewUpload = struct
               begin
                 send_small_block c sock up.up_file up.up_pos 
                   msg_block_size_int;
-                up.up_pos <- Int64.add up.up_pos 
-                  (Int64.of_int msg_block_size_int);
+                up.up_pos <- up.up_pos ++ (Int64.of_int msg_block_size_int);
                 let per_client = per_client-msg_block_size_int in
-                if can_write_len sock max_msg_size then
-                  send_client_block c sock per_client
+                send_client_block c sock per_client
               end
         | _ -> ()
     
@@ -171,14 +171,9 @@ module NewUpload = struct
       do_if_connected  c.client_source.DonkeySources.source_sock (fun sock ->
 (*    lprintf "upload_to_client %d connected\n"  (maxi max_msg_size size); *)
           
-(* changed in 2.5.27: mini instead of maxi *)
           let size = mini max_msg_size size in
-          if CommonUploads.can_write_len sock size then 
-            begin
-(*              lprintf "can_write_len %d\n"  (maxi max_msg_size size); *)
-              send_client_block c sock size;
-            end;
-          (match c.client_upload with
+          send_client_block c sock size;
+           (match c.client_upload with
               None -> ()
             | Some up ->
                 if !CommonUploads.has_upload = 0 then
