@@ -56,6 +56,10 @@ open DonkeyStats
    Once the block has been finished allow changing order.
 *)
 
+let lprintf_nl () =
+  lprintf "%s[EDK] "
+    (log_time ()); lprintf_nl2
+
 let sort_file_queue c =
   match c.client_download with
     Some _ -> ()
@@ -64,21 +68,21 @@ let sort_file_queue c =
         [] -> ()
       | [ (file, chunks, up) ] ->
           if !verbose_download || c.client_debug then begin
-              lprintf "sort_file_queue: single file. client(%d): %s, file(%d): %s\n" (client_num c) c.client_name (file_num file) (file_best_name file);
+              lprintf_nl () "sort_file_queue: single file. client(%d): %s, file(%d): %s" (client_num c) c.client_name (file_num file) (file_best_name file);
             end
       | (file, chunks, up) :: _ ->
           let fn = file_num file in
           if !verbose_download || c.client_debug then begin
-              lprintf "sort_file_queue: multiple files. client(%d): %s, file(%d): %s\n" (client_num c) c.client_name (file_num file) (file_best_name file);
+              lprintf_nl () "sort_file_queue: multiple files. client(%d): %s, file(%d): %s" (client_num c) c.client_name (file_num file) (file_best_name file);
             end;
           c.client_file_queue <- List.stable_sort (fun (f1, _, _) (f2, _, _) ->
               let v = file_priority f2 - file_priority f1 in
               if v <> 0 then v else
               let s1 = if (file_size f1) > Int64.zero then
-                  Int64.to_int (Int64.div (Int64.mul (file_downloaded f1) (Int64.of_int 100)) (file_size f1))
+                  Int64.to_int ((file_downloaded f1) ** 100L // (file_size f1))
                 else 0 in
               let s2 = if (file_size f2) > Int64.zero then
-                  Int64.to_int (Int64.div (Int64.mul (file_downloaded f2) (Int64.of_int 100)) (file_size f2))
+                  Int64.to_int ((file_downloaded f2) ** 100L // (file_size f2))
                 else 0 in
               s2 - s1
           ) c.client_file_queue;
@@ -87,7 +91,7 @@ let sort_file_queue c =
           | (file, chunks, _) :: _ ->
               if (file_num file) <> fn then begin
                   if !verbose_download || c.client_debug then begin
-                      lprintf "sort_file_queue: queue change. client(%d): %s, file(%d): %s\n" (client_num c) c.client_name (file_num file) (file_best_name file);
+                      lprintf_nl () "sort_file_queue: queue change. client(%d): %s, file(%d): %s" (client_num c) c.client_name (file_num file) (file_best_name file);
                     end;
 (*
 (*                  c.client_chunks <- chunks; *)
@@ -112,6 +116,8 @@ let unshare_file file =
   match file.file_shared with
     None -> ()
   | Some s -> 
+      if !verbose_share then
+        lprintf_nl () "unshare_file %s" file.file_diskname;
       file.file_shared <- None;
       decr nshared_files;
       CommonShared.shared_calculate_total_bytes ();
@@ -129,11 +135,14 @@ let unshare_file file =
 let declare_completed_file file = 
   DonkeyShare.remember_shared_info file (file_disk_name file);
   file_completed (as_file file);
+  Int64Swarmer.remove_swarmer file.file_swarmer;
+  file.file_swarmer <- None;
+  
   
   unshare_file file;
-  (*ignore (CommonShared.new_shared "completed" 0 (
+  ignore (CommonShared.new_shared "completed" 0 (
       file_best_name file )
-    (file_disk_name file));*)
+    (file_disk_name file));
   (try
       let format = CommonMultimedia.get_info
           (file_disk_name file) in
@@ -171,14 +180,14 @@ let check_file_downloaded file =
           
           let downloaded = Int64Swarmer.downloaded swarmer in
           if file_downloaded file <> downloaded then begin
-              lprintf "ERROR: file_downloaded file (%Ld) <> downloaded swarmer (%Ld)\n"
+              lprintf_nl () "ERROR: file_downloaded file (%Ld) <> downloaded swarmer (%Ld)"
                 (file_downloaded file) downloaded
             end;
           
           if verified then begin
               if (file_size file <> downloaded)
               then
-                lprintf "DonkeyOneFile: Downloaded size differs after complete verification\n";
+                lprintf_nl () "Downloaded size differs after complete verification";
               download_finished file
             end
             
@@ -212,13 +221,13 @@ let add_client_chunks c file client_chunks =
             if f != file then iter tail
             else begin
                 Int64Swarmer.update_uploader up
-                (AvailableBoolBitmap client_chunks);
-                Array.blit client_chunks 0 chunks 0 (Array.length chunks)
+                (AvailableBitv client_chunks);
+                Bitv.blit client_chunks 0 chunks 0 (Bitv.length chunks)
               end
             
         | [] ->
             let up = Int64Swarmer.register_uploader swarmer (as_client c) 
-              (AvailableBoolBitmap client_chunks) in
+              (AvailableBitv client_chunks) in
             c.client_file_queue <-  c.client_file_queue @
               [file, client_chunks, up]
       in
@@ -388,7 +397,7 @@ let rec get_from_client c =
 let request_slot c = 
     if c.client_slot = SlotNotAsked then begin
       if !verbose_download then begin
-          lprintf "start_download...\n";
+          lprintf_nl () "start_download";
         end;
       do_if_connected c.client_source.DonkeySources.source_sock (fun sock ->
           sort_file_queue c;
@@ -407,11 +416,13 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
     
   match c.client_download with
     None -> 
-      lprintf "DonkeyOneFile.block_received: block received but no file !\n  Received: %s\n" (Md4.to_string md4)
+      if !verbose_hidden_errors then
+        lprintf_nl ()"block_received: block received but no file !, received: %s" (Md4.to_string md4)
   | Some (file, up) ->
       
       if file.file_md4 <> md4 then begin
-          lprintf "DonkeyOneFile.block_received: block for bad file\n  Received: %s\n  Expected: %s\n" (Md4.to_string md4) (Md4.to_string file.file_md4)
+          if !verbose_hidden_errors then
+            lprintf_nl () "block_received: block for bad file, received: %s, expected: %s" (Md4.to_string md4) (Md4.to_string file.file_md4)
         end else begin
           DonkeySources.set_request_result c.client_source file.file_sources File_upload;
           
@@ -419,7 +430,6 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
           
           set_client_state c (Connected_downloading (file_num file));
           let len64 = Int64.of_int bloc_len in
-          count_download c file len64;
 
 (* TODO: verify the received data has been requested *)
           
@@ -432,7 +442,7 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
             with
             | e ->
                 let m =
-(*		      Printf.sprintf "File %s begin_pos=%s bloc_begin=%d bloc_len=%d:\nError %s while writing block%s\n" (file_best_name file) (Int64.to_string begin_pos) t.Q.bloc_begin t.Q.bloc_len (Printexc2.to_string e)  *)
+(*          Printf.sprintf "File %s begin_pos=%s bloc_begin=%d bloc_len=%d:\nError %s while writing block%s\n" (file_best_name file) (Int64.to_string begin_pos) t.Q.bloc_begin t.Q.bloc_len (Printexc2.to_string e)  *)
                   (match e with 
                       Unix.Unix_error (Unix.ENOSPC, _, _) -> " (Disk full?)"
                     | _ -> "") in
@@ -451,8 +461,7 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
 (*            List.iter Int64Swarmer.alloc_range c.client_ranges; *)
           let new_downloaded = 
             Int64Swarmer.downloaded swarmer in
-          c.client_downloaded <- c.client_downloaded ++ (
-            new_downloaded -- old_downloaded);
+          count_download c (new_downloaded -- old_downloaded);
 (*
           if not (List.mem c.client_ip bb.block_contributors) then
             bb.block_contributors <- c.client_ip :: 
@@ -462,7 +471,7 @@ let block_received c md4 begin_pos bloc bloc_pos bloc_len =
           
           if new_downloaded -- old_downloaded < len64 then
               if !verbose_download then begin
-                lprintf "ALREADY RECEIVED: %Ld < %Ld\n"
+                lprintf_nl () "ALREADY RECEIVED: %Ld < %Ld"
               (new_downloaded -- old_downloaded) len64;          
               end;
           get_from_client c
@@ -475,10 +484,10 @@ let search_found filter search md4 tags =
   let new_tags = ref [] in
   List.iter (fun tag ->
       match tag with
-        { tag_name = "filename"; tag_value = String s } -> file_name := s
-      | { tag_name = "size"; tag_value = Uint64 v } -> file_size := v
-      | { tag_name = "availability";
-	  tag_value = (Uint64 v| Fint64 v) } ->
+        { tag_name = Field_Filename; tag_value = String s } -> file_name := s
+      | { tag_name = Field_Size; tag_value = Uint64 v } -> file_size := v
+      | { tag_name = Field_Availability;
+    tag_value = (Uint64 v| Fint64 v) } ->
           availability := Int64.to_int v;  new_tags := tag :: !new_tags
       | _ -> new_tags := tag :: !new_tags
   ) tags;
