@@ -21,29 +21,30 @@ open Queues
 open Md4
 open CommonDownloads
 open CommonTypes
+open CommonStats
   (*
-  
+
 module MyList = struct
-    type 'a mylist = 
+    type 'a mylist =
       Nil
     | Cons of int * 'a * 'a mylist
 
-    type ('a,'b) assoc = 
-      { 
+    type ('a,'b) assoc =
+      {
         v1 : 'a;
         mutable v2 : 'b;
       }
-      
+
     let nil = Nil
-    let length list = 
+    let length list =
       match list with Nil -> 0 | Cons (len,_,_) -> len
-      
+
     let add v list = Cons (1+ length list, v, list)
     let head list =
       match list with
         Nil -> raise Not_found
       | Cons (_,v,_) -> v
-          
+
     let tail list =
       match list with
         Nil -> raise Not_found
@@ -64,7 +65,7 @@ module MyList = struct
     let assoc x y = { v1 = x; v2 = y; }
     let assoc_get x = x.v2
     let assoc_set x y = x.v2 <- y
-      
+
     let rec find_assoc v list =
       match list with
         Nil -> raise Not_found
@@ -76,7 +77,7 @@ module MyList = struct
         Nil -> raise Not_found
       | Cons (_,vv,tail) ->
           if vv.v1 == v then vv else find_assq v tail
-          
+
     let rec mem_assoc v list =
       match list with
         Nil -> raise Not_found
@@ -88,7 +89,7 @@ module MyList = struct
         Nil -> raise Not_found
       | Cons (_,vv,tail) ->
           vv.v1 == v || mem_assq v tail
-          
+
   end
 
 open MyList
@@ -101,7 +102,7 @@ type client_tag_name =
 | CT_CLIENT_VERSION
 | CT_CLIENT_EXTENDED
 | CT_CLIENT_UDPPORT
-  
+
 type server_tag_name =
 | ST_SERVER_NAME
 | ST_SERVER_DESCRIPTION
@@ -110,7 +111,7 @@ type server_tag_name =
 | ST_SERVER_PING
 | ST_SERVER_PROF
 | ST_SERVER_HISTORY
-  
+
 type file_tag_name =
 | FT_FILE_FILENAME
 | FT_FILE_SIZE
@@ -125,11 +126,12 @@ type file_tag_name =
 | FT_FILE_DISKNAME
 | FT_FILE_PRIORITY
 | FT_FILE_STATUS
-    *)  
+    *)
 
 type emule_proto = {
     mutable emule_comments : int;
     mutable emule_version : int;
+    mutable emule_release : string;
     mutable emule_secident : int;
     mutable emule_noviewshared : int;
     mutable emule_supportpreview : int;
@@ -156,7 +158,7 @@ type emule_tag_name =
 | ET_MOD_TAROD
 | ET_TAROD_VERSION
 | ET_MOD_PLUS
-(*  
+(*
 type pref_tag_name =
 | PT_PREF_NAME
 | PT_PREF_PORT
@@ -167,9 +169,9 @@ type request_record = {
   mutable last_request : int;
   mutable nwarnings : int;
 }
-  
+
 type client_score =
-  Client_not_connected 
+  Client_not_connected
 | Client_has_file
 | Client_has_priority_file
 | Client_has_chunk
@@ -182,7 +184,7 @@ type reliability =
 | Reliability_reliable
 | Reliability_suspicious of int
 
-type brand = 
+type brand =
   Brand_unknown
 | Brand_edonkey
 | Brand_cdonkey
@@ -196,9 +198,29 @@ type brand =
 | Brand_server
 | Brand_amule
 | Brand_lphant
-  
-let brand_count = 13
-  
+| Brand_emuleplus
+| Brand_hydranode
+
+let brand_list = [
+   ( Brand_unknown   , "unknown"      , "unk" ) ;
+   ( Brand_edonkey   , "eDonkey"      , "eDK" ) ;
+   ( Brand_mldonkey1 , "old mldonkey" , "oML" ) ;
+   ( Brand_mldonkey2 , "new mldonkey" , "nML" ) ;
+   ( Brand_overnet   , "Overnet"      , "OVR" ) ;
+   ( Brand_newemule  , "eMule"        , "eMU" ) ;
+   ( Brand_server    , "server"       , "SER" ) ;
+   ( Brand_mldonkey3 , "trusted mld"  , "tML" ) ;
+   ( Brand_cdonkey   , "cDonkey"      , "cDK" ) ;
+   ( Brand_lmule     , "xMule"        , "xMU" ) ;
+   ( Brand_shareaza  , "shareaza"     , "sZA" ) ;
+   ( Brand_amule     , "aMule"        , "aMU" ) ;
+   ( Brand_lphant    , "lPhant"       , "lPH" ) ;
+   ( Brand_emuleplus , "ePlus"        , "eM+" ) ;
+   ( Brand_hydranode , "Hydra"        , "Hyd" ) ;
+  ]
+
+let brand_count = List.length brand_list
+
 type brand_mod =
   Brand_mod_unknown
 | Brand_mod_extasy
@@ -314,39 +336,173 @@ type brand_mod =
 | Brand_mod_enos
 | Brand_mod_webys
 
-let brand_mod_count = 113
+let brand_mod_list = [
+   ( Brand_mod_unknown       , "unknown"       , ""     ) ;
+   ( Brand_mod_extasy        , "Extasy"        , "ext"  ) ;
+   ( Brand_mod_hunter        , "Hunter"        , "hun"  ) ;
+   ( Brand_mod_sivka         , "Sivka"         , "siv"  ) ;
+   ( Brand_mod_ice           , "IcE"           , "ice"  ) ;
+   ( Brand_mod_plus          , "Plus"          , "plu"  ) ;
+   ( Brand_mod_lsd           , "LSD"           , "lsd"  ) ;
+   ( Brand_mod_maella        , "Maella"        , "mae"  ) ;
+   ( Brand_mod_pille         , "Pille"         , "pil"  ) ;
+   ( Brand_mod_morphkad      , "MorphKad"      , "mo1"  ) ;
+   ( Brand_mod_efmod         , "eF-MOD"        , "efm"  ) ;
+   ( Brand_mod_xtreme        , "Xtreme"        , "xtr"  ) ;
+   ( Brand_mod_bionic        , "Bionic"        , "bio"  ) ;
+   ( Brand_mod_pawcio        , "Pawcio"        , "paw"  ) ;
+   ( Brand_mod_zzul          , "ZZUL"          , "zzu"  ) ;
+   ( Brand_mod_blackhand     , "Black Hand"    , "bla"  ) ;
+   ( Brand_mod_lovelace      , "lovelace"      , "lov"  ) ;
+   ( Brand_mod_morphnext     , "MorphNext"     , "mo2"  ) ;
+   ( Brand_mod_fincan        , "fincan"        , "fin"  ) ;
+   ( Brand_mod_ewombat       , "eWombat"       , "ewo"  ) ;
+   ( Brand_mod_morph         , "Morph"         , "mo3"  ) ;
+   ( Brand_mod_mortillo      , "MorTillo"      , "mot"  ) ;
+   ( Brand_mod_lh            , "LionHeart"     , "lh"   ) ;
+   ( Brand_mod_emulespana    , "emulEspa\241a" , "esp"  ) ;
+   ( Brand_mod_blackrat      , "BlackRat"      , "blr"  ) ;
+   ( Brand_mod_enkeydev      , "enkeyDev"      , "ekd"  ) ;
+   ( Brand_mod_gnaddelwarz   , "Gnaddelwarz"   , "gna"  ) ;
+   ( Brand_mod_phoenixkad    , "pHoeniX-KAD"   , "pkd"  ) ;
+   ( Brand_mod_koizo         , "koizo"         , "koi"  ) ;
+   ( Brand_mod_ed2kfiles     , "ed2kFiles"     , "edf"  ) ;
+   ( Brand_mod_athlazan      , "Athlazan"      , "ath"  ) ;
+   ( Brand_mod_cryptum       , "Cryptum"       , "cry"  ) ;
+   ( Brand_mod_lamerzchoice  , "LamerzChoice"  , "lam"  ) ;
+   ( Brand_mod_notdead       , "NotDead"       , "nod"  ) ;
+   ( Brand_mod_peace         , "peace"         , "pea"  ) ;
+   ( Brand_mod_goldicryptum  , "GoldiCryptum"  , "gcr"  ) ;
+   ( Brand_mod_eastshare     , "EastShare"     , "eas"  ) ;
+   ( Brand_mod_mfck          , "[MFCK]"        , "mfc"  ) ;
+   ( Brand_mod_echanblard    , "eChanblard"    , "ech"  ) ;
+   ( Brand_mod_sp4rk         , "Sp4rK"         , "sp4"  ) ;
+   ( Brand_mod_powermule     , "PowerMule"     , "pow"  ) ;
+   ( Brand_mod_bloodymad     , "bloodymad"     , "blo"  ) ;
+   ( Brand_mod_roman2k       , "Roman2K"       , "rom"  ) ;
+   ( Brand_mod_gammaoh       , "GaMMaOH"       , "gam"  ) ;
+   ( Brand_mod_elfenwombat   , "ElfenWombat"   , "elf"  ) ;
+   ( Brand_mod_o2            , "O2"            , "o2"   ) ;
+   ( Brand_mod_dm            , "DM"            , "DM"   ) ;
+   ( Brand_mod_sfiom         , "SF-IOM"        , "SFI"  ) ;
+   ( Brand_mod_magic_elseve  , "Magic-Elseve"  , "MEl"  ) ;
+   ( Brand_mod_schlumpmule   , "SchlumpMule"   , "sch"  ) ;
+   ( Brand_mod_lc            , "LC"            , "LC"   ) ;
+   ( Brand_mod_noamson       , "NoamSon"       , "NoS"  ) ;
+   ( Brand_mod_stormit       , "Stormit"       , "Sto"  ) ;
+   ( Brand_mod_omax          , "OMaX"          , "OMX"  ) ;
+   ( Brand_mod_mison         , "Mison"         , "Mis"  ) ;
+   ( Brand_mod_phoenix       , "Phoenix"       , "pPho" ) ;
+   ( Brand_mod_spiders       , "Spiders"       , "spi"  ) ;
+   ( Brand_mod_iberica       , "Ib\233rica"    , "Ib"   ) ;
+   ( Brand_mod_mortimer      , "Mortimer"      , "mor"  ) ;
+   ( Brand_mod_stonehenge    , "Stonehenge"    , "sto"  ) ;
+   ( Brand_mod_xlillo        , "Xlillo"        , "Xli"  ) ;
+   ( Brand_mod_imperator     , "ImperatoR"     , "Imp"  ) ;
+   ( Brand_mod_raziboom      , "Raziboom"      , "Raz"  ) ;
+   ( Brand_mod_khaos         , "Khaos"         , "Kha"  ) ;
+   ( Brand_mod_hardmule      , "Hardmule"      , "Har"  ) ;
+   ( Brand_mod_sc            , "SC"            , "SC"   ) ;
+   ( Brand_mod_cy4n1d        , "Cy4n1d"        , "Cy4"  ) ;
+   ( Brand_mod_dmx           , "DMX"           , "DMX"  ) ;
+   ( Brand_mod_ketamine      , "Ketamine"      , "Ket"  ) ;
+   ( Brand_mod_blackmule     , "Blackmule"     , "blm"  ) ;
+   ( Brand_mod_morphxt       , "MorphXT"       , "Mxt"  ) ;
+   ( Brand_mod_ngdonkey      , "ngdonkey"      , "ngd"  ) ;
+   ( Brand_mod_cyrex         , "Cyrex"         , "haw"  ) ;
+   ( Brand_mod_hawkstar      , "Hawkstar"      , "neo"  ) ;
+   ( Brand_mod_neomule       , "Neo Mule"      , "cyr"  ) ;
+   ( Brand_mod_aldo          , "aldo"          , "ald"  ) ;
+   ( Brand_mod_emulede       , "emule.de"      , "ede"  ) ;
+   ( Brand_mod_zx            , "zx"            , "zx"   ) ;
+   ( Brand_mod_ibericaxt     , "ib\233ricaxt"  , "iBx"  ) ;
+   ( Brand_mod_candymule     , "candy-mule"    , "can"  ) ;
+   ( Brand_mod_ackronic      , "ackronic"      , "ack"  ) ;
+   ( Brand_mod_rappis        , "rappis"        , "rap"  ) ;
+   ( Brand_mod_overdose      , "overdose"      , "ove"  ) ;
+   ( Brand_mod_hebmule       , "hebmule"       , "heb"  ) ;
+   ( Brand_mod_senfei        , "senfei"        , "sen"  ) ;
+   ( Brand_mod_spoofmod      , "spoofmod"      , "spo"  ) ;
+   ( Brand_mod_fusspilz      , "fusspilz"      , "fus"  ) ;
+   ( Brand_mod_rocket        , "rocket"        , "roc"  ) ;
+   ( Brand_mod_warezfaw      , "warezfaw"      , "war"  ) ;
+   ( Brand_mod_emusicmule    , "emusicmule"    , "emm"  ) ;
+   ( Brand_mod_aideadsl      , "aideadsl"      , "aid"  ) ;
+   ( Brand_mod_epo           , "epo"           , "epo"  ) ;
+   ( Brand_mod_kalitsch      , "kalitsch"      , "kal"  ) ;
+   ( Brand_mod_raynz         , "raynz"         , "ray"  ) ;
+   ( Brand_mod_serverclient  , "serverclient"  , "sc"   ) ;
+   ( Brand_mod_bl4ckbird     , "bl4ckbird"     , "b4b"  ) ;
+   ( Brand_mod_bl4ckf0x      , "bl4ckf0x"      , "b4f"  ) ;
+   ( Brand_mod_rt            , "rt"            , "rt"   ) ;
+   ( Brand_mod_airionix      , "air-ionix"     , "aio"  ) ;
+   ( Brand_mod_ionix         , "ionix"         , "ion"  ) ;
+   ( Brand_mod_tornado       , "tornado"       , "tor"  ) ;
+   ( Brand_mod_antifaker     , "anti-faker"    , "anf"  ) ;
+   ( Brand_mod_netf          , "netf"          , "nef"  ) ;
+   ( Brand_mod_nextemf       , "nextemf"       , "nxf"  ) ;
+   ( Brand_mod_proemule      , "proemule"      , "pem"  ) ;
+   ( Brand_mod_szemule       , "szemule"       , "sze"  ) ;
+   ( Brand_mod_darkmule      , "darkmule"      , "dar"  ) ;
+   ( Brand_mod_miragemod     , "miragemod"     , "mir"  ) ;
+   ( Brand_mod_nextevolution , "nextevolution" , "nxe"  ) ;
+   ( Brand_mod_pootzgrila    , "pootzgrila"    , "poo"  ) ;
+   ( Brand_mod_freeangel     , "freeangel"     , "fre"  ) ;
+   ( Brand_mod_enos          , "enos"          , "eno"  ) ;
+   ( Brand_mod_webys         , "webys"         , "wys"  ) ;
+  ]
 
-type source_uid = 
+let brand_mod_count = List.length brand_mod_list
+
+let brand_to_string brand =
+  find_brand_to_string brand brand_list false
+
+let brand_to_string_short brand =
+  find_brand_to_string brand brand_list true
+
+let brand_mod_to_string brand =
+  find_brand_to_string brand brand_mod_list false
+
+let brand_mod_to_string_short brand =
+  find_brand_to_string brand brand_mod_list true
+
+let brand_to_int brand =
+  find_int_of_brand brand brand_list
+
+let brand_mod_to_int brand =
+  find_int_of_brand brand brand_mod_list
+
+type source_uid =
   Direct_address of Ip.t * int
 | Indirect_address of Ip.t * int * int64
 | Invalid_address of string * string
 
 let id_of_ip ip = Ip.to_int64 (Ip.rev ip)
 let ip_of_id id = Ip.rev (Ip.of_int64 id)
-  
+
 module DonkeySources = CommonSources.Make(struct
-      
+
       open Options
 
       type source_brand = bool
       let value_to_source_brand = value_to_bool
       let source_brand_to_value = bool_to_value
       let dummy_source_brand = false
-        
+
       type t = source_uid
       type source_uid = t
 
       let module_name = "DonkeySources"
-        
+
       let direct_source s =
         match s with Direct_address _ -> true | _ -> false
-      
+
       let indirect_source s =
         match s with Indirect_address _ -> true | _ -> false
-      
+
       let dummy_source_uid = Direct_address (Ip.null, 0)
-            
-      let value_to_source_uid v = 
+
+      let value_to_source_uid v =
         match v with
           List [ip;port] | SmallList [ip;port] ->
             let ip = Ip.of_string (value_to_string ip) in
@@ -358,27 +514,28 @@ module DonkeySources = CommonSources.Make(struct
             let id = try id_of_ip (Ip.of_string (value_to_string id))
               with _ -> value_to_int64 id in
             Indirect_address (ip, port, id)
-        | _ -> 
+        | _ ->
             failwith "bad client address"
-      
-      let source_uid_to_value s = 
+
+      let source_uid_to_value s =
         match s with
-          Direct_address (ip, port) -> 
+          Direct_address (ip, port) ->
             SmallList [string_to_value (Ip.to_string ip); int_to_value port]
-        | Indirect_address (ip, port, id) -> 
+        | Indirect_address (ip, port, id) ->
             SmallList [
-              string_to_value (Ip.to_string ip); 
+              string_to_value (Ip.to_string ip);
               int_to_value port;
               int64_to_value id;
             ]
         | Invalid_address _ -> failwith "Invalid address"
     end)
-  
+
 type server = (*[]*){
     mutable server_server : server CommonServer.server_impl;
     mutable server_ip : Ip.t;
     mutable server_cid : Ip.t option;
     mutable server_port : int;
+    mutable server_realport : int option; (* in case we connect through auxport; this is the true one *)
     mutable server_sock : tcp_connection;
     mutable server_nqueries : int;
     mutable server_search_queries : CommonTypes.search Fifo.t;
@@ -399,14 +556,14 @@ type server = (*[]*){
     mutable server_last_message : int; (* used only by mldonkey server *)
 
     mutable server_id_requests : file option Fifo.t;
-    
+
     mutable server_queries_credit : int;
     mutable server_waiting_queries : file list;
     mutable server_sent_all_queries : bool;
     mutable server_has_zlib : bool;
-    
+
     mutable server_flags : int;
-  } 
+  }
 
 
 and user = {
@@ -441,32 +598,30 @@ and result = {
     mutable result_index : Store.index;
   }
 *)
-  
+
 and search_event =
 (*  Result of result *)
 | Waiting of int
 
-and file_change_kind = 
+and file_change_kind =
   NoFileChange
 | FileAvailabilityChange
 | FileInfoChange
 
-and client_change_kind = 
+and client_change_kind =
   NoClientChange
 | ClientStateChange
 | ClientFriendChange
 | ClientInfoChange
 | ClientFilesChange
 
-and server_change_kind = 
+and server_change_kind =
   NoServerChange
 | ServerStateChange
 | ServerUsersChange
 | ServerInfoChange
 | ServerBusyChange
 
-and availability = bool array
-
   (*
 and source = {
 (*    source_num : int; *)
@@ -485,7 +640,7 @@ and source = {
     mutable source_sock : tcp_connection;
   }
 *)
-  
+
 and client = {
     client_client : client CommonClient.client_impl;
     mutable client_kind : source_uid;
@@ -494,13 +649,13 @@ and client = {
 (*    mutable client_chunks : availability; *)
 (*     mutable client_sock : tcp_connection; *)
     mutable client_ip : Ip.t;
-    mutable client_download : (file * Int64Swarmer.uploader) option; 
+    mutable client_download : (file * Int64Swarmer.uploader) option;
 (*    mutable client_block : Int64Swarmer.block option; *)
 (*    mutable client_zones : (int64 * int64 * Int64Swarmer.range) list; *)
 (*    mutable client_connection_control : connection_control; *)
     mutable client_file_queue : (
       file * (* has displayed when connected *)
-      availability *
+      Bitv.t *
       Int64Swarmer.uploader
       ) list;
     mutable client_next_view_files :  int;
@@ -517,7 +672,7 @@ and client = {
     mutable client_downloaded : Int64.t;
     mutable client_uploaded : Int64.t;
     mutable client_brand : brand;
-    mutable client_mod_brand : brand_mod;
+    mutable client_brand_mod : brand_mod;
     mutable client_banned : bool;
     mutable client_score : int;
     mutable client_next_queue : int;
@@ -532,21 +687,25 @@ and client = {
     client_emule_proto : emule_proto;
     mutable client_comp : compressed_parts option;
     mutable client_connection_time : int;
+    mutable client_sent_challenge : Int64.t;
+    mutable client_req_challenge : Int64.t;
+    mutable client_public_key : string option;
+    mutable client_sui_verified : bool option;
   }
-  
+
 and compressed_parts = {
     comp_md4 : Md4.t;
-    comp_pos : int64; 
+    comp_pos : int64;
     comp_total : int;
     mutable comp_len : int;
     mutable comp_blocs : string list;
   }
 
-and slot_status = 
+and slot_status =
   SlotNotAsked
 | SlotAsked
 | SlotReceived
-  
+
 and upload_info = {
     mutable up_file : file;
     mutable up_pos : int64;
@@ -556,15 +715,15 @@ and upload_info = {
   }
 
 (*
-  
-and chunk = 
+
+and chunk =
   PresentTemp
 | AbsentTemp
 | PartialTemp of block
 | PresentVerified
 | AbsentVerified
 | PartialVerified of block
-  
+
 and block = {
     mutable block_present: bool;
     block_begin : int64;
@@ -576,7 +735,7 @@ and block = {
     mutable block_pos : int;
     block_file : file;
   }
-  
+
 and zone = {
     mutable zone_begin : int64;
     mutable zone_end : int64;
@@ -592,7 +751,7 @@ and file_request = {
     mutable  request_result : request_result;
   }
 
-and request_result = 
+and request_result =
 | File_possible   (* we asked, but didn't know *)
 | File_not_found  (* we asked, the file is not there *)
 | File_expected   (* we asked, because it was announced *)
@@ -601,22 +760,23 @@ and request_result =
 | File_chunk      (* the file has chunks we want *)
 | File_upload     (* we uploaded from this client *)
 *)
-  
-and client_kind = 
+
+and client_kind =
   SourceClient of client
-| SourceLastConnection of 
+| SourceLastConnection of
   int *
   int * (* last connection attempt *)
   int   (* booked client num *)
-  
+
 and file = {
     file_file : file CommonFile.file_impl;
     file_md4 : Md4.t;
 (*    file_exists : bool; *)
-    
+
     mutable file_swarmer : Int64Swarmer.t option;
-    
+
     mutable file_nchunks : int;
+    mutable file_diskname : string;
 (*    mutable file_chunks : chunk array; *)
 (*    mutable file_chunks_order : int array; *)
 (*    mutable file_chunks_age : int array; *)
@@ -624,7 +784,7 @@ and file = {
 (*    mutable file_absent_chunks : (int64 * int64) list; *)
     mutable file_filenames : (string * GuiTypes.ips_list) list;
 (*    mutable file_nsources : int; *)
-    mutable file_computed_md4s : Md4.t array; 
+    mutable file_computed_md4s : Md4.t array;
     mutable file_format : format;
 (*    mutable file_available_chunks : int array; *)
     mutable file_shared : file CommonShared.shared_impl option;
@@ -645,26 +805,25 @@ and file_to_share = {
     mutable shared_fd : Unix32.t;
     shared_shared : file_to_share CommonShared.shared_impl;
   }
-  
+
 module UdpClientMap = Map.Make(struct
       type t = source_uid
       let compare = compare
     end)
-  
+
 type udp_client = {
     udp_client_ip : Ip.t;
     udp_client_port : int;
     udp_client_can_receive : bool;
     mutable udp_client_last_conn : int;
   }
-  
+
 and file_group = {
     mutable group : udp_client UdpClientMap.t;
   }
 
-
 type old_result = result
-    
+
 exception NoSpecifiedFile
 exception Already_done
 
@@ -675,74 +834,73 @@ type shared_file_info = {
     sh_size : int64;
   }
 
-  
 open CommonFile
 open CommonClient
 open CommonServer
-open CommonResult 
-        
+open CommonResult
+
 let set_client_state c s =
   let cc = as_client c.client_client in
   set_client_state cc s
-        
+
 let set_client_disconnected c =
   let cc = as_client c.client_client in
   set_client_disconnected cc
-   
+
 let set_server_state s state =
   let ss = as_server s.server_server in
   if server_state ss <> state then begin
       set_server_state ss state;
     end
-  
+
 let file_state file =
   CommonFile.file_state (as_file file.file_file)
-  
+
 let file_last_seen file = file.file_file.impl_file_last_seen
-  
+
 let file_must_update file =
   file_must_update (as_file file.file_file)
-    
+
 let client_state client =
   CommonClient.client_state (as_client client.client_client)
-    
+
 let client_new_file client r =
   client_new_file (as_client client.client_client) ""
   r
-    
+
 let server_state server =
   CommonServer.server_state (as_server server.server_server)
 
-(*    
-module SourcesQueueCreate = Queues.Make (struct  
+(*
+module SourcesQueueCreate = Queues.Make (struct
 
       type t = source
       let compare s1 s2 = compare s1.source_addr s2.source_addr
     end)
-  
+
   (*
-    let lifo = lifo 
+    let lifo = lifo
     let fifo = fifo
-    
+
     module SourcesSet = Set.Make (
         struct
           type t = int * source
-          let compare (t1,s1) (t2,s2) = 
+          let compare (t1,s1) (t2,s2) =
             if s1.source_addr = s2.source_addr then begin
-                0 end else              
+                0 end else
             let x = compare t1 t2 in
             if x = 0 then compare s1.source_addr s2.source_addr else x
         end
       )
 
-    let oldest_first () = 
+    let oldest_first () =
       let t = ref SourcesSet.empty in
       of_impl {
         head = (fun _ -> try SourcesSet.min_elt !t with _ -> raise Fifo.Empty);
         put = (fun x ->  t := SourcesSet.add x !t);
         length = (fun _ -> SourcesSet.cardinal !t);
         take = (fun _ ->
-            try 
+            try
               let x = SourcesSet.min_elt !t in
               t := SourcesSet.remove x !t;
               x
@@ -752,7 +910,7 @@ module SourcesQueueCreate = Queues.Make (struct
         put_back = (fun e -> t := SourcesSet.add e !t);
       }
 
-    let oldest_last () = 
+    let oldest_last () =
       let t = ref SourcesSet.empty in
       of_impl {
         head = (fun _ ->
@@ -813,42 +971,8 @@ module SourcesQueueCreate = Queues.Make (struct
 *)
   *)
 
-type brand_stat = {
-  mutable brand_seen : int;
-  mutable brand_banned : int;
-  mutable brand_filerequest : int;
-  mutable brand_download : Int64.t;
-  mutable brand_upload : Int64.t;
-}
-  
-type brand_mod_stat = {
-  mutable brand_mod_seen : int;
-  mutable brand_mod_banned : int;
-  mutable brand_mod_filerequest : int;
-  mutable brand_mod_download : Int64.t;
-  mutable brand_mod_upload : Int64.t;
-}
-
-let dummy_stats =
-  {
-    brand_seen = 0;
-    brand_banned = 0;
-    brand_filerequest = 0;
-    brand_download = Int64.zero;
-    brand_upload = Int64.zero
-  }
-
-let dummy_mod_stats =
-  {
-    brand_mod_seen = 0;
-    brand_mod_banned = 0;
-    brand_mod_filerequest = 0;
-    brand_mod_download = Int64.zero;
-    brand_mod_upload = Int64.zero
-  }
-
   (*
-let string_of_file_tag_name name = 
+let string_of_file_tag_name name =
   match name with
   | FT_FILE_SIZE -> "size"
   | FT_FILE_FILENAME -> "filename"
@@ -873,7 +997,7 @@ let string_of_server_tag_name name =
   | ST_SERVER_PING -> "ping"
   | ST_SERVER_PROF -> "prof"
   | ST_SERVER_HISTORY -> "history"
-    
+
 let string_of_client_tag_name name =
   match name with
   | CT_CLIENT_NAME -> "name"
@@ -890,27 +1014,27 @@ let string_of_emule_tag_name name =
   | ET_SOURCEEXCHANGE -> "sourceexchange"
   | ET_COMMENTS -> "comments"
   | ET_EXTENDEDREQUEST -> "extendedrequest"
-  | ET_COMPATIBLECLIENT -> "compatableclient"
+  | ET_COMPATIBLECLIENT -> "compatibleclient"
   | ET_MOD_FEATURESET -> "mod_featureset"
   | ET_MOD_PROTOCOL -> "mod_protocol"
   | ET_MOD_VERSION -> "mod_version"
   | ET_MOD_TAROD -> "mod_tarod"
   | ET_TAROD_VERSION -> "tarod_version"
   | ET_MOD_PLUS -> "mod_plus"
-    
+
 let string_of_pref_tag_name name =
   match name with
   | PT_PREF_NAME -> ""
   | PT_PREF_PORT -> ""
   | PT_PREF_VERSION -> ""
   | PT_PREF_TEMP -> ""
-        *)
+ *)
   (*
 let dummy_source = {
 (*    source_num = 0; *)
     source_addr = (Ip.null, 0);
     source_files = [];
-              
+
     source_last_score = 0;
     source_last_age = 0;
     source_client_num = 0;
@@ -927,10 +1051,11 @@ let dummy_source = {
 let dummy_emule_proto = {
     emule_comments = 0;
     emule_version = 0;
+    emule_release = "";
     emule_secident = 0;
     emule_noviewshared = 0;
     emule_supportpreview = 0;
-    
+
     emule_compression = 0; (* 1 *)
     emule_sourceexchange = 0; (* 3 *)
     emule_multipacket = 0; (* 1 *)
@@ -939,6 +1064,5 @@ let dummy_emule_proto = {
     emule_udpver = 0; (* 4 *)
   }
 
-let emule_proto () = 
+let emule_proto () =
   { dummy_emule_proto with emule_version = 0 }
-  
