@@ -25,7 +25,7 @@ open Options
 
 open BasicSocket
 open TcpBufferedSocket
-  
+
 open CommonShared
 open CommonUploads
 open CommonOptions
@@ -36,8 +36,8 @@ open CommonComplexOptions
 open CommonTypes
 open CommonFile
 open CommonGlobals
-open CommonDownloads  
-  
+open CommonDownloads
+
 open FileTPTypes
 open FileTPOptions
 open FileTPGlobals
@@ -46,36 +46,52 @@ open FileTPProtocol
 
 open FileTPClients
 
+(* prints a new logline with date, module and starts newline *)
+let lprintf_nl () =
+  lprintf "%s[HTTP] "
+    (log_time ()); lprintf_nl2
+
+(* prints a new logline with date, module and does not start newline *)
+let lprintf_n () =
+  lprintf "%s[HTTP] "
+    (log_time ()); lprintf
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         MAIN                                          *)
 (*                                                                       *)
 (*************************************************************************)
-  
-let http_send_range_request c range sock d = 
+
+let http_send_range_request c range sock d =
   let url = d.download_url in
+
+  let real_url =
+    (Str.global_replace (Str.regexp " ") "%20" url.Url.full_file) in
+
+  let referer = d.download_referer in
   
   let (x,y) = range in
-  let range = Printf.sprintf "%Ld-%Ld" x (y -- Int64.one) in
+  let range = Printf.sprintf "%Ld-%Ld" x (Int64.pred y) in
 
   let buf = Buffer.create 100 in
-  
-  Printf.bprintf buf "GET %s HTTP/1.0\r\n" url.Url.full_file;
+
+  Printf.bprintf buf "GET %s HTTP/1.0\r\n" real_url;
 
 (*
             (match d.download_uri with
                 FileByUrl url -> Printf.bprintf buf "GET %s HTTP/1.0\r\n" url
-              | FileByIndex (index, name) -> 
+              | FileByIndex (index, name) ->
                   Printf.bprintf buf "GET /get/%d/%s HTTP/1.1\r\n" index
                     name); *)
   Printf.bprintf buf "Host: %s\r\n" c.client_hostname;
-  Printf.bprintf buf "User-Agent: %s\r\n" user_agent; 
+  Printf.bprintf buf "User-Agent: %s\r\n" user_agent;
+  Printf.bprintf buf "Referer: %s\r\n" (Url.to_string referer);
   Printf.bprintf buf "Range: bytes=%s\r\n" range;
   Printf.bprintf buf "Connection: Keep-Alive\r\n";
   Printf.bprintf buf "\r\n";
   let s = Buffer.contents buf in
   if !verbose_msg_clients then
-    lprintf "SENDING REQUEST to %s: %s\n" 
+    lprintf_nl () "SENDING REQUEST to %s: %s"
       c.client_hostname
       (String.escaped s);
   write_string sock s;
@@ -87,78 +103,97 @@ let http_send_range_request c range sock d =
 (*                                                                       *)
 (*************************************************************************)
 
-  
-let rec client_parse_header c gconn sock header = 
-  if !verbose_msg_clients then begin
-      lprintf "CLIENT PARSE HEADER\n"; 
-    end;
+let rec client_parse_header c gconn sock header =
+  if !verbose_msg_clients then
+    lprintf_nl () "CLIENT PARSE HEADER";
   try
     set_lifetime sock 3600.;
-    let d = 
-      match c.client_requests with 
+    let d =
+      match c.client_requests with
         [] -> failwith "No download request !!!"
       | d :: tail ->
           c.client_requests <- tail;
           d
     in
     connection_ok c.client_connection_control;
-    set_client_state c Connected_initiating;    
+    set_client_state c Connected_initiating;
     if !verbose_msg_clients then begin
-        lprintf "HEADER FROM CLIENT:\n";
-        AnyEndian.dump_ascii header; 
+        lprintf_nl () "HEADER FROM CLIENT:";
+        AnyEndian.dump_ascii header;
       end;
     let file = d.download_file in
     let size = file_size file in
-    
+
     let endline_pos = String.index header '\n' in
-    let http, code = 
+    let http, code =
       match String2.split (String.sub header 0 endline_pos
         ) ' ' with
-      | http :: code :: ok :: _ -> 
+      | http :: code :: ok :: _ ->
           let code = int_of_string code in
           if not (String2.starts_with (String.lowercase http) "http") then
             failwith "Not in http protocol";
           http, code
       | _ -> failwith "Not a HTTP header line"
     in
-    if !verbose_msg_clients then begin
-        lprintf "GOOD HEADER FROM CONNECTED CLIENT\n";
-      end;
-    
+    if !verbose_msg_clients then
+      lprintf_nl () "GOOD HEADER FROM CONNECTED CLIENT\n";
+
     set_rtimeout sock 120.;
 (*              lprintf "SPLIT HEADER...\n"; *)
     let lines = Http_client.split_header header in
 (*              lprintf "REMOVE HEADLINE...\n"; *)
     let first_line, headers = match lines with
-        [] -> raise Not_found        
+        [] -> raise Not_found
       | line :: headers -> line, headers
     in
 (*                  lprintf "CUT HEADERS...\n"; *)
     let headers = Http_client.cut_headers headers in
 (*                  lprintf "START POS...\n"; *)
-    
-    
+
     if !verbose_unknown_messages then begin
         let unknown_header = ref false in
         List.iter (fun (header, _) ->
             unknown_header := !unknown_header || not (List.mem header known_download_headers)
         ) headers;
         if !unknown_header then begin
-            lprintf "FT DEVEL: Download Header contains unknown fields\n";
-            lprintf "    %s\n" first_line;
+            lprintf_n () "Download Header contains unknown fields";
+            lprintf_nl2 "    %s" first_line;
             List.iter (fun (header, (value,header2)) ->
-                lprintf "    [%s] = [%s](%s)\n" header value header2;
+                lprintf_nl () "    [%s] = [%s](%s)" header value header2;
             ) headers;
-            lprintf "FT DEVEL: end of header\n";        
+            lprintf_nl () "end of header";
           end;
       end;
-    
-    
+
+    (* I think this is already handeled with the new headder check before
+       download start code
+    (* If the header contains a redirection *)
+    if (code = 302) then begin
+      let (newurl, _) = List.assoc "location" headers in
+
+      remove_file file;
+      file_cancel (as_file file);
+      let u = Url.of_string newurl in
+
+      c.client_hostname <- u.Url.server;
+      c.client_port <- u.Url.port;
+
+      let file = new_file (Md4.random ()) u.Url.full_file zero in
+
+      lprintf_nl () "DOWNLOAD FILE %s" (file_best_name  file);
+      if not (List.memq file !current_files) then begin
+    current_files := file :: !current_files;
+      end;
+      add_download file c u r;
+      FileTPClients.get_file_from_source c file;
+
+     end;
+    *)
+
     if  code < 200 || code > 299 then
       failwith "Bad HTTP code";
-    
-    
-    let start_pos, end_pos = 
+
+    let start_pos, end_pos =
       try
         let (range,_) = List.assoc "content-range" headers in
         try
@@ -176,61 +211,62 @@ let rec client_parse_header c gconn sock header =
           let y = Int64.of_string (
               String.sub range (dash_pos+1) (slash_pos - dash_pos - 1))
           in
-          if slash_pos = star_pos - 1 then 
-            x,y ++ Int64.one (* "bytes x-y/*" *)
+          if slash_pos = star_pos - 1 then
+            x, Int64.succ y (* "bytes x-y/*" *)
           else
           let z = Int64.of_string (
               String.sub range (slash_pos+1) (len - slash_pos -1) )
           in
-          if y = z then x -- Int64.one, size else 
-            x,y ++ Int64.one
-        with 
+          if y = z then Int64.pred x, size else
+            x, Int64.succ y
+        with
         | e ->
-            lprintf "Exception %s for range [%s]\n" 
+            lprintf_nl () "Exception %s for range [%s]"
               (Printexc2.to_string e) range;
             raise e
-      with e -> 
+      with e ->
           try
             if code <> 206 && code <> 200 then raise Not_found;
             let (len,_) = List.assoc "content-length" headers in
             let len = Int64.of_string len in
-            lprintf "Specified length: %Ld\n" len;
+            if !verbose then lprintf_nl () "Specified length: %Ld" len;
             match d.download_ranges with
               [] -> raise Not_found
-            | (start_pos,end_pos,r) :: _ -> 
-                lprintf "WARNING: Assuming client is replying to range\n";
+            | (start_pos,end_pos,r) :: _ ->
+                lprintf_nl () "WARNING: Assuming client is replying to range";
                 if len <> end_pos -- start_pos then
                   begin
-                    lprintf "\n\nERROR: bad computed range: %Ld-%Ld/%Ld \n%s\n"
+                    lprintf_nl () "ERROR: bad computed range: %Ld-%Ld/%Ld \n%s"
                       start_pos end_pos len
                       (String.escaped header);
                     raise Not_found
                   end;
                 (start_pos, end_pos)
-          with _ -> 
+          with _ ->
 (* A bit dangerous, no ??? *)
-              lprintf "ERROR: Could not find/parse range header (exception %s), disconnect\nHEADER: %s\n" 
-                (Printexc2.to_string e)
-              (String.escaped header);
+              if !verbose_hidden_errors then
+                lprintf_nl () "ERROR: Could not find/parse range header (exception %s), disconnect\nHEADER: %s"
+                    (Printexc2.to_string e)
+                    (String.escaped header);
               disconnect_client c (Closed_for_error "Bad HTTP Range");
               raise Exit
-    in 
+    in
     (try
         let (len,_) = List.assoc "content-length" headers in
         let len = Int64.of_string len in
-        lprintf "Specified length: %Ld\n" len;
+        if !verbose then lprintf_nl () "Specified length: %Ld" len;
         if len <> end_pos -- start_pos then
           begin
             failwith "\n\nERROR: bad computed range: %Ld-%Ld/%Ld \n%s\n"
               start_pos end_pos len
               (String.escaped header);
           end
-      with _ -> 
-          lprintf "[WARNING]: no Content-Length field\n%s\n"
+      with _ ->
+          lprintf_nl () "[WARNING]: no Content-Length field\n%s\n"
             (String.escaped header)
     );
-    
-    lprintf "Receiving range: %Ld-%Ld (len = %Ld)\n%s\n"    
+
+    if !verbose then lprintf_nl () "Receiving range: %Ld-%Ld (len = %Ld)\n%s"
       start_pos end_pos (end_pos -- start_pos)
     (String.escaped header)
     ;
@@ -240,32 +276,32 @@ let rec client_parse_header c gconn sock header =
     for i = 1 to max_queued_ranges do
       if List.length d.download_ranges <= max_queued_ranges then
         (try get_from_client sock c with _ -> ());
-    done; 
+    done;
     gconn.gconn_handler <- Reader (fun gconn sock ->
         if file_state file <> FileDownloading then begin
             disconnect_client c Closed_by_user;
             raise Exit;
           end;
-        
+
         let b = TcpBufferedSocket.buf sock in
-        let to_read = min (end_pos -- !counter_pos) 
+        let to_read = min (end_pos -- !counter_pos)
           (Int64.of_int b.len) in
-(*
-        lprintf "Reading: end_pos %Ld counter_pos %Ld len %d = to_read %Ld\n"
+
+        if !verbose then lprintf_nl () "Reading: end_pos %Ld counter_pos %Ld len %d = to_read %Ld"
 end_pos !counter_pos b.len to_read;
-   *)
+
         let to_read_int = Int64.to_int to_read in
 (*
-  lprintf "CHUNK: %s\n" 
+        if !verbose then lprintf "CHUNK: %s\n"
           (String.escaped (String.sub b.buf b.pos to_read_int)); *)
         let swarmer = match file.file_swarmer with
             None -> assert false | Some sw -> sw
         in
-        let old_downloaded = 
+        let old_downloaded =
           Int64Swarmer.downloaded swarmer in
-(*        List.iter (fun (_,_,r) -> Int64Swarmer.free_range r)  
+(*        List.iter (fun (_,_,r) -> Int64Swarmer.free_range r)
         d.download_ranges; *)
-        
+
         begin
           try
             match d.download_uploader with
@@ -273,29 +309,29 @@ end_pos !counter_pos b.len to_read;
             | Some up ->
                 Int64Swarmer.received up
                   !counter_pos b.buf b.pos to_read_int;
-          with e -> 
-              lprintf "FT: Exception %s in Int64Swarmer.received\n"
+          with e ->
+              lprintf_nl () "Exception %s in Int64Swarmer.received"
                 (Printexc2.to_string e)
         end;
         c.client_reconnect <- true;
 (*          List.iter (fun (_,_,r) ->
               Int64Swarmer.alloc_range r) d.download_ranges; *)
-        let new_downloaded = 
+        let new_downloaded =
           Int64Swarmer.downloaded swarmer in
-        
+
         (match d.download_ranges with
-            [] -> lprintf "EMPTY Ranges !!!\n"
-          | r :: _ -> 
+            [] -> lprintf_nl () "EMPTY Ranges!"
+          | r :: _ ->
 (*
               let (x,y) = Int64Swarmer.range_range r in
-              lprintf "Received %Ld [%Ld] (%Ld-%Ld) -> %Ld\n"
+              if !verbose then lprintf_nl "Received %Ld [%Ld] (%Ld-%Ld) -> %Ld"
                 !counter_pos to_read
-                x y 
+                x y
                 (new_downloaded -- old_downloaded)
-*)        
+*)
               ()
         );
-        
+
         if new_downloaded = file_size file then
           download_finished file;
 (*
@@ -318,20 +354,22 @@ lprintf "READ: buf_used %d\n" to_read_int;
                 if d.download_ranges = [] then raise Exit;
                 gconn.gconn_handler <- HttpHeader (client_parse_header c);
           end)
-  
+
   with e ->
-      lprintf "Exception %s in client_parse_header\n" (Printexc2.to_string e);
-      AnyEndian.dump header;      
+      if !verbose_hidden_errors then
+        begin
+          lprintf_nl () "Exception %s in client_parse_header" (Printexc2.to_string e);
+          AnyEndian.dump header
+        end;
       disconnect_client c (Closed_for_exception e);
       raise e
-      
 
 (*************************************************************************)
 (*                                                                       *)
 (*                         MAIN                                          *)
 (*                                                                       *)
 (*************************************************************************)
-        
+
 let http_set_sock_handler c sock =
   set_fileTP_sock sock (HttpHeader (client_parse_header c))
 
@@ -340,7 +378,7 @@ let http_set_sock_handler c sock =
 (*                         MAIN                                          *)
 (*                                                                       *)
 (*************************************************************************)
-  
+
 let http_check_size url start_download_file =
   let module H = Http_client in
   let r = {
@@ -350,16 +388,16 @@ let http_check_size url start_download_file =
       H.req_request = H.HEAD;
       H.req_user_agent = user_agent;
     } in
-  
-  H.whead r (fun headers ->      
-      lprintf "RECEIVED HEADERS\n";
+
+  H.whead r (fun headers ->
+      if !verbose then lprintf_nl () "RECEIVED HEADERS";
       let content_length = ref None in
       List.iter (fun (name, content) ->
           if String.lowercase name = "content-length" then
-            try          
+            try
               content_length := Some (Int64.of_string content)
-            with _ -> 
-                lprintf "bad content length [%s]\n" content;
+            with _ ->
+                lprintf_nl () "bad content length [%s]" content;
       ) headers;
       match !content_length with
         None -> failwith "Unable to start download (HEAD failed)"
@@ -374,7 +412,7 @@ let http_check_size url start_download_file =
 
 let http_connect token c f =
   let ip = Ip.from_name c.client_hostname in
-  connect token "fileTP download" 
+  connect token "fileTP download"
       (Ip.to_inet_addr ip) c.client_port
       (fun sock event ->
         match event with
@@ -386,14 +424,13 @@ let http_connect token c f =
 (* You can only use the CONNECTED signal if the socket is not yet controlled
 by the bandwidth manager... 2004/02/03: Normally, not true anymore, it should now work
   even in this case... *)
-        
+
         | CONNECTED ->
-          lprintf "CONNECTED !!! Asking for range...\n"; 
+          if !verbose then lprintf_nl () "CONNECTED !!! Asking for range...";
           f sock
         | _ -> ()
     )
-    
-  
+
 let proto =
   {
     proto_send_range_request = http_send_range_request;
@@ -402,5 +439,3 @@ let proto =
     proto_string = "http";
     proto_connect = http_connect;
   }
-  
-  
\ No newline at end of file
