@@ -35,7 +35,8 @@ open CommonFile
 open CommonTypes
 open CommonGlobals
 open CommonHosts
-  
+
+open G2Network
 open G2Types
 open G2Globals
 open G2Options
@@ -43,12 +44,15 @@ open G2Protocol
 open G2ComplexOptions
 open G2Proto
 
+(* TODO: try to find a more general function *)
+let g2_tag_of_name name = field_of_string name
+  
 let update_client u =
   new_client u.user_kind
 
 let find_urn urn =
   try
-    [Hashtbl.find CommonUploads.shareds_by_uid (Uid.to_string urn)]
+    [find_by_uid urn]
    with _ -> []
 
 let gnutella_uid md4 =
@@ -72,7 +76,7 @@ let g2_packet_handler s sock gconn p =
   if !verbose_msg_servers then begin
       lprintf "Received %s packet from %s:%d: \n%s\n" 
         (match sock with Connection _  -> "TCP" | _ -> "UDP")
-      (Ip.to_string h.host_addr) h.host_port
+      (Ip.string_of_addr h.host_addr) h.host_port
         (Print.print p);
     end;
   match p.g2_payload with 
@@ -104,12 +108,13 @@ let g2_packet_handler s sock gconn p =
         ])
   
   | UPROD -> ()
-  
+  (* We have sent a UPROC request, this is the reply of the client *)
+      
   | LNI ->
       List.iter (fun p ->
           match p.g2_payload with
             LNI_V v -> s.server_vendor <- v
-          | LNI_HS (leaves,_) -> s.server_nusers <- leaves
+          | LNI_HS (leaves,_) -> s.server_nusers <- Int64.of_int leaves
           | _ -> ()
       ) p.g2_children;
       server_send sock s 
@@ -130,7 +135,7 @@ let g2_packet_handler s sock gconn p =
         g2_payload = QKA; 
         g2_children = [
           {
-            g2_payload = QKA_QK (Int32.of_int 123456);
+            g2_payload = QKA_QK 123456l;
             g2_children = [];
           };
           {
@@ -143,6 +148,9 @@ let g2_packet_handler s sock gconn p =
   
   | QKA ->
       H.host_queue_add active_udp_queue h (last_time ());
+      if (Queues.Queue.length active_udp_queue) > 100 then
+        let oh = H.host_queue_take active_udp_queue in
+        ();
       List.iter (fun c ->
           match c.g2_payload with
             QKA_QK key -> s.server_query_key <- UdpQueryKey key
@@ -154,8 +162,8 @@ let g2_packet_handler s sock gconn p =
             match ss.search_search with
             | UserSearch (_,words,xml_query) ->
                 server_send_query ss.search_uid words xml_query NoConnection s
-            | FileWordSearch (_,words) -> ()
-(*                server_send_query ss.search_uid words NoConnection s *)
+(*            | FileWordSearch (_,words) -> ()
+(*                server_send_query ss.search_uid words NoConnection s *) *)
             | FileUidSearch (file, uid) ->
                 server_ask_uid NoConnection s ss.search_uid uid file.file_name
       ) searches_by_uid;
@@ -231,7 +239,7 @@ let g2_packet_handler s sock gconn p =
             in
             q
           in
-          Array.iter (fun file ->
+          List.iter (fun (file,_) ->
               files := file :: !files
           ) ( CommonUploads.query q)
         end;
@@ -240,7 +248,7 @@ let g2_packet_handler s sock gconn p =
         server_send sock s (packet (QH2 (0,md4))
           ( 
             (packet (QH2_GU !!client_uid) []) ::
-            (packet (QH2_NA (h.host_addr, h.host_port)) []) ::
+            (packet (QH2_NA (Ip.ip_of_addr h.host_addr, h.host_port)) []) ::
             (packet (QH2_V "MLDK") []) ::
             (packet QH2_UPRO [packet  (QH2_UPRO_NICK (local_login ())) []]) ::
             (List.map (fun sh ->
@@ -263,7 +271,7 @@ packet QH2_H (
                     !uids
                   )
 *)
-                  
+                  let info = IndexedSharedFiles.get_result sh.shared_info in
                   packet QH2_H (
                     (packet (QH2_H_DN (
                           Filename.basename sh.shared_codedname)) []) ::
@@ -271,7 +279,7 @@ packet QH2_H (
                     (packet (QH2_H_G 1) []) :: (* meaning ??? *)
                     (List.map (fun uid ->
                           packet (QH2_H_URN uid) []  
-                      ) sh.shared_uids)
+                      ) info.shared_uids)
                   )
               ) !files)
           ))
@@ -282,7 +290,7 @@ packet QH2_H (
           match c.g2_payload with
             KHL_NH (ip,port) 
           | KHL_CH ((ip,port),_) ->
-              let h = H.new_host ip port true in ()
+              let h = H.new_host (Ip.addr_of_ip ip) port Ultrapeer in ()
           | _ -> ()
       ) p.g2_children;
       let children = ref [] in
@@ -293,7 +301,7 @@ packet QH2_H (
               Connected _ ->
                 let p = packet 
                     (KHL_CH 
-                      ((h.host_addr, h.host_port), int64_time ()))
+                      ((Ip.ip_of_addr h.host_addr, h.host_port), int64_time ()))
                   [
                     (packet (KHL_CH_V s.server_vendor) [])
                   ] in
@@ -316,8 +324,8 @@ packet QH2_H (
       List.iter (fun c ->
           match c.g2_payload with
           | QA_D ((ip,port),_) ->
-              let h = H.new_host ip port true in
-              h.host_connected <- last_time ();
+              let h = H.new_host (Ip.addr_of_ip ip) port Ultrapeer in
+              H.connected h;
               begin
                 match ss with
                   None -> ()
@@ -326,8 +334,8 @@ packet QH2_H (
               end
 (* These ones have not been searched yet *)
           | QA_S ((ip,port),_) -> 
-              let h = H.new_host ip port true in
-              h.host_connected <- last_time ();
+              let h = H.new_host (Ip.addr_of_ip ip) port Ultrapeer in
+              H.connected h;
           
           
           | _ -> ()
@@ -375,7 +383,8 @@ XML ("audios",
           let s = Hashtbl.find searches_by_uid suid in
           Some s with
           _ -> 
-            lprintf "***** No Corresponding Search ****\n";
+            if !verbose then
+              lprintf "***** No Corresponding Search ****\n";
             None 
       in
       let user_nick = ref "" in
@@ -421,8 +430,9 @@ XML ("audios",
                   let xml = Xml.xml_of (Xml.parse_string xml) in
                   xml_info := Some xml
                 with e ->
-                    lprintf "Exception %s while parsing: \n%s\n"
-                      (Printexc2.to_string e) xml
+                    if !verbose_hidden_errors then
+                      lprintf "Exception %s while parsing: \n%s\n"
+                        (Printexc2.to_string e) xml
               
               end
           | _ -> ()              
@@ -438,11 +448,13 @@ XML ("audios",
                     let (file_type, tags, _) = Xml.xml_of file in
                     (urn, size, name, url, 
                       List.map (fun (tag, v) ->
-                          string_tag tag v) tags)
+                          string_tag (g2_tag_of_name tag) v) 
+                      tags)
                 ) user_files files
               end else begin
-                lprintf "ERROR: Not enough XML entries %d/%d\n"
-                  (List.length files) (List.length user_files);
+                if !verbose_hidden_errors then
+                  lprintf "ERROR: Not enough XML entries %d/%d\n"
+                    (List.length files) (List.length user_files);
                 user_files 
               end
             
@@ -469,7 +481,7 @@ XML ("audios",
         (match !user_vendor with Some v -> user.user_vendor <- v | _ -> ());
         user.user_uid <- !user_uid;
         user.user_nick <- !user_nick; 
-        user.user_gnutella2 <- true;
+(*        user.user_gnutella2 <- true; *)
         user
       in
       
@@ -482,7 +494,8 @@ XML ("audios",
             | Some uid ->
                 Printf.sprintf "/uri-res/N2R?%s" (Uid.to_string uid)
           in
-          
+
+          (*
           (match size with
               None -> ()
             | Some size ->
@@ -493,14 +506,16 @@ XML ("audios",
                   let c = update_client user in
                   add_download file c (FileByUrl url)
                 with _ -> ());
+*)
           
           (match urn with
               None -> ()
             | Some uid ->
                 try
                   let file = Hashtbl.find files_by_uid uid in
-                  lprintf "++++++++++++ RECOVER FILE BY UID %s +++++++++++\n" 
-                    file.file_name; 
+                  if !verbose then
+                    lprintf "++++++++++++ RECOVER FILE BY UID %s +++++++++++\n" 
+                  file.file_name;
                   
                   (match size with
                       None -> ()
@@ -545,12 +560,13 @@ file_must_update file;
 
           
 let udp_packet_handler ip port msg = 
-  let h = H.new_host ip port true in
-  h.host_connected <- last_time ();
+  let addr = Ip.addr_of_ip ip in
+  let h = H.new_host addr port Ultrapeer in
+  H.connected h;
 (*  if !verbose_udp then
     lprintf "Received UDP packet from %s:%d: \n%s\n" 
       (Ip.to_string ip) port (Print.print msg);*)
-  let s = new_server ip port in
+  let s = new_server addr port in
   s.server_connected <- int64_time ();
   g2_packet_handler s NoConnection () msg
   (*
@@ -583,11 +599,17 @@ let init s sock gconn =
 (* A good session: PI, KHL, LNI *)
   
     
-let udp_handler ip port buf =
+let udp_client_handler ip port buf =
   if String.length buf > 3 && String.sub buf 0 3 = "GND" then
     try
       udp_packet_handler ip port
         (parse_udp_packet ip port buf)
     with AckPacket | FragmentedPacket -> ()
   else
-    lprintf "Unexpected UDP packet: \n%s\n" (String.escaped buf)
+    if !verbose then
+      lprintf "Unexpected UDP packet: \n%s\n" (String.escaped buf)
+
+      
+let update_shared_files () = ()
+let declare_word _ = new_shared_words := true
+  
