@@ -41,16 +41,13 @@ open SlskTypes
     
 open CommonNetwork
   
-let network = new_network "Soulseek"  
+let network = new_network "SLSK" "Soulseek"  
     [ 
     NetworkHasServers; 
     NetworkHasSearch;
     NetworkHasRooms;
     NetworkHasChat;
   ]
-    (fun _ -> !!network_options_prefix)
-  (fun _ -> !!commit_in_subdir)
-(*    network_options_prefix commit_in_subdir *)
   
 let connection_manager = network.network_connection_manager      
   
@@ -115,7 +112,7 @@ let new_server addr port=
           server_server = server_impl;
           server_name = "<unknown>";
           server_addr = addr;
-          server_nusers = 0;
+          server_nusers = Int64.zero;
           server_info = "";
           server_connection_control = new_connection_control ();
           server_sock = NoConnection;
@@ -259,7 +256,7 @@ let new_file file_id name file_size =
       let file_temp = Filename.concat !!temp_directory 
           (Printf.sprintf "SK-%s" (Md4.to_string file_id)) in
       let current_size = try
-          Unix32.getsize file_temp
+          Unix32.getsize file_temp true
         with e ->
             lprintf "Exception %s in current_size\n" (Printexc2.to_string e); 
             Int64.zero
