@@ -129,6 +129,7 @@ let _ =
         P.server_network = network.network_num;
         P.server_addr = s.server_addr;
         P.server_port = s.server_port;
+        P.server_realport = 0;
         P.server_score = 0;
         P.server_tags = [];
         P.server_nusers = s.server_nusers;
@@ -217,6 +218,7 @@ let _ =
         P.file_last_seen = BasicSocket.last_time ();
         P.file_priority = file_priority (as_file file.file_file);
         P.file_uids = [];
+        P.file_sub_files = [];
       }    
   );
   (*
@@ -264,10 +266,12 @@ let _ =
         P.client_chat_port = 0 ;
         P.client_connect_time = last_time ();
         P.client_software = "";
+        P.client_release = "";
         P.client_emulemod = "";
         P.client_downloaded = zero;
         P.client_uploaded = zero;
         P.client_upload = None;
+	P.client_sui_verified = None;
 (*    client_sock_addr = ""; *)
       }
   );
