@@ -20,12 +20,17 @@
 
 open Autoconf
 
+external format_int: string -> int -> string = "caml_format_int"
+external format_int32: string -> int32 -> string = "caml_int32_format"
+external format_nativeint: string -> nativeint -> string = "caml_nativeint_format"
+external format_int64: string -> int64 -> string = "caml_int64_format"
+external format_float: string -> float -> string = "caml_format_float"
 
-external format_int: string -> int -> string = "format_int"
-external format_int32: string -> int32 -> string = "int32_format"
-external format_nativeint: string -> nativeint -> string = "nativeint_format"
-external format_int64: string -> int64 -> string = "int64_format"
-external format_float: string -> float -> string = "format_float"
+let log_time () =
+let t = Unix.localtime (Unix.time ()) in
+  let { Unix.tm_year = tm_year; Unix.tm_mon = tm_mon; Unix.tm_mday = tm_mday;
+        Unix.tm_hour = tm_hour; Unix.tm_min = tm_min; Unix.tm_sec = tm_sec } = t in
+    Printf.sprintf "%4d/%02d/%02d %02d:%02d:%02d " (tm_year+1900) (tm_mon+1) tm_mday tm_hour tm_min tm_sec
 
 let bad_format fmt pos =
   invalid_arg
@@ -49,7 +54,7 @@ let format_string format s =
     try parse_format false 1 with Failure _ -> bad_format format 0 in
   if String.length s < p then begin
     let res = String.make p ' ' in
-    if neg 
+    if neg
     then String.blit s 0 res 0 (String.length s)
     else String.blit s 0 res (p - String.length s) (String.length s);
     res
@@ -77,7 +82,7 @@ let extract_format fmt start stop widths =
       in fill_format start (List.rev widths)
 
 (* Decode a %format and act on it.
-   [fmt] is the printf format style, and [pos] points to a [%] character.  
+   [fmt] is the printf format style, and [pos] points to a [%] character.
    After consuming the appropriate number of arguments and formatting
    them, one of the three continuations is called:
    [cont_s] for outputting a string (args: string, next pos)
@@ -91,7 +96,7 @@ let extract_format fmt start stop widths =
    rely on the fact that we'll get a "nul" character if we access
    one past the end of the string.  These "nul" characters are then
    caught by the [_ -> bad_format] clauses below.
-   Don't do this at home, kids. *) 
+   Don't do this at home, kids. *)
 
 let scan_format fmt pos cont_s cont_a cont_t =
   let rec scan_flags widths i =
@@ -163,7 +168,7 @@ let scan_format fmt pos cont_s cont_a cont_t =
     | _ ->
         bad_format fmt pos
   in scan_flags [] (pos + 1)
-  
+
 let cprintf kont fmt =
   let fmt = (Obj.magic fmt : string) in
   let len = String.length fmt in
@@ -171,7 +176,7 @@ let cprintf kont fmt =
   let rec doprn i =
     if i >= len then begin
       let res = Buffer.contents dest in
-      Buffer.clear dest;  (* just in case kprintf is partially applied *)
+      Buffer.reset dest;  (* just in case kprintf is partially applied *)
       Obj.magic (kont res)
     end else
     match String.unsafe_get fmt i with
@@ -184,17 +189,20 @@ let cprintf kont fmt =
   and cont_t printer i =
     Buffer.add_string dest (printer ()); doprn i
   in doprn 0
-  
-let lprintf_handler = ref (fun s -> 
-      Printf.printf "Message [%s] discared\n" s;
-      print_newline ();
+
+let lprintf_handler = ref (fun s ->
+      Printf.printf "Message [%s] discarded\n" s;
   )
-  
-let lprintf fmt = 
+
+let lprintf fmt =
   cprintf (fun s -> try !lprintf_handler s with _ -> ())
   (fmt : ('a,unit, unit) format )
-  
-let lprintf_nl fmt = 
+
+let lprintf_nl fmt =
+  cprintf (fun s -> try !lprintf_handler ((Printf.sprintf "%s" (log_time ()))^s^"\n") with _ -> ())
+  (fmt : ('a,unit, unit) format )
+
+let lprintf_nl2 fmt =
   cprintf (fun s -> try !lprintf_handler (s^"\n") with _ -> ())
   (fmt : ('a,unit, unit) format )
 
@@ -202,19 +210,19 @@ let lprint_newline () = lprintf "\n"
 let lprint_char c = lprintf "%c" c
 let lprint_int c = lprintf "%d" c
 let lprint_string c = lprintf "%s" c
-  
+
 let set_lprintf_handler f =
   lprintf_handler := f
-  
+
 let lprintf_max_size = ref 100
 let lprintf_size = ref 0
 let lprintf_fifo = Fifo.create ()
 let lprintf_to_stdout = ref true
 
 let lprintf_output = ref (Some stderr)
-  
+
 let _ =
-  set_lprintf_handler (fun s ->       
+  set_lprintf_handler (fun s ->
       match !lprintf_output with
         Some out when !lprintf_to_stdout ->
           Printf.fprintf out "%s" s; flush out
@@ -223,14 +231,14 @@ let _ =
             ignore (Fifo.take lprintf_fifo)
           else
             incr lprintf_size;
-          Fifo.put lprintf_fifo s 
+          Fifo.put lprintf_fifo s
   )
 
 let detach () =
   match !lprintf_output with
     Some oc when oc == Pervasives.stdout -> lprintf_output := None
   | _ -> ()
-  
+
 let close_log () =
   lprintf_to_stdout := false;
   match !lprintf_output with
@@ -240,7 +248,6 @@ let close_log () =
         close_out oc;
       lprintf_output := None
 
-        
 let log_to_file oc =
   close_log ();
   lprintf_output := Some oc;
@@ -254,6 +261,124 @@ let log_to_buffer buf =
       Buffer.add_string buf s
     done
   with _ -> ()
-      
+
 let set_logging b =
   lprintf_to_stdout := b
+
+(* html_mods *)
+
+let html_mods_commands buf n c l =
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\"
+    name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0
+    width=\\\"100pc\\\"\\>\\<tbody\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (w,x,y,z)  ->
+     (* Class Title Onclick Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\"
+     title=\\\"%s\\\" onMouseOver=\\\"mOvr(this,'mOvr1');\\\" onMouseOut=\\\"mOut(this);\\\"
+     onClick=\\\"%s\\\" \\>%s\\</td\\>"
+     w x y z;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</tbody\\>\\</table\\>\\</div\\>"
+
+(*
+  html_mods_commands buf "commandsTable" "commands" [
+    ( "bu bbig", "Extend search to more servers and view results", "mSub('output','vr');", "Extend search" ) ;
+*)
+
+
+let html_mods_table_header buf n c l =
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>"
+    c n n c;
+    if List.length l > 0 then begin
+        Printf.bprintf buf "\\<tr\\>";
+        List.iter (fun (w,x,y,z)  ->
+         (* Sort Class Title Value *)
+         Printf.bprintf buf "\\<td onClick=\\\"_tabSort(this,%s);\\\" class=\\\"%s\\\" title=\\\"%s\\\"\\>%s\\</td\\>"
+         w x y z;
+        ) l;
+        Printf.bprintf buf "\\</tr\\>"
+      end
+(* Add colspan functionality to html_mods_table_header *)
+
+let html_mods_table_header_colspan buf n c l =
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (v,w,x,y,z)  ->
+     (* Sort Class Title Value *)
+     Printf.bprintf buf "\\<td colspan=%s onClick=\\\"_tabSort(this,%s);\\\" class=\\\"%s\\\" title=\\\"%s\\\"\\>%s\\</td\\>"
+     v w x y z;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>"
+
+let html_mods_table_no_header buf n c l =
+    (* 1 row * n cols *)
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_table_one_row buf n c l =
+    (* 1 row * n cols *)
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_table_one_col buf n c l =
+    (* n rows * 1 col *)
+    (* Name Class List *)
+    Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table id=\\\"%s\\\" name=\\\"%s\\\" class=\\\"%s\\\" cellspacing=0 cellpadding=0\\>\\<tr\\>"
+    c n n c;
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<tr\\>\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>\\</tr\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l;
+    Printf.bprintf buf "\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_td buf l =
+    (* List *)
+    List.iter (fun (t,c,d)  ->
+    (* Title Class Value *)
+     Printf.bprintf buf "\\<td class=\\\"%s\\\" %s\\>%s\\</td\\>"
+     c (if t <> "" then "title=\\\"" ^ t ^ "\\\"" else "") d;
+    ) l
+
+
+let html_mods_big_header_start buf c l =
+  Printf.bprintf buf "\\<div class=\\\"%s\\\"\\>\\<table class=main border=0 cellspacing=0 cellpadding=0\\>\\<tr\\>\\<td\\>" c;
+  Printf.bprintf buf "\\<table cellspacing=0 cellpadding=0 width=100%%\\>\\<tr\\>";
+  Printf.bprintf buf "\\<td width=100%%\\>\\</td\\>";
+  let len = List.length l in
+  let cnt = ref 0 in
+  List.iter (fun s ->
+    incr cnt;
+    Printf.bprintf buf "\\<td nowrap class=\\\"fbig%s\\\"\\>%s\\</td\\>" (if !cnt = len then " pr" else "") s;
+  ) l;
+  Printf.bprintf buf "\\</tr\\>\\</table\\>\\</td\\>\\</tr\\>\\<tr\\>\\<td\\>"
+
+let html_mods_big_header_end buf =
+  Printf.bprintf buf "\\</td\\>\\</tr\\>\\</table\\>\\</div\\>"
+
+let html_mods_counter = ref true
+
+let html_mods_cntr () =
+    html_mods_counter := not !html_mods_counter;
+    if !html_mods_counter then 1 else 2
+
+let html_mods_cntr_init () =
+  html_mods_counter := true
