@@ -54,15 +54,31 @@ let is_directory filename =
 let is_link filename =
   try let s = Unix.lstat filename in s.st_kind = S_LNK with _ -> false
 
+let chmod f o = 
+  try Unix.chmod f o with e -> lprintf_nl "warning: chmod failed on %s: %s" f (Printexc2.to_string e)
+
 let rec safe_mkdir dir =  
   if Sys.file_exists dir then begin
       if not (is_directory dir) then 
-        failwith (Printf.sprintf "%s not a directory" dir)
+        failwith (Printf.sprintf "%s already exists but is not a directory" dir)
+    end
+  else 
+  if is_link dir then
+    begin try
+      let dir = opendir dir in () 
+      with
+        Unix.Unix_error (EACCES, _, _) -> lprintf_nl "access denied for directory %s" dir; exit 73
+      | Unix.Unix_error (ENOENT, _, _) -> lprintf_nl "directory %s not found, orphaned link?" dir; exit 73
+      | e -> lprintf_nl "error %s for directory %s" (Printexc2.to_string e) dir; exit 73
     end
   else begin
       let predir = Filename.dirname dir in
       if predir <> dir then safe_mkdir predir;
-(*      if dir <> "." then *) Unix.mkdir dir 0o775
+      begin try
+        Unix.mkdir dir 0o775
+      with
+        e -> lprintf_nl "error %s for directory %s" (Printexc2.to_string e) dir; exit 73
+      end
     end    
     
     
@@ -101,7 +117,7 @@ let rename oldname newname =
   try Unix.rename oldname newname with
     Unix_error(EXDEV,_,_) as e ->
 (* renaming is not enough, we must COPY *)
-      lprintf "COPY %s TO %s\n" oldname newname; 
+      lprintf_nl "COPY %s TO %s" oldname newname; 
       let copied = ref false in
       try
         copy oldname newname; 
@@ -115,11 +131,14 @@ let rename oldname newname =
 external c_seek64 : Unix.file_descr -> int64 -> Unix.seek_command -> int64 =
   "unix_lseek_64" 
 external c_getsize64 : string -> int64 = "ml_getsize64"
-external c_ftruncate64 : Unix.file_descr -> int64 -> unit =
+external c_getfdsize64 : Unix.file_descr -> int64 = "ml_getfdsize64"
+external c_ftruncate64 : Unix.file_descr -> int64 -> bool -> unit =
   "mld_ftruncate_64"
 external c_getdtablesize : unit -> int = "ml_getdtablesize"
 external c_sizeofoff_t : unit -> int = "ml_sizeofoff_t"
 
+external endianness : unit -> string = "ml_check_endianness"
+
 let rec remove_all_directory dirname =
   let files = list_directory dirname in
   List.iter (fun file ->
@@ -129,4 +148,27 @@ let rec remove_all_directory dirname =
       else
         Sys.remove filename
   ) files;
-  Unix.rmdir dirname
\ No newline at end of file
+  Unix.rmdir dirname
+
+let random () =
+  let s = String.create 7 in
+  for i = 0 to 6 do
+    s.[i] <- char_of_int (97 + Random.int 26)
+  done;
+  s
+
+let rec can_write_to_directory dirname =
+  let temp_file = Filename.concat dirname "tmp_" ^ random () ^ "_mld.tmp" in
+  try
+    (let oc = open_out_gen [Open_creat; Open_wronly; Open_append] 0o600 temp_file in
+      output_string oc "accesstest";
+      close_out oc);
+    (try Sys.remove temp_file with _ -> ())
+  with
+    Sys_error s when s = temp_file ^ ": " ^ (Unix.error_message Unix.EACCES) ->
+      lprintf_nl "can not create files in directory %s, check rights..." dirname; exit 73
+  | Sys_error s when s = temp_file ^ ": " ^ (Unix.error_message Unix.ENOENT) ->
+      (try safe_mkdir dirname; can_write_to_directory dirname with _ ->
+        lprintf_nl "%s does not exist and can not be created, exiting..." dirname; exit 73)
+  | Sys_error s -> lprintf_nl "%s for directory %s" s dirname; exit 73
+  | e -> lprintf_nl "%s for directory %s" (Printexc2.to_string e) dirname; exit 73
