@@ -28,6 +28,15 @@ external inflate_end: stream -> unit = "camlzip_inflateEnd"
 external update_crc: int32 -> string -> int -> int -> int32
                    = "camlzip_update_crc32"
 
+external zlib_version : unit -> string = "camlzip_zlibversion"
+
+let zlib_version_num () =
+  begin
+    try
+      let s = Printf.sprintf "%s" (zlib_version ()) in s
+    with e -> ""
+  end
+
 let buffer_size = 1024
 
 let compress ?(level = 6) ?(header = true) refill flush =
@@ -80,6 +89,8 @@ let compress_string ?(level = 6) inbuf =
 
 (* header info from camlzip/gpl *)
 let gzip_string ?(level = 6) inbuf =
+  if String.length inbuf <= 0 then "" else
+  begin
   let zs = deflate_init level false in
   let out_crc = ref Int32.zero in
   let rec compr inpos outbuf outpos =
@@ -106,7 +117,7 @@ let gzip_string ?(level = 6) inbuf =
   let write_int32 wbuf n =
     let r = ref n in
     for i = 1 to 4 do
-      Buffer.add_char wbuf (char_of_int (Int32.to_int (Int32.logand !r (Int32.of_int 255))));
+      Buffer.add_char wbuf (char_of_int (Int32.to_int (Int32.logand !r 0xffl)));
       r := Int32.shift_right_logical !r 8
     done
   in
@@ -121,7 +132,7 @@ let gzip_string ?(level = 6) inbuf =
   write_int32 buf !out_crc;
   write_int32 buf (Int32.of_int (String.length inbuf));
   Buffer.contents buf
-
+  end
 
 let uncompress ?(header = true) refill flush =
   let inbuf = String.create buffer_size
