@@ -341,19 +341,15 @@ module Make(M: sig
     
     let file s =
       let digest = String.create hash_length in
-      let file_size = Unix32.getsize s in
+      let file_size = Unix32.getsize s false in
       unsafe_file digest s file_size;
       digest
     
     let digest_subfile fd pos len =
       let digest = String.create hash_length in
-      let (fd, pos, filename) = Unix32.fd_of_chunk fd pos len in
-      digest_subfile digest fd pos len;
-      (match filename with
-          None -> ()
-        | Some filename ->
-            Sys.remove filename
-      );
+      Unix32.apply_on_chunk fd pos len 
+        (fun fd pos ->
+          digest_subfile digest fd pos len);
       digest
     
     let create () =  String.create hash_length
@@ -391,8 +387,9 @@ module Make(M: sig
     
     
     let up s = int_of_char s.[0]
-    let up2 s = (int_of_char s.[0])*256+(int_of_char s.[1])
-    let up3 s = (int_of_char s.[0])*65536+(int_of_char s.[1])*256+(int_of_char s.[2])
+    let up2 s = ((int_of_char s.[0]) lsl 8) lor (int_of_char s.[1])
+    let up3 s = ((int_of_char s.[0]) lsl 16) lor 
+                ((int_of_char s.[1]) lsl 8) lor (int_of_char s.[2])
   
     let enabled = true
   end
@@ -473,11 +470,11 @@ module Tiger = Make(struct
         
       let unsafe_file digest filename = 
         Printf2.lprintf "Tiger.unsafe_file not implemented\n";
-        exit 2
+        exit 70
         
       let digest_subfile digest fd pos len = 
         Printf2.lprintf "Tiger.digest_subfile not implemented\n";
-        exit 2
+        exit 70
     
       module Base = Base32
         
@@ -493,14 +490,9 @@ module PreTigerTree = Make(struct
       
       let unsafe_file digest filename file_size = 
         let fd = Unix32.create_diskfile filename [Unix.O_RDONLY] 0o444 in
-        let (fd, pos, filename) = Unix32.fd_of_chunk fd Int64.zero file_size in
-        let digest = digest_subfile digest fd pos file_size in
-        (match filename with
-            None -> ()
-          | Some filename ->
-              Sys.remove filename
-        );
-        digest
+        Unix32.apply_on_chunk fd Int64.zero file_size 
+          (fun fd pos ->
+            digest_subfile digest fd pos file_size)
     
       module Base = Base32
         
