@@ -22,6 +22,7 @@
 
 #include <string.h> 
 #include <ctype.h>
+#include <caml/config.h>
 
 #define lseek XXXXXXXXX
 #define read XXXXXXXXX
@@ -126,8 +127,7 @@ value try_poll(value fdlist, value timeout) /* ML */
 /*        printf("SETTING %d TO %d\n", fd, nfds); */
         pfds[nfds] = v;
         nfds++;
-      } else
-        ;
+      } // else ;
     }
   }
 /*  printf("POLL: %d/%d\n", nfds, ufds_size); */
@@ -273,9 +273,8 @@ value ml_getdtablesize(value unit)
       
 #else
 
-      printf("Your shell allows %d file descriptors, but select only allows %d file descriptors\n", dtablesize, maxselectfds);
       maxfd = maxselectfds;
-      printf("Limit has been set to %d\n", maxfd);
+		  // printf("[Info] File descriptor limit: MIN(process: %d, select(): %d) = %d\n", dtablesize, maxselectfds, maxfd);
 
 #endif
   }
@@ -351,7 +350,7 @@ value ml_setsock_iptos_throughput(value sock_v)
 
 value ml_sizeofoff_t(value unit)
 {
-  return Val_int(sizeof(off_t));
+  return Val_int(sizeof(OFF_T));
 }
 
 /*******************************************************************
@@ -369,6 +368,20 @@ value ml_getsize64(value path)
   return copy_int64(os_getfilesize(String_val(path)));
 }
 
+/*******************************************************************
+
+
+                         ml_getfdsize64
+
+
+*******************************************************************/
+
+value ml_getfdsize64(value fd_v)
+{
+/*  int ret; */
+  OS_FD fd = Fd_val(fd_v);
+  return copy_int64(os_getfdsize(fd));
+}
 
 /*******************************************************************
 
@@ -379,12 +392,13 @@ value ml_getsize64(value path)
 *******************************************************************/
 
 #define ZEROS_LEN 1024
-value mld_ftruncate_64(value fd_v, value len_v)
+value mld_ftruncate_64(value fd_v, value len_v, value sparse)
 {
-  off_t len = Int64_val(len_v);
+  OFF_T len = Int64_val(len_v);
   OS_FD fd = Fd_val(fd_v);  
+	 int use_sparse = Bool_val(sparse);
 
-  os_ftruncate(fd, len);
+  os_ftruncate(fd, len, use_sparse);
     
   return Val_unit;
 }
@@ -486,17 +500,17 @@ unsigned char hash_buffer[HASH_BUFFER_LEN];
 value HASH_NAME##_unsafe64_fd (value digest_v, value fd_v, value pos_v, value len_v) \
 { \
   OS_FD fd = Fd_val(fd_v); \
-  off_t pos = Int64_val(pos_v); \
-  off_t len = Int64_val(len_v); \
+  OFF_T pos = Int64_val(pos_v); \
+  OFF_T len = Int64_val(len_v); \
   unsigned char *digest = String_val(digest_v); \
   HASH_CONTEXT context; \
-  int nread; \
+  ssize_t nread; \
  \
   HASH_INIT (&context); \
   os_lseek(fd, pos, SEEK_SET); \
  \
   while (len!=0){ \
-    int max_nread = HASH_BUFFER_LEN > len ? len : HASH_BUFFER_LEN; \
+    size_t max_nread = HASH_BUFFER_LEN > len ? len : HASH_BUFFER_LEN; \
  \
     nread = os_read (fd, hash_buffer, max_nread); \
  \
@@ -538,7 +552,7 @@ value HASH_NAME##_unsafe_file (value digest_v, value filename_v, value file_size
   unsigned char *digest = String_val(digest_v); \
   FILE *file; \
   HASH_CONTEXT context; \
-  int len; \
+  size_t len; \
  \
   if ((file = fopen (filename, "rb")) == NULL) \
     raise_not_found(); \
@@ -572,19 +586,19 @@ ML_HASH(md4,MD4_CTX,MD4Init,MD4Update,md4_finish)
 *******************************************************************/
 #include "tiger.h"
 
-static void tiger_tree_fd(OS_FD fd, int len, int pos, int block_size, char *digest)
+static void tiger_tree_fd(OS_FD fd, size_t len, size_t pos, size_t block_size, char *digest)
 {
   static char tiger_buffer[BLOCK_SIZE+1];
   if(block_size == BLOCK_SIZE){
-    int length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
+    size_t length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
     char *s = tiger_buffer+1;
-    int toread = length;
+    size_t toread = length;
     char *curs = s;
       while (toread!=0){
       int max_nread = toread;
 /* HASH_BUFFER_LEN > toread ? toread : HASH_BUFFER_LEN; */
 
-      int nread = os_read (fd, curs, max_nread);
+      ssize_t nread = os_read (fd, curs, max_nread);
 
         if(nread <= 0) {
         unix_error(errno, "tiger_safe_fd: Read", Nothing);
@@ -610,8 +624,8 @@ static void tiger_tree_fd(OS_FD fd, int len, int pos, int block_size, char *dige
 value tigertree_unsafe64_fd (value digest_v, value fd_v, value pos_v, value len_v)
 {
   OS_FD fd = Fd_val(fd_v);
-  off_t pos = Int64_val(pos_v);
-  off_t len = Int64_val(len_v);
+  OFF_T pos = Int64_val(pos_v);
+  OFF_T len = Int64_val(len_v);
   unsigned char *digest = String_val(digest_v);
 /*  int nread; */
 
@@ -662,7 +676,7 @@ value ml_setlcnumeric(value no)
 
 // #include "socketaddr.h"
 #ifndef _WIN32
-#include <sys/types.h>
+// #include <sys/types.h>
 #include <netdb.h>
 #endif
 
@@ -831,6 +845,8 @@ static void * hasher_thread(void * arg)
 {
   struct timeval now;
   struct timespec timeout;
+
+#if !defined(PTW32_STATIC_LIB)	
   sigset_t mask;
 
   /* Block all signals so that we don't try to execute a Caml signal handler */
@@ -838,6 +854,7 @@ static void * hasher_thread(void * arg)
   pthread_sigmask(SIG_BLOCK, &mask, NULL);
   
   nice(19);
+#endif
   
   pthread_mutex_lock(&mutex);
 
@@ -899,3 +916,380 @@ value ml_has_pthread(value unit)
 
 #endif
 
+#include <caml/custom.h>
+#include <caml/callback.h>
+
+#if defined(__MINGW32__)
+/* taken from www.greatchief.plus.com/gpc/os-hacks.h */
+#define _fullpath(res,path,size) \
+  (GetFullPathNameW ((path), (size), (res), NULL) ? (res) : NULL)
+
+#define realpath(path,resolved_path) _fullpath(resolved_path, path, MAX_PATH)
+
+#define FAKED_BLOCK_SIZE 512 /* fake block size */
+
+/* linux-compatible values for fs type */
+#define MSDOS_SUPER_MAGIC     0x4d44
+#define NTFS_SUPER_MAGIC      0x5346544E
+
+struct statfs {
+   long    f_type;     /* type of filesystem (see below) */
+   long    f_bsize;    /* optimal transfer block size */
+   long    f_blocks;   /* total data blocks in file system */
+   long    f_bfree;    /* free blocks in fs */
+   long    f_bavail;   /* free blocks avail to non-superuser */
+   long    f_files;    /* total file nodes in file system */
+   long    f_ffree;    /* free file nodes in fs */
+   long    f_fsid;     /* file system id */
+   long    f_namelen;  /* maximum length of filenames */
+   long    f_spare[6]; /* spare for later */
+};
+
+static int statfs (const unsigned char *path, struct statfs *buf)
+  {
+    HINSTANCE h;
+    FARPROC f;
+    int retval = 0;
+    WCHAR tmp [MAX_PATH], resolved_path [MAX_PATH];
+    WCHAR * wpath=utf8_to_utf16(path);
+    realpath(wpath, resolved_path);
+    free(wpath);
+    if (!resolved_path)
+      retval = - 1;
+    else
+      {
+        /* check whether GetDiskFreeSpaceExA is supported */
+        h = LoadLibraryA ("kernel32.dll");
+        if (h)
+          f = GetProcAddress (h, "GetDiskFreeSpaceExW");
+        else
+          f = NULL;
+        if (f)
+          {
+            ULARGE_INTEGER bytes_free, bytes_total, bytes_free2;
+            if (!f (resolved_path, &bytes_free2, &bytes_total, &bytes_free))
+              {
+                errno = ENOENT;
+                retval = - 1;
+              }
+            else
+              {
+                buf -> f_bsize = FAKED_BLOCK_SIZE;
+                buf -> f_bfree = (bytes_free.QuadPart) / FAKED_BLOCK_SIZE;
+                buf -> f_files = buf -> f_blocks = (bytes_total.QuadPart) / FAKED_BLOCK_SIZE;
+                buf -> f_ffree = buf -> f_bavail = (bytes_free2.QuadPart) / FAKED_BLOCK_SIZE;
+              }
+          }
+        else
+          {
+            DWORD sectors_per_cluster, bytes_per_sector;
+            if (h) FreeLibrary (h);
+            if (!GetDiskFreeSpaceW (resolved_path, &sectors_per_cluster,
+                   &bytes_per_sector, &buf -> f_bavail, &buf -> f_blocks))
+              {
+                errno = ENOENT;
+                retval = - 1;
+              }
+            else
+              {
+                buf -> f_bsize = sectors_per_cluster * bytes_per_sector;
+                buf -> f_files = buf -> f_blocks;
+                buf -> f_ffree = buf -> f_bavail;
+                buf -> f_bfree = buf -> f_bavail;
+              }
+          }
+        if (h) FreeLibrary (h);
+      }
+
+    /* get the FS volume information */
+    if (wcsspn (L":", resolved_path) > 0) resolved_path [3] = '\0'; /* we want only the root */    
+    if (GetVolumeInformationW (resolved_path, NULL, 0, &buf -> f_fsid, &buf -> f_namelen, NULL, tmp, MAX_PATH))
+    /* http://msdn.microsoft.com/library/default.asp?url=/library/en-us/fileio/fs/getvolumeinformation.asp */
+     {
+     	if (_wcsicmp (L"NTFS", tmp) == 0)
+     	 {
+     	   buf -> f_type = NTFS_SUPER_MAGIC;
+     	 }
+     	else
+     	 {
+     	   buf -> f_type = MSDOS_SUPER_MAGIC;
+     	 }
+     }
+    else
+     {
+       errno = ENOENT;
+       retval = - 1;
+     }
+    return retval;
+}
+
+static value
+copy_statfs (struct statfs *buf)
+{
+  CAMLparam0 ();
+  CAMLlocal2 (bufv, v);
+  bufv = caml_alloc (11, 0);
+  v = copy_int64 (buf->f_type); caml_modify (&Field (bufv, 0), v);
+  v = copy_int64 (buf->f_bsize); caml_modify (&Field (bufv, 1), v);
+  v = copy_int64 (buf->f_blocks); caml_modify (&Field (bufv, 2), v);
+  v = copy_int64 (buf->f_bfree); caml_modify (&Field (bufv, 3), v);
+  v = copy_int64 (buf->f_bavail); caml_modify (&Field (bufv, 4), v);
+  v = copy_int64 (buf->f_files); caml_modify (&Field (bufv, 5), v);
+  v = copy_int64 (buf->f_ffree); caml_modify (&Field (bufv, 6), v);
+  v = copy_int64 (buf->f_namelen); caml_modify (&Field (bufv, 8), v);
+  v = copy_string ("-1"); caml_modify (&Field (bufv, 9), v);
+  v = copy_int64 (-1); caml_modify (&Field (bufv, 10), v);
+  CAMLreturn (bufv);
+}
+
+CAMLprim value
+statfs_statfs (value pathv)
+{
+  CAMLparam1 (pathv);
+  CAMLlocal1 (bufv);
+  const unsigned char *path = String_val (pathv);
+  struct statfs buf;
+  if (statfs (path, &buf) == -1)
+    raise_constant(*(value *)caml_named_value("error"));
+  bufv = copy_statfs (&buf);
+  CAMLreturn (bufv);
+}
+
+#else /* defined(__MINGW32__) */
+
+#if (defined HAVE_SYS_PARAM_H && HAVE_SYS_MOUNT_H)
+#  include <sys/param.h>
+#  include <sys/mount.h>
+#  define HAVE_STATS 1
+#endif
+#ifdef HAVE_SYS_VFS_H
+#  include <sys/vfs.h>
+#  define HAVE_STATS 1
+#endif
+
+#ifdef HAVE_STATS
+static value
+#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__)
+copy_statfs (struct statvfs *buf)
+#else
+copy_statfs (struct statfs *buf)
+#endif
+{
+  CAMLparam0 ();
+  CAMLlocal2 (bufv, v);
+  bufv = caml_alloc (11, 0);
+#if ((defined (sun) || defined (__sun__))) || (defined(__FreeBSD__) && __FreeBSD_version >= 503001) || defined(__OpenBSD__) || defined(__NetBSD__)
+  v = copy_int64 (-1); caml_modify (&Field (bufv, 0), v);
+#else
+  v = copy_int64 (buf->f_type); caml_modify (&Field (bufv, 0), v);
+#endif /* ((defined (sun) || defined (__sun__))) || (defined(__FreeBSD__) && __FreeBSD_version >= 503001) || defined(__OpenBSD__) || defined(__NetBSD__) */
+  v = copy_int64 (buf->f_bsize); caml_modify (&Field (bufv, 1), v);
+  v = copy_int64 (buf->f_blocks); caml_modify (&Field (bufv, 2), v);
+  v = copy_int64 (buf->f_bfree); caml_modify (&Field (bufv, 3), v);
+  v = copy_int64 (buf->f_bavail); caml_modify (&Field (bufv, 4), v);
+  v = copy_int64 (buf->f_files); caml_modify (&Field (bufv, 5), v);
+  v = copy_int64 (buf->f_ffree); caml_modify (&Field (bufv, 6), v);
+#if ((defined (sun) || defined (__sun__))) || defined (__hpux__)
+  v = copy_int64 (-1); caml_modify (&Field (bufv, 7), v);
+  v = copy_int64 (buf->f_namemax); caml_modify (&Field (bufv, 8), v);
+  v = copy_string (buf->f_basetype); caml_modify (&Field (bufv, 9), v);
+  v = copy_int64 (buf->f_frsize); caml_modify (&Field (bufv, 10), v);
+#else
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__)
+#  if defined(__OpenBSD__) || defined(__NetBSD__) || (defined(__FreeBSD__) && __FreeBSD_version < 502000)
+#    include <sys/syslimits.h>
+     v = copy_int64 (NAME_MAX); caml_modify (&Field (bufv, 8), v);
+#  else
+#    if defined(__APPLE__)
+#      include <unistd.h>
+       v = copy_int64 (_PC_NAME_MAX); caml_modify (&Field (bufv ,8 ), v);
+#    else
+       v = copy_int64 (buf->f_namemax); caml_modify (&Field (bufv, 8), v);
+#    endif /* (__APPLE__) */
+#  endif /* (__OpenBSD__) || defined(__NetBSD__) || (defined(__FreeBSD__) && __FreeBSD_version < 502000) */
+  v = copy_string (buf->f_fstypename); caml_modify (&Field (bufv, 9), v);
+#else
+  v = copy_int64 (buf->f_namelen); caml_modify (&Field (bufv, 8), v);
+  v = copy_string ("-1"); caml_modify (&Field (bufv, 9), v);
+#endif /* defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) */
+  caml_modify (&Field (bufv, 7), Val_unit);
+  v = copy_int64 (-1); caml_modify (&Field (bufv, 10), v);
+#endif /*  ((defined (sun) || defined (__sun__))) || defined (__hpux__) */
+  CAMLreturn (bufv);
+}
+#endif
+
+CAMLprim value
+statfs_statfs (value pathv)
+{
+#ifdef HAVE_STATS
+  CAMLparam1 (pathv);
+  CAMLlocal1 (bufv);
+  const char *path = String_val (pathv);
+#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__)
+  struct statvfs buf;
+  if (statvfs (path, &buf) == -1)
+#else
+  struct statfs buf;
+  if (statfs (path, &buf) == -1)
+#endif
+    raise_constant(*(value *)caml_named_value("error"));
+  bufv = copy_statfs (&buf);
+  CAMLreturn (bufv);
+#else
+  raise_constant(*(value *)caml_named_value("not supported"));
+#endif
+}
+#endif /* defined(__MINGW32__) */
+
+
+#if defined(__MINGW32__)
+static HWND myHWND = NULL;
+#endif
+
+value
+external_start (value version)
+{
+
+// Disable close button on console
+#if defined(__MINGW32__)
+  char *buf = "[MLDonkey TitleSearch]\0"; // if multiple instances
+  char *title = String_val (version);
+  SetConsoleTitle((LPCTSTR)buf);
+  myHWND = FindWindowEx(NULL, NULL, NULL, (LPCTSTR)buf);
+  SetConsoleTitle((LPCTSTR)title);
+
+  if (myHWND != NULL) {
+    HMENU hmenu = GetSystemMenu(myHWND, FALSE);
+    MENUITEMINFO CloseItem;
+    CloseItem.cbSize = sizeof(MENUITEMINFO);
+    CloseItem.fMask = MIIM_ID;
+    CloseItem.wID = SC_MINIMIZE+1;
+    EnableMenuItem(hmenu, SC_CLOSE, MF_BYCOMMAND | MF_GRAYED);
+    SetMenuItemInfo(hmenu, SC_CLOSE, FALSE, &CloseItem);
+    DrawMenuBar(myHWND);
+  }
+#endif
+
+#if defined(HAVE_PTHREAD) && defined(PTW32_STATIC_LIB)
+	pthread_win32_process_attach_np();
+#endif
+	return Val_unit;
+
+}
+
+// Crypto 
+void crypto_exit();
+void createKey(char buf[]);
+unsigned long loadKey(char privateKeyBase64[], char buf[]);
+int createSignature(byte *buf, int maxLen, byte *key, int keyLen, uint32_t cInt, uint8_t ipType, uint32_t ip);
+int verifySignature(byte *key, int keyLen, byte *sig, int sigLen, uint32_t cInt, uint8_t ipType, uint32_t ip);
+
+value
+external_exit (void) 
+{
+// Revert console system menu
+#if defined(__MINGW32__)
+  if (myHWND != NULL) {
+    HMENU hmenu = GetSystemMenu(myHWND, FALSE);
+    MENUITEMINFO CloseItem;
+    CloseItem.cbSize = sizeof(MENUITEMINFO);
+    CloseItem.fMask = MIIM_ID;
+    CloseItem.wID = SC_CLOSE;
+    SetMenuItemInfo(hmenu, SC_MINIMIZE+1, FALSE, &CloseItem);
+    EnableMenuItem(hmenu, SC_CLOSE, MF_BYCOMMAND | MF_ENABLED);
+    DrawMenuBar(myHWND);
+  }
+#endif
+
+#if defined(HAVE_PTHREAD) && defined(PTW32_STATIC_LIB)
+	pthread_win32_process_detach_np();
+#endif
+
+	crypto_exit();
+	return Val_unit;
+}
+
+value
+ml_uname(void) {
+	char buf[4096];
+	buf[0] = '\0';
+	os_uname(buf);
+	return caml_copy_string(buf);
+}
+
+
+// return private key Base64Encoded
+value 
+ml_createKey() {
+	char buf[4096];
+	createKey(buf);
+	return caml_copy_string(buf);
+}
+
+// return public key
+value 
+ml_loadKey(value privatekey) {
+  char *s = String_val(privatekey);
+	char buf[4096];
+	unsigned long len =	loadKey(s, buf);
+
+  value res;
+  res = caml_alloc_string(len);
+  memmove(String_val(res), buf, len);
+
+	return res;
+}
+
+value
+ml_createSignature(value m_key, value m_keyLen, value m_cInt, value m_ipType, value m_ip) {
+
+	byte *key = (byte*) String_val(m_key);
+	int keyLen = Int_val(m_keyLen);
+	uint32_t cInt = Int64_val(m_cInt);
+	int ipType = Int_val(m_ipType);
+	uint32_t ip = Int64_val(m_ip);
+
+	byte buf[4096];
+
+	int len = createSignature(buf, 200, key, keyLen, cInt, ipType, ip);
+
+  value res;
+  res = caml_alloc_string(len);
+  memmove(String_val(res), buf, len);
+
+	return res;
+}
+
+value 
+ml_verifySignature(value m_key, value m_keyLen, value m_sig, value m_sigLen, value m_cInt, value m_ipType, value m_ip) {
+
+	byte* key = (byte*) String_val(m_key);
+	int keyLen = Int_val(m_keyLen);
+	byte* sig = (byte*) String_val(m_sig);
+	int sigLen = Int_val(m_sigLen);
+	uint32_t cInt = Int64_val(m_cInt);
+	int ipType = Int_val(m_ipType);
+	uint32_t ip = Int64_val(m_ip);
+	
+	return Val_bool( 
+			verifySignature(key, keyLen, sig, sigLen, cInt, ipType, ip)
+	);
+}
+
+value
+ml_verifySignature_bytecode(value *argv, int argn) {
+	return ml_verifySignature(argv[0], argv[1], argv[2], argv[3], argv[4], argv[5], argv[6]);
+}
+
+value ml_check_endianness(void)
+{
+  CAMLparam0 ();
+  CAMLlocal1 (v);
+#ifdef ARCH_BIG_ENDIAN
+  v = copy_string ("big endian");
+#else
+  v = copy_string ("little endian");
+#endif
+  CAMLreturn (v);
+}
