@@ -18,11 +18,7 @@
 */
 
 #include "tiger.h"
-
-/* Big endian:                                         */
-#if !(defined(__alpha)||defined(__i386__)||defined(__vax__))
-#define BIG_ENDIAN
-#endif
+#include <caml/config.h>
 
 /* The following macro denotes that an optimization    */
 /* for Alpha is required. It is used only for          */
@@ -176,7 +172,8 @@ void tiger_compress(word64 *str, word64 state[3])
 static void static_tiger(word64 *str, word64 length, word64 res[3])
 {
   register word64 i, j;
-  unsigned char temp[64];
+  word64 temp_[64 / 8];
+  unsigned char *temp = (unsigned char *) temp_;
 
   res[0]=0x0123456789ABCDEFLL;
   res[1]=0xFEDCBA9876543210LL;
@@ -192,7 +189,7 @@ static void static_tiger(word64 *str, word64 length, word64 res[3])
 
   for(i=length; i>=64; i-=64)
     {
-#ifdef BIG_ENDIAN
+#ifdef ARCH_BIG_ENDIAN
       for(j=0; j<64; j++)
 	temp[j^7] = ((byte*)str)[j];
       tiger_compress(((word64*)temp), res);
@@ -202,7 +199,7 @@ static void static_tiger(word64 *str, word64 length, word64 res[3])
       str += 8;
     }
 
-#ifdef BIG_ENDIAN
+#ifdef ARCH_BIG_ENDIAN
   for(j=0; j<i; j++)
     temp[j^7] = ((byte*)str)[j];
 
@@ -750,54 +747,53 @@ word64 table[4*256] = {
 /* If these headers are placed at the beginning of the file, the computation
  * is incorrect... */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdio.h>
-
-#if 0
-#define MAX_TIGER_CHUNK_SIZE 8192
-static char tiger_buffer[MAX_TIGER_CHUNK_SIZE];
-
-void tiger_hash(char prefix, char *s, int len, unsigned char *digest)
+void swap_digest(unsigned char *digest)
 {
-   
-         if(len>= MAX_TIGER_CHUNK_SIZE){
-	                printf("tiger_hash: error chunk too big\n");
-	                exit(2);
-     }
-   
-   
-   tiger_buffer[0] = prefix;
-   memcpy(tiger_buffer+1, s, len);
-   static_tiger((word64*)tiger_buffer, (len+1), (word64*) (digest));
+  #ifdef ARCH_BIG_ENDIAN
+  {
+    int i,j; 
+    for(i=0; i<3; i++){
+      for(j=0;j<4;++j){
+        char s0 = digest[i*8+j];
+        char s1 = digest[i*8+7-j];
+        digest[i*8+j] = s1;
+        digest[i*8+7-j] = s0;
+      }
+    }
+  }
+  #endif
+  
 }
-#else
+
+#include <string.h>
+
+char hexa(int i)
+{
+  if(i>9) return 'A'+i-10; else return '0'+i;
+}
+
 #define MAX_TIGER_CHUNK_SIZE 1024
 static word64 tiger_buffer[MAX_TIGER_CHUNK_SIZE];
-void tiger_hash(char prefix, char *s, int len, unsigned char *digest)
+
+void tiger_hash(char prefix, char *s, long len, unsigned char *digest)
 {
-   
-      char *buffer = (char*) tiger_buffer;
-   
-      if(len>= 8*MAX_TIGER_CHUNK_SIZE)
-     {
-	
-	        printf("tiger_hash: error chunk too big\n");
-	        exit(2);
-     }
-   
-   
-      buffer[0] = prefix;
-      memcpy(buffer+1, s, len);
-      static_tiger(tiger_buffer, (len+1), (word64*) (digest));
+  char *buffer = (char*) tiger_buffer;
+  word64 ndigest[3];
+
+  buffer[0] = prefix;
+  memcpy(buffer +1, s, len);
+
+  memcpy(ndigest, digest, 3 * sizeof(word64));
+  static_tiger(tiger_buffer, (len +1), (word64*) (ndigest));
+  memcpy(digest, ndigest, 3 * sizeof(word64));
+
+  swap_digest(digest);
 }
-#endif
 
-
-void tiger_tree_string(char *s, int len, int pos, int block_size, char *digest)
+void tiger_tree_string(char *s, size_t len, size_t pos, size_t block_size, char *digest)
 {
   if(block_size == BLOCK_SIZE){
-    int length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
+    size_t length = (len - pos > BLOCK_SIZE) ? BLOCK_SIZE : len - pos;
     tiger_hash(0, s+pos, length, digest);
   } else {    
     if(pos+block_size/2 >=len){
@@ -827,14 +823,14 @@ void tiger_tree_string(char *s, int len, int pos, int block_size, char *digest)
 #include "caml/fail.h"
 #include "caml/alloc.h"
 
-int tiger_block_size(int len)
+unsigned long tiger_block_size(unsigned long len)
 {
-  int block_size = BLOCK_SIZE;
+  unsigned long block_size = BLOCK_SIZE;
   while(block_size < len) block_size *= 2;
   return block_size;
 }
 
-value tiger_unsafe_string(value digest_v, value string_v, value len_v)
+value tigertree_unsafe_string(value digest_v, value string_v, value len_v)
 {
   unsigned char *digest = String_val(digest_v);
   unsigned char *string = String_val(string_v);
@@ -845,29 +841,15 @@ value tiger_unsafe_string(value digest_v, value string_v, value len_v)
   return Val_unit;
 }
 
-#if 0
-#include <stdio.h>
-
-value tiger_unsafe_file (value digest_v, value filename_v)
+value tiger_unsafe_string(value digest_v, value string_v, value len_v)
 {
-  char *filename  = String_val(filename_v);
   unsigned char *digest = String_val(digest_v);
-  FILE *file;
-  tiger_state_t context;
-  int len;
+  unsigned char *string = String_val(string_v);
+  long len = Long_val(len_v);
 
-  if ((file = fopen (filename, "rb")) == NULL)
-    raise_not_found();
-
-  else {
-    tiger_init (&context);
-    while ((len = fread (hash_buffer, 1, HASH_BUFFER_LEN, file)))
-      tiger_append (&context, hash_buffer, len);
-    tiger_finish (&context, digest);
-
-    fclose (file);
-  }
+  static_tiger ((word64*)string, len, (word64*) digest);  
+  swap_digest(digest);
+  
   return Val_unit;
 }
-#endif
 
