@@ -19,14 +19,24 @@
 
 type t
 
-  
+val external_start : string -> unit
+val external_exit : unit -> unit
+val uname : unit -> string
+
+(* CryptoPP *)
+val create_key : unit -> string
+val load_key : string -> string
+val create_signature : string -> int -> int64 -> int -> int64 -> string
+val verify_signature : string -> int -> string -> int -> int64 -> int -> int64 -> bool
+
+val create_dir_mask : string ref
 val close : t -> unit
 (* val force_fd : t -> Unix.file_descr *)
   
 (* val seek64 : t -> int64 -> Unix.seek_command -> int64  *)
-val getsize : string -> int64
-val getsize64 : t -> int64
-val ftruncate64 : t -> int64 -> unit
+val getsize : string -> bool -> int64
+val getsize64 : t -> bool -> int64
+val ftruncate64 : t -> int64 -> bool -> unit
 
 val close_all : unit -> unit
 
@@ -46,14 +56,14 @@ val max_buffered : int64 ref
 val remove : t -> unit
   
 val read : t -> int64 -> string -> int -> int -> unit
-val allocate_chunk :  t -> int64 -> int -> unit
+(*val allocate_chunk :  t -> int64 -> int -> unit*)
   
 val copy_chunk : t -> t -> int64 -> int64 -> int -> unit
     
 val file_exists : string -> bool
 
-val fd_of_chunk : t -> int64 -> int64 -> 
-  (Unix.file_descr * int64 * string option) 
+val apply_on_chunk : t -> int64 -> int64 -> 
+  (Unix.file_descr -> int64 -> 'a) -> 'a
   
 
 val create_diskfile : string -> Unix.open_flag list -> int -> t
@@ -76,4 +86,19 @@ val ro_flag :  Unix.open_flag list
 val rw_flag :  Unix.open_flag list
   
 val destroy : t -> unit
-  
\ No newline at end of file
+  
+val bad_fd : t
+  
+val destroyed : t -> bool
+
+val bsize : string -> int64 option
+val blocks : string -> int64 option
+val bfree : string -> int64 option
+val bavail : string -> int64 option
+val fnamelen : string -> int64 option
+val disktotal : string -> int64 option
+val diskfree : string -> int64 option
+val diskused : string -> int64 option
+val percentused : string -> int option
+val percentfree : string -> int option
+val filesystem : string -> string
