@@ -28,7 +28,7 @@ open Unix
 open Url
 open TcpBufferedSocket
 
-  
+
 type http_request =
   GET
 | POST
@@ -38,12 +38,7 @@ type http_request =
 | TRACE
 
 let verbose = ref false
-  
-  (*
-| OPTIONS of url option (* None = '*' *)
-| CONNECT of string * int
-*)
-  
+
 type request = {
     req_headers : ( string * string ) list;
     req_user_agent : string;
@@ -53,11 +48,17 @@ type request = {
     req_url : url;
     req_request : http_request;
     req_referer : Url.url option;
+    req_retry : int;
+    req_max_retry : int;
   }
 
 type content_handler = 
   int -> (string * string) list -> TcpBufferedSocket.t -> int -> unit
 
+let lprintf_nl () =
+  lprintf "%s[HTTPcl] "
+      (log_time ()); lprintf_nl2
+
 let basic_request = {
     req_url = Url.of_string "http://www.mldonkey.net/";
     req_referer = None;
@@ -67,6 +68,8 @@ let basic_request = {
     req_headers = [];
     req_user_agent = "Wget 1.4";
     req_accept = "*/*";
+    req_retry = 0;
+    req_max_retry = 0;
   }
       
 let make_full_request r =
@@ -92,14 +95,14 @@ let make_full_request r =
     url);
   Printf.bprintf res " HTTP/1.0\r\nHost: %s%s\r\n" url.server (if url.port != 80 then Printf.sprintf ":%d" url.port else "");
   List.iter (fun (a,b) ->
-      Printf.bprintf res "%s: %s\r\n" a b      
+      Printf.bprintf res "%s: %s\r\n" a b
   ) r.req_headers;
   Printf.bprintf res "User-Agent: %s\r\n" r.req_user_agent;
   Printf.bprintf res "Accept: %s\r\n" r.req_accept;
   Printf.bprintf res "Connection: close\r\n";
  (match r.req_referer with None -> ()
     | Some url -> 
-        Printf.bprintf res "Referer: %s\r\n"  (Url.to_string_no_args url));
+        Printf.bprintf res "Referer: %s\r\n" (Url.to_string_no_args url));
   if is_real_post then begin
       let post = Buffer.create 80 in
       let rec make_post = function
@@ -117,7 +120,7 @@ let make_full_request r =
     Buffer.add_string res "\r\n";
   let s = Buffer.contents res in
   if !verbose then
-    lprintf "Http_client.make_full_request on URL: %s\n" s;
+    lprintf_nl () "make_full_request on URL: %s" (String.escaped s);
   s
 
 let split_head s =
@@ -139,10 +142,13 @@ let parse_header headers_handler sock header =
   match headers with 
     [] -> failwith "Ill formed reply"
   | ans :: headers ->
-      if !verbose then lprintf "Http_client.parse_header: ANSWER %s\n" ans;
+      if !verbose then lprintf_nl () "parse_header: ANSWER %s" ans;
       let ans_code = int_of_string (String.sub ans 9 3) in
       let headers = List.map (fun s ->
             let sep = String.index s ':' in
+            (* TODO: we should lowercase the names here!
+               The header-names are case-insensitive,
+               therefore we only use lowercased names. *)
             let name_head = String.sub s 0 sep in
             let size = String.length s in
             let content_head = String.sub s (sep+2) (size-sep-2) in
@@ -152,14 +158,14 @@ let parse_header headers_handler sock header =
         headers_handler sock ans_code headers;
       with _ -> 
           TcpBufferedSocket.close sock (Closed_for_error "bad header")
-  
+
 let read_header header_handler sock nread =  
   let b = TcpBufferedSocket.buf sock in
   let end_pos = b.pos + b.len in
   let new_pos = end_pos - nread in
   let new_pos = maxi 0 (new_pos - 1) in
   (*
-  lprintf "received [%s]" (String.escaped 
+  lprintf "received [%s]" (String.escaped
       (String.sub b.buf new_pos nread));
   *)
   let rec iter i =
@@ -171,7 +177,7 @@ let read_header header_handler sock nread =
           let len = i + 2 - b.pos in
           let header = String.sub b.buf b.pos len in
           buf_used b len;
-          header_handler sock header        
+          header_handler sock header
         else
         if c = '\r' && i <= end_pos - 3 && b.buf.[i+2] = '\n' then
           let len = i + 3 - b.pos in
@@ -192,27 +198,12 @@ let http_reply_handler nr headers_handler sock nread =
   nr := true;
   read_header (parse_header headers_handler) sock nread
   
-let get_page r content_handler f =
+let rec get_page r content_handler f =
+  let error = ref false in
   let rec get_url level r =
-    
+  try
     let url = r.req_url in
-(*
-    let args = ref [] in
-    let headers = ref [] in
-    let ispost = ref false in
-    let timeout = ref 300.0 in
-    let proxy = ref None in
-    List.iter (function   
-      | Args l -> args := l@ !args
-      | Headers l -> headers := l @ !headers;
-      | Post -> ispost := true
-      | Proxy (h,p) -> proxy := Some (h,p)
-    ) get_args;
-    let args = !args in
-    let headers = !headers in
-    let ispost = !ispost in
-    let proxy = !proxy in
-*)    
+    let level = r.req_retry in
     let request = make_full_request r in
     
     
@@ -223,121 +214,160 @@ let get_page r content_handler f =
     in
 (*    lprintf "async_ip ...\n"; *)
     Ip.async_ip server (fun ip ->
-(*        lprintf "IP done %s:%d\n" (Ip.to_string ip) port; *)
+(*         lprintf "IP done %s:%d\n" (Ip.to_string ip) port;*)
         let token = create_token unlimited_connection_manager in
         let sock = TcpBufferedSocket.connect token "http client connecting"
-            (Ip.to_inet_addr ip)
-          port (fun _ e -> 
-              ()
-(*              lprintf "Event %s\n"
-                (match e with
-                  WRITE_DONE -> "WRITE_DONE"
-                | CAN_REFILL -> "CAN_REFILL"
-                | BUFFER_OVERFLOW -> "BUFFER_OVERFLOW"
-                | READ_DONE n -> Printf.sprintf "READ_DONE %d" n
-                | BASIC_EVENT e ->
-                    match e with
-                      (CLOSED s) -> Printf.sprintf "CLOSED %s" s
-                    | RTIMEOUT -> "RTIMEOUT"
-                    | LTIMEOUT -> "LTIMEOUT"
-                    | WTIMEOUT -> "WTIMEOUT"
-                    | CAN_READ -> "CAN_READ"
-                    | CAN_WRITE -> "CAN_WRITE"
-              ) *)
+            (try Ip.to_inet_addr ip with e -> raise Not_found)
+          port (fun sock e -> 
+              () 
+(*              if !verbose then
+                  lprintf "Event %s\n"
+                    (match e with
+                      CONNECTED -> "CONNECTED"
+                    | WRITE_DONE -> "WRITE_DONE"
+                    | CAN_REFILL -> "CAN_REFILL"
+                    | BUFFER_OVERFLOW -> "BUFFER_OVERFLOW"
+                    | READ_DONE n -> Printf.sprintf "READ_DONE %d" n
+                    | BASIC_EVENT e ->
+                        match e with
+                          (CLOSED s) -> Printf.sprintf "CLOSED %s" (string_of_reason s)
+                        | RTIMEOUT -> "RTIMEOUT"
+                        | LTIMEOUT -> "LTIMEOUT"
+                        | WTIMEOUT -> "WTIMEOUT"
+                        | CAN_READ -> "CAN_READ"
+                        | CAN_WRITE -> "CAN_WRITE"
+                    )
+ *)
           )
         in
         let nread = ref false in
         if !verbose then 
-          lprintf "Http_client.get_page: %s\n" (String.escaped request);
+          lprintf_nl () "get_page: %s" (String.escaped request);
         TcpBufferedSocket.write_string sock request;
-        TcpBufferedSocket.set_reader sock (http_reply_handler nread 
+        TcpBufferedSocket.set_reader sock (http_reply_handler nread
             (default_headers_handler url level));
         set_rtimeout sock 5.;
         TcpBufferedSocket.set_closer sock (fun _ _ -> ()
 (*        lprintf "Connection closed nread:%b\n" !nread; *)
         )
+
     )
-  
+  with e -> lprintf_nl () "error in get_url"; raise Not_found
+
   and default_headers_handler old_url level sock ans_code headers =
-(*
-    List.iter (fun (name, value) ->
-        lprintf "[%s]=[%s]" name value;
-    ) headers;
-*)
+    let print_headers () =
+      List.iter
+        (fun (name, value) ->
+          lprintf_nl () "[%s]=[%s]" name value;
+        ) headers;
+    in
+    (* print_headers (); *)
     match ans_code with
       200 ->
-(*
-  lprintf "ans_code: %d\n" ans_code;
-  List.iter (fun (name, content) ->
-      lprintf "HEADER %s:%s" name content;
-      ) 
-headers;
-*)
-        TcpBufferedSocket.set_closer sock (fun _ _ -> 
-(*            lprintf "default_headers_handler closer\n"; *)
-            f ());
-        
+        (*
+        lprintf "ans_code: %d\n" ans_code;
+        print_headers ();
+        *)
+        TcpBufferedSocket.set_closer sock
+            (fun _ _ -> 
+              (* lprintf "default_headers_handler closer\n"; *)
+              f ()
+            );
+
         let content_length = ref (-1) in
         List.iter (fun (name, content) ->
             if String.lowercase name = "content-length" then
-              try          
+              try
                 content_length := int_of_string content
               with _ -> 
-                  lprintf "bad content length [%s]\n" content;
+                  lprintf_nl () "bad content length [%s]" content;
         ) headers;
         let location = "Location", Url.to_string old_url in
         let content_handler = content_handler !content_length (location::headers) in
         set_reader sock content_handler;
         let buf = TcpBufferedSocket.buf sock in
         if buf.len > 0 then
-          content_handler sock buf.len 
-    
+          content_handler sock buf.len
+
     | 301 | 302 | 304 ->
-        lprintf "Http_client %d: Redirect\n" ans_code;
-        if level < 10 then
+        if !verbose then lprintf_nl () "%d: Redirect" ans_code;
+	let retrynum = r.req_retry in
+        if retrynum < r.req_max_retry then
           begin
             try
-              let url = List.assoc "Location" headers in
-              lprintf "Redirected to %s\n" url; 
-              List.iter (fun (name, value) ->
-                  lprintf "[%s]=[%s]\n" name value;
-              ) headers;
-              
-              let url = if String.length url > 0 && url.[0] <> '/' then
-                  url
+              let url = ref "" in
+              List.iter
+                (fun (name, content) ->
+                  if String.lowercase name = "location" then
+                    url := content;
+                ) headers;
+              if !verbose then
+                print_headers ();
+              let url =
+                if String.length !url > 0 && !url.[0] <> '/' then
+                  !url
                 else
                   Printf.sprintf "http://%s:%d%s"
-                    old_url.Url.server old_url.Url.port url 
+                    old_url.Url.server old_url.Url.port !url
               in
-              lprintf "Redirected to %s\n" url;               
-              let r = { r with req_url = Url.of_string url } in
-              get_url (level+1) r
+              if !verbose then lprintf_nl () "Redirected to %s" url;
+              let r = { r with
+			req_url = Url.of_string url;
+			req_retry = retrynum+1 }
+	      in
+	      get_page r content_handler f
             
             with e ->
-                List.iter (fun (name, value) ->
-                    lprintf "[%s]=[%s]\n" name value;
-                ) headers
+                lprintf_nl () "error understanding redirect response %d" ans_code;
+                print_headers ();
+                raise Not_found
                 
           end
+        else 
+          lprintf_nl () "more than %d redirections, aborting." r.req_max_retry;
+          raise Not_found
           
     | 404 ->
-        lprintf "Http_client 404: Not found %s\n" (Url.to_string_no_args r.req_url);
+        lprintf_nl () "404: Not found for: %s" (Url.to_string_no_args r.req_url);
         close sock (Closed_for_error "bad reply");
         raise Not_found
+
+    | 502 | 503 | 504 ->
+        if !verbose then lprintf_nl () "%d: Unavailable" ans_code;
+	let retrynum = r.req_retry in
+        if retrynum < r.req_max_retry then
+          begin
+            if !verbose then
+              print_headers ();
+	    let seconds = (retrynum+1)*10 in
+              lprintf_nl () "retry %d/%d in %d seconds for %s"
+	        (retrynum+1) r.req_max_retry seconds (Url.to_string_no_args r.req_url);
+	    let r = { r with req_retry = retrynum+1 } in
+	      add_timer (float(seconds)) (fun t -> get_page r content_handler f)
+	  end
+        else 
+          lprintf_nl () "more than %d retries, aborting." r.req_max_retry;
+          raise Not_found
           
     | _ ->
-        lprintf "Http_client: bad reply %d\n" ans_code;
+        lprintf_nl () "%d: bad reply for: %s"
+          ans_code (Url.to_string_no_args r.req_url);
         close sock (Closed_for_error "bad reply");
         raise Not_found
-
-  in get_url 0 r
-
+  in
+  get_url 0 r;
+  if !error = true then begin
+      lprintf_nl () "failed!";
+      raise Not_found
+    end
+  
   
 let wget r f = 
   
   let file_buf = Buffer.create 1000 in
   let file_size = ref 0 in
   
+  try
   get_page r
     (fun maxlen headers sock nread ->
 (*      lprintf "received %d\n" nread; *)
@@ -357,35 +387,39 @@ let wget r f =
         end)
   (fun _ ->  
       let s = Buffer.contents file_buf in
-      if s = "" then begin  
-          lprintf "Empty content for url %s\n" 
+      if s = "" then begin
+          lprintf_nl () "Empty content for url %s"
             (Url.to_string r.req_url);
         end;
-      let filename = Filename.temp_file "http_" ".tmp" in
+      let ext = (String.lowercase (Filename2.extension r.req_url.Url.short_file)) in
+      let filename = Filename.temp_file "http_" (if ext = "" then ".tmp" else ext) in
       let oc = open_out_bin filename in
       output_string oc s;
       close_out oc;
       try
         (f filename : unit);
-        Sys.remove filename 
-      with  e ->  lprintf
+        Sys.remove filename
+      with e ->  lprintf_nl ()
             "Exception %s in loading downloaded file %s"
             (Printexc2.to_string e) filename;
-          Sys.remove filename 
+          Sys.remove filename;
+          raise Not_found
   )
-  
+  with e -> lprintf_nl () "Exception %s in wget" (Printexc2.to_string e); raise Not_found
+
 let whead r f = 
   
   get_page r
     (fun maxlen headers ->
-      lprintf "Http_client.headers...\n";
+     if !verbose then
+      lprintf_nl () "headers...";
       (try f headers with _ -> ());
       fun sock nread -> 
         close sock Closed_by_user
-  )
+    )
   (fun _ ->  ())
-  
-let wget_string r f progress = 
+
+let wget_string r f progress =
     
   let file_buf = Buffer.create 1000 in
   let file_size = ref 0 in
@@ -411,8 +445,8 @@ let wget_string r f progress =
       f (Buffer.contents file_buf)
   )
 
-  
-let split_header header =     
+
+let split_header header =
   for i = 0 to String.length header - 1 do
     if header.[i] = '\r' then header.[i] <- '\n';
   done;
@@ -436,6 +470,5 @@ let cut_headers headers =
           String.sub s (pos+1) (len-pos-1), key
     ) headers
   with e ->
-      lprintf "Exception in cut_headers: %s\n" (Printexc2.to_string e);
+      lprintf_nl () "Exception in cut_headers: %s" (Printexc2.to_string e);
       raise e
-      
