@@ -21,13 +21,16 @@ open Int64ops
 open Printf2
 open BasicSocket
 
+
+let latencies = Hashtbl.create 2131
+
 let max_opened_connections = ref (fun () -> maxi 20 (MlUnix.max_sockets - 50))
 let max_connections_per_second = ref (fun () -> 50)
 
 let opened_connections = ref 0
 let opened_connections_this_second = ref 0
 
-let max_buffer_size = ref 100000
+let max_buffer_size = ref 10000000
 
 let bind_address = ref Unix.inet_addr_any
 let ip_packet_size = ref 40
@@ -73,8 +76,8 @@ type buf = {
     mutable buf : string;
     mutable pos : int;
     mutable len : int;
-    mutable max_buf_size:int;
-    mutable min_buf_size: int
+    mutable max_buf_size : int;
+    mutable min_buf_size : int
   }
 
 type t = {
@@ -97,12 +100,13 @@ type t = {
     mutable write_power : int;
     mutable read_power : int;
 
-    mutable peer_ip : Ip.t;
+    mutable peer_addr : (Ip.t * int) option;
     mutable my_ip : Ip.t;
 
     mutable noproxy : bool;
     mutable connecting : bool;
-    mutable host : string;
+    mutable host : Ip.t;
+    mutable connect_time : float;
 
     mutable token : token;
 
@@ -128,7 +132,7 @@ and bandwidth_controler = {
     mutable moved_bytes : int64;
     mutable lost_bytes : int array; (* 3600 samples*)
     mutable forecast_bytes : int;
-    
+
     mutable ndone_last_second : int;
   }
 
@@ -177,11 +181,11 @@ let schedule_connections () =
   let max_wanted = !max_opened_connections () in
   let current_connections = !opened_connections in
   let max_connections_per_second = !max_connections_per_second () in
-  
+
   let rec iter todo_managers done_managers =
 
 (*
-    lprintf "todo_managers %d done_managers %d\n" 
+    lprintf "todo_managers %d done_managers %d\n"
 (List.length todo_managers) (List.length done_managers);
   *)
     match todo_managers with
@@ -227,22 +231,21 @@ let unlimited_connection_manager = create_connection_manager "Unlimited"
 
 let reset_connection_scheduler _ =
   if !verbose_bandwidth > 0 then begin
-      lprintf "[BW1 %6d] Connections opened this second : %d/%d total %d/%d\n" 
+      lprintf "[BW1 %6d] Connections opened this second : %d/%d total %d/%d\n"
       (last_time ()) !opened_connections_this_second (!max_connections_per_second ()) !opened_connections (!max_opened_connections ());
-          
     end;
   List.iter (fun cm ->
       if !verbose_bandwidth > 0 then begin
           if cm.nconnections_last_second > 0 then
             lprintf "[BW1 %6d]    %s opened %d connections last second\n"
               (last_time ()) cm.cm_name cm.nconnections_last_second;
-          if cm.nwaiting_connections > 0 then 
+          if cm.nwaiting_connections > 0 then
             lprintf "[BW1 %6d]    %s still waits for %d connections\n"
               (last_time ()) cm.cm_name cm.nwaiting_connections;
-       end;    
+       end;
       cm.nconnections_last_second <- 0;
   ) !connection_managers;
-  
+
   opened_connections_this_second := 0
 
 let use_token token fd =
@@ -262,26 +265,43 @@ let use_token token fd =
 (*                                                                       *)
 (*************************************************************************)
 
+let add_connect_latency ip time =
+  if ip <> Ip.null && time > 1. then
+    let delay = current_time () -. time in
+    let delayf = 1000. *. delay in
+    let delay = int_of_float delayf in
+    let delay = if delay > 65000 then 65000 else delay in
+(*
+lprintf "add_connect_latency %s -> %d (%f)\n"
+(Ip.to_string ip) delay delayf;
+  *)
+    try
+      let latency, samples = Hashtbl.find latencies ip in
+      incr samples;
+      if !latency > delay then latency := delay
+    with _ ->
+        Hashtbl.add latencies ip (ref delay, ref 1)
+
 let forecast_bytes t nbytes =
   match t with
     None -> ()
-  | Some bc ->       
+  | Some bc ->
       let nip_packets = 1 + nbytes / !mtu_packet_size in
       let nbytes = nbytes + nip_packets * !ip_packet_size in
       let nframes = 1 + nbytes / packet_frame_size in
-      bc.forecast_bytes <- bc.forecast_bytes + 
+      bc.forecast_bytes <- bc.forecast_bytes +
         (nframes * packet_frame_size)
-  
+
 let register_bytes t nbytes =
   match t with
     None -> ()
-  | Some bc ->       
+  | Some bc ->
       let nip_packets = 1 + nbytes / !mtu_packet_size in
       let nbytes = nbytes + nip_packets * !ip_packet_size in
       let nframes = 1 + nbytes / packet_frame_size in
-      bc.remaining_bytes <- bc.remaining_bytes - 
+      bc.remaining_bytes <- bc.remaining_bytes -
       (nframes * packet_frame_size)
-  
+
 let forecast_download t n =
   forecast_bytes t.read_control n
 
@@ -313,10 +333,10 @@ let best_packet_size nbytes =
 (*************************************************************************)
 
 let copy_read_buffer = ref true
-  
+
 let big_buffer_len = 65536
 let big_buffer = String.create big_buffer_len
-  
+
 let min_buffer_read = 2000
 let min_read_size = min_buffer_read - 100
 
@@ -389,9 +409,9 @@ let buf_add t b s pos1 len =
       end
     else
     if b.len + len > b.max_buf_size then begin
-        lprintf "BUFFER OVERFLOW %d+%d> %d\n" b.len len b.max_buf_size ;
+        lprintf "[TCP_BS]: BUFFER OVERFLOW %d+%d> %d " b.len len b.max_buf_size ;
 
-        lprintf "MESSAGE [";
+        lprintf "MESSAGE: [";
         for i = pos1 to pos1 + (mini len 20) - 1 do
           lprintf "(%d)" (int_of_char s.[i]);
         done;
@@ -399,6 +419,8 @@ let buf_add t b s pos1 len =
         lprintf "]\n";
 
         t.event_handler t BUFFER_OVERFLOW;
+(* TODO: why do we have this ??? in case of BUFFER_OVERFLOW, just close the
+  socket !!! *)
       end
     else
     let new_len = mini (maxi (2 * max_len) (b.len + len)) b.max_buf_size  in
@@ -425,6 +447,10 @@ let buf_add t b s pos1 len =
 (*************************************************************************)
 
 let buf t = t.rbuf
+let setsock_iptos_throughput t =
+  ignore (setsock_iptos_throughput (fd (t.sock_in)));
+  ignore (setsock_iptos_throughput (fd (t.sock_out)))
+
 let closed t = closed t.sock_out
 let error t = t.error
 let sock_used t nused = buf_used t.rbuf nused
@@ -432,7 +458,7 @@ let remaining_to_write t =  t.wbuf.len
 let nread t = t.nread
 let nwritten t = t.nwrite
 let can_write t =  t.wbuf.len = 0
-let can_write_len t len = 
+let can_write_len t len =
   let b = t.wbuf.max_buf_size > t.wbuf.len + len in
 (*  if not b then
     lprintf "can_write_len failed: %d < %d + %d\n"
@@ -441,7 +467,7 @@ let can_write_len t len =
 let not_buffer_more t max =  t.wbuf.len < max
 let can_fill t = t.wbuf.len < (t.wbuf.max_buf_size / 2)
 let get_rtimeout t = get_rtimeout t.sock_in
-let max_refill t = t.wbuf.max_buf_size - t.wbuf.len 
+let max_refill t = t.wbuf.max_buf_size - t.wbuf.len
 
 let close t s =
 (*
@@ -480,7 +506,7 @@ let shutdown t s =
       lprintf "shutdown\n";
 end;
   *)
-  (try 
+  (try
       BasicSocket.shutdown t.sock_out s;
       if t.sock_in != t.sock_out then
         BasicSocket.shutdown t.sock_in s;
@@ -516,23 +542,29 @@ let write t s pos1 len =
 (*       lprintf "WRITE [%s]\n" (String.escaped (String.sub s pos1 len)); *)
           let nw = MlUnix.write fd s pos1 len in
           if !verbose_bandwidth > 1 then begin
-              lprintf "[BW2 %6d] immediate write %d/%d on %s:%d\n" (last_time ()) 
-              nw len 
+              lprintf "[BW2 %6d] immediate write %d/%d on %s:%d\n" (last_time ())
+              nw len
                 t.name (sock_num t.sock_out);
             end;
-          
+
           register_upload t len;
           forecast_download t 0;
 
 (*          if t.monitored then begin
               lprintf "write: direct written %d\n" nw;
 end; *)
-          tcp_uploaded_bytes := Int64.add !tcp_uploaded_bytes (Int64.of_int nw);
+          tcp_uploaded_bytes := !tcp_uploaded_bytes ++ (Int64.of_int nw);
           (match t.write_control with
               None -> ()
             | Some bc ->
-                bc.moved_bytes <-
-                Int64.add bc.moved_bytes (Int64.of_int nw));
+                bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nw));
+          if t.nwrite = 0 then begin
+(*              if t.connecting then
+                lprintf "WRITE BEFORE CONNECTION.......\n";
+              lprintf "add_connect_latency at %f\n" (current_time ()); *)
+              add_connect_latency t.host t.connect_time;
+            end;
+
           t.nwrite <- t.nwrite + nw;
           if nw = 0 then (close t Closed_by_peer; pos2) else
             pos1 + nw
@@ -574,7 +606,7 @@ let can_read_handler t sock max_len =
       big_buffer, 0, big_buffer_len
     else
     let can_write_in_buffer =
-      if b.buf = "" then 
+      if b.buf = "" then
         if b.min_buf_size <= min_buffer_read then begin
             b.buf <- new_string ();
             min_buffer_read
@@ -640,14 +672,14 @@ let can_read_handler t sock max_len =
         lprintf "READ LIMITED BY BW CONTROL: %d\n" nread;
       end;
 *)
-    
+
     if !verbose_bandwidth > 1 then begin
-        lprintf "[BW2 %6d] %sread %d/%d/%d on %s:%d\n" (last_time ()) 
-        (if old_len > 0 then "completing " else "") nread can_read max_len 
+        lprintf "[BW2 %6d] %sread %d/%d/%d on %s:%d\n" (last_time ())
+        (if old_len > 0 then "completing " else "") nread can_read max_len
           t.name (sock_num t.sock_in);
       end;
-    
-    
+
+
     if !copy_read_buffer then begin
 (*        lprintf "Copying %d bytes\n" nread; *)
         buf_add t b big_buffer 0 nread
@@ -664,12 +696,11 @@ let can_read_handler t sock max_len =
         lprintf "    given by buffer: %d\n" buffer_len;
     end;
 *)
-    
-    tcp_downloaded_bytes := Int64.add !tcp_downloaded_bytes (Int64.of_int nread);
+
+    tcp_downloaded_bytes := !tcp_downloaded_bytes ++ (Int64.of_int nread);
     (match t.read_control with
         None -> () | Some bc ->
-          bc.moved_bytes <-
-          Int64.add bc.moved_bytes (Int64.of_int nread));
+          bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nread));
 
     t.nread <- t.nread + nread;
     if nread > 0 then begin
@@ -680,7 +711,6 @@ let can_read_handler t sock max_len =
       close t Closed_by_peer;
     end else begin
 
-      
       try
 (*              if t.monitored then
    (lprintf "event handler READ DONE\n"; ); *)
@@ -713,22 +743,25 @@ let can_write_handler t sock max_len =
           try
 (*     lprintf "try write %d/%d\n" max_len t.wbuf.len; *)
             let fd = fd sock in
-(*            lprintf "WRITE [%s]\n" (String.escaped 
+(*            lprintf "WRITE [%s]\n" (String.escaped
               (String.sub b.buf b.pos max_len)); *)
             let nw = MlUnix.write fd b.buf b.pos max_len in
             if !verbose_bandwidth > 1 then begin
-                lprintf "[BW2 %6d] postponed %swrite %d/%d/%d on %s:%d\n" (last_time ()) (if max_len < b.len then "partial " else "") nw max_len b.len 
+                lprintf "[BW2 %6d] postponed %swrite %d/%d/%d on %s:%d\n" (last_time ()) (if max_len < b.len then "partial " else "") nw max_len b.len
                   t.name (sock_num t.sock_out);
               end;
-            
+
 (*            if t.monitored then
 (lprintf "written %d\n" nw; ); *)
-        tcp_uploaded_bytes := Int64.add !tcp_uploaded_bytes (Int64.of_int nw);
+        tcp_uploaded_bytes := !tcp_uploaded_bytes ++ (Int64.of_int nw);
         (match t.write_control with
             None -> ()
           | Some bc ->
-              bc.moved_bytes <-
-              Int64.add bc.moved_bytes (Int64.of_int nw));
+              bc.moved_bytes <- bc.moved_bytes ++ (Int64.of_int nw));
+            if t.nwrite = 0 then begin
+(*                lprintf "add_connect_latency at %f\n" (current_time ()); *)
+                add_connect_latency t.host t.connect_time;
+              end;
         t.nwrite <- t.nwrite + nw;
         b.len <- b.len - nw;
         b.pos <- b.pos + nw;
@@ -781,7 +814,23 @@ let tcp_handler_write t sock =
               end
           end
   end
-  
+
+let get_latencies verbose =
+  let b = Buffer.create 300 in
+  let counter = ref 0 in
+  Hashtbl.iter (fun ip (latency, samples) ->
+      incr counter;
+  ) latencies;
+  LittleEndian.buf_int b !counter;
+  Hashtbl.iter (fun ip (latency, samples) ->
+      if !verbose then lprintf "   Latency TCP: %s -> %d (%d samples)\n" (Ip.to_string ip) !latency !samples;
+      LittleEndian.buf_ip b ip;
+      LittleEndian.buf_int16 b !latency;
+      LittleEndian.buf_int16 b !samples;
+  ) latencies;
+  Hashtbl.clear latencies;
+  Buffer.contents b
+
 let tcp_handler t event =
   match event with
   | CAN_READ ->
@@ -808,7 +857,7 @@ let tcp_handler t event =
         if t.nwrite = 0 && t.noproxy && t.connecting then  begin
             t.connecting <- false;
             t.event_handler t CONNECTED;
-            t.nwrite = 0 
+            t.nwrite = 0
           end else true
       in
       if can_write then tcp_handler_write t t.sock_out
@@ -877,10 +926,10 @@ let reset_bandwidth_controlers _ =
       bc.remaining_bytes_user bc.total_bytes bc.remaining_bytes;
       bc.remaining_bytes <- bc.total_bytes - bc.forecast_bytes;
       if !verbose_bandwidth > 0 && bc.ndone_last_second > 0 then
-        lprintf "[BW1 %6d] %s read %d/%d last second\n" (last_time ()) 
+        lprintf "[BW1 %6d] %s read %d/%d last second\n" (last_time ())
           bc.bc_name bc.ndone_last_second bc.total_bytes;
       if !verbose_bandwidth > 0 && bc.forecast_bytes > 0 then
-        lprintf "[BW1 %6d] %s forecast read %d bytes for next second\n" (last_time ()) 
+        lprintf "[BW1 %6d] %s forecast read %d bytes for next second\n" (last_time ())
           bc.bc_name bc.forecast_bytes;
       bc.forecast_bytes <- 0;
       bc.ndone_last_second <- 0;
@@ -891,10 +940,10 @@ let reset_bandwidth_controlers _ =
       bc.remaining_bytes_user bc.total_bytes bc.remaining_bytes;
       bc.remaining_bytes <- bc.total_bytes - bc.forecast_bytes;
       if !verbose_bandwidth > 0 && bc.ndone_last_second > 0 then
-        lprintf "[BW1 %6d] %s wrote %d/%d last second\n" (last_time ()) 
+        lprintf "[BW1 %6d] %s wrote %d/%d last second\n" (last_time ())
           bc.bc_name bc.ndone_last_second bc.total_bytes;
       if !verbose_bandwidth > 0 && bc.forecast_bytes > 0 then
-        lprintf "[BW1 %6d] %s forecast write %d bytes for next second\n" (last_time ()) 
+        lprintf "[BW1 %6d] %s forecast write %d bytes for next second\n" (last_time ())
           bc.bc_name bc.forecast_bytes;
       bc.forecast_bytes <- 0;
       bc.ndone_last_second <- 0;
@@ -908,9 +957,9 @@ let compute_lost_byte bc =
   if bc.total_bytes = 0 then -1 else
   let sum = ref Int64.zero in
   for i = 0 to 3600-1 do
-    sum := Int64.add !sum (Int64.of_int bc.lost_bytes.(i));
+    sum := !sum ++ (Int64.of_int bc.lost_bytes.(i));
   done;
-  Int64.to_int (Int64.div !sum (Int64.of_int 3600))
+  Int64.to_int (!sum // 3600L)
 
 let moved_bytes bc = bc.moved_bytes
 
@@ -973,7 +1022,7 @@ let close_after_write t =
     set_handler t WRITE_DONE (fun t ->
 (*      lprintf "close_after_write: CLOSE\n";  - log output removed  *)
         shutdown t Closed_by_user)
-    
+
 exception Http_proxy_error of string
 let http_proxy = ref None
 
@@ -997,7 +1046,8 @@ let set_reader t f =
               let rstr_pos = 12 (*String.index_from b.buf (rcode_pos+1) ' '*) in
               let rstr_end = String.index_from b.buf (rstr_pos+1) '\n' in
               let rstr = String.sub b.buf (rstr_pos+1) (rstr_end-rstr_pos-1) in
-              lprintf "From proxy for %s: %s %s\n" sock.host rcode rstr;
+              lprintf "From proxy for %s: %s %s\n"
+                (Ip.to_string sock.host) rcode rstr;
               rcode, rstr, rstr_end
             with _ ->
                 "", "", 0
@@ -1047,7 +1097,7 @@ let set_refill t f =
 
 let set_connected t f =
   set_handler t CONNECTED f
-    
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         Socket Configuration                          *)
@@ -1066,8 +1116,8 @@ let set_write_controler t bc =
   set_allow_write t.sock_out bc.allow_io;
   bandwidth_controler t t.sock_out
 
-let set_monitored t =
-  t.monitored <- true
+let set_monitored t b = t.monitored <- b
+let monitored t = t.monitored
 
 let set_rtimeout s t = set_rtimeout s.sock_in t
 let set_wtimeout s t = set_wtimeout s.sock_out t
@@ -1125,11 +1175,12 @@ let create token name fd handler =
       write_control = None;
       write_power = 1;
       read_power = 1;
-      peer_ip = Ip.null;
+      connect_time = 0.;
+      peer_addr = None;
       my_ip = Ip.null;
       noproxy = true;
       connecting = false;
-      host = "";
+      host = Ip.null;
       compression = None;
     } in
   let sock = BasicSocket.create name fd (fun _ event ->
@@ -1140,9 +1191,8 @@ let create token name fd handler =
         ;
     );
   set_dump_info sock (dump_socket t);
-  if !debug then begin
-      lprintf "[fd %d %s]\n" (sock_num sock) name;
-    end;
+  if !debug then
+      lprintf_nl "[tBS] fd %d %s" (sock_num sock) name;
   t.sock_in <- sock;
   t.sock_out <- sock;
   t
@@ -1176,27 +1226,27 @@ let create_pipe token name fd_in fd_out handler =
       write_control = None;
       write_power = 1;
       read_power = 1;
-      peer_ip = Ip.null;
+      peer_addr = None;
       my_ip = Ip.null;
       noproxy = true;
       connecting = false;
-      host = "";
+      host = Ip.null;
+      connect_time = 0.;
       compression = None;
     } in
-  
+
   let fname = (fun _ ->
         Printf.sprintf "%s (nread: %d nwritten: %d) [U %s,D %s]" name
           t.nread t.nwrite (string_of_bool (t.read_control <> None))
         (string_of_bool (t.write_control <> None));
         ;
     ) in
-  
-  
+
+
   let sock_in = BasicSocket.create name fd_in (fun _ event ->
         tcp_handler t event) in
-  if !debug then begin
-      lprintf "[fd %d %s]\n" (sock_num sock_in) name;
-    end;
+  if !debug then
+      lprintf_nl "[tBS] fd %d %s" (sock_num sock_in) name;
   set_printer sock_in fname;
   set_dump_info sock_in (dump_socket t);
 
@@ -1204,7 +1254,7 @@ let create_pipe token name fd_in fd_out handler =
         tcp_handler t event) in
   set_printer sock_out fname;
   set_dump_info sock_out (dump_socket t);
-  
+
   t.sock_in <- sock_in;
   t.sock_out <- sock_out;
   t
@@ -1237,11 +1287,12 @@ let create_blocking token name fd handler =
       write_control = None;
       write_power = 1;
       read_power = 1;
-      peer_ip = Ip.null;
+      peer_addr = None;
       my_ip = Ip.null;
       noproxy = true;
       connecting = false;
-      host = "";
+      host = Ip.null;
+      connect_time = 0.;
       compression = None;
     } in
   let sock = create_blocking name fd (fun sock event ->
@@ -1272,12 +1323,13 @@ let connect token name host port handler =
         None -> Ip.null, 0
       | Some (h, p) -> Ip.from_name h, p
     in
-    let use_proxy = proxy_ip <> Ip.null && proxy_ip <> (Ip.of_inet_addr host) in
+    let ip = Ip.of_inet_addr host in
+    let use_proxy = proxy_ip <> Ip.null && proxy_ip <> ip in
     if use_proxy then begin
 (* connect to proxy in blocking mode, so we sure, connections established when we send CONNECT *)
         lprintf "via proxy\n";
         Unix.connect s (Unix.ADDR_INET(Ip.to_inet_addr proxy_ip, proxy_port));
-        
+
         let buf = Buffer.create 200 in
         let dotted_host = Unix.string_of_inet_addr host in
         Printf.bprintf buf "CONNECT %s:%d HTTP/1.1\n" dotted_host port;
@@ -1292,22 +1344,24 @@ let connect token name host port handler =
         ()
       end;
     let t = create token name s handler in
-    
+
     if !verbose_bandwidth > 1 then begin
         lprintf "[BW2 %6d] connect on %s:%d\n" (last_time ()) t.name (sock_num t.sock_out);
       end;
-    
-    token.connection_manager.nconnections_last_second <- 
+
+    token.connection_manager.nconnections_last_second <-
       token.connection_manager.nconnections_last_second + 1;
-    
+
     must_write t.sock_out true;
     try
+      t.host <- ip;
+(*      lprintf "add_connect at %f\n" (current_time ()); *)
+      t.connect_time <- current_time ();
       if use_proxy then begin
           t.noproxy <- false;
-          t.host <- (Unix.string_of_inet_addr host)
         end else
         Unix.connect s (Unix.ADDR_INET(host,port));
-      
+
       t.connecting <- true;
       register_upload t 0;    (* The TCP SYN packet *)
       forecast_download t 0;  (* The TCP ACK packet *)
@@ -1322,15 +1376,21 @@ let connect token name host port handler =
         forecast_download t 0;  (* The TCP ACK packet *)
         forecast_upload t 0;    (* The TCP ACK packet *)
         t
+    | Unix.Unix_error (Unix.ENETUNREACH,_,_) as e ->
+	close t Closed_connect_failed;
+        raise e
     | e ->
-        lprintf "For host %s port %d\n" (Unix.string_of_inet_addr host)
-        port;
+        lprintf "For host %s:%d   "
+            (Unix.string_of_inet_addr host) port;
         close t Closed_connect_failed;
         raise e
-  with e ->
-      lprintf "+++ Exception BEFORE CONNECT %s\n" (Printexc2.to_string e);
+  with
+    Unix.Unix_error (Unix.ENETUNREACH,_,_) as e -> raise e
+  | e ->
+      lprintf "EXCEPTION %s  before connect to host %s:%d\n"
+          (Printexc2.to_string e) (Unix.string_of_inet_addr host) port;
       raise e
-      
+
 
 
 (*************************************************************************)
@@ -1351,22 +1411,28 @@ let my_ip t =
     | _ -> raise Not_found
   else t.my_ip
 
-let peer_ip t =
-  if t.peer_ip = Ip.null then
-  let fd = fd t.sock_out in
-  match Unix.getpeername fd with
-      Unix.ADDR_INET (ip, port) ->
-        let ip = Ip.of_inet_addr ip in
-        t.peer_ip <- ip; ip
+let peer_addr t =
+  match t.peer_addr with
+    Some (ip, port) -> (ip,port)
+  | None ->
+      let fd = fd t.sock_out in
+      match Unix.getpeername fd with
+        Unix.ADDR_INET (ip, port) ->
+          let ip = Ip.of_inet_addr ip in
+          t.peer_addr <- Some (ip, port);
+          ip, port
   | _ -> raise Not_found
-  else
-    t.peer_ip
 
+let peer_ip t = fst (peer_addr t)
+let peer_port t = snd (peer_addr t)
+
+          (*
 let host t =
   let fd = fd t.sock_out in
   match Unix.getpeername fd with
     Unix.ADDR_INET (ip, port) -> Ip.of_inet_addr ip, port
   | _ -> raise Not_found
+        *)
 
 (*************************************************************************)
 (*                                                                       *)
@@ -1381,7 +1447,7 @@ let internal_buf = Buffer.create 17000
 
 let simple_send_buf buf sock =
   let s = Buffer.contents buf in
-  Buffer.clear buf;
+  Buffer.reset buf;
   buf_int8 buf 228;
   let len = String.length s in
   buf_int buf len;
@@ -1389,7 +1455,8 @@ let simple_send_buf buf sock =
   write sock s 0 len
 
 let value_send sock m =
-  Buffer.clear internal_buf;
+  (* Buffer.reset internal_buf; *)
+  Buffer.reset internal_buf;
   Buffer.add_string internal_buf (Marshal.to_string m []);
   simple_send_buf internal_buf sock
 
@@ -1442,7 +1509,7 @@ let set_max_input_buffer  t len =
 
 let set_max_output_buffer  t len =
   t.wbuf.max_buf_size <- len
-  
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         Compression                                   *)
@@ -1452,24 +1519,26 @@ let set_max_output_buffer  t len =
 let to_deflate = ref []
 let to_deflate_len = ref 0
 
-let compression_buffer_len = 20000
+let compression_buffer_len = !max_buffer_size / 10
 let compression_buffer = String.create compression_buffer_len
 
 let deflate_connection sock =
-  lprintf "Creating deflate connection\n";
+  (* lprintf "Creating deflate connection\n"; *)
   let comp = Some (Zlib.inflate_init true, Zlib.deflate_init 6 true,
       buf_create !max_buffer_size, buf_create !max_buffer_size) in
   sock.compression <- comp
 
 let rec iter_deflate sock zs wbuf =
   if wbuf.len > 0 then begin
-      lprintf "iter_deflate\n";
+(*      lprintf "iter_deflate\n"; *)
       let (_, used_in, used_out) = Zlib.deflate zs
           wbuf.buf wbuf.pos wbuf.len
           compression_buffer 0 compression_buffer_len
           Zlib.Z_SYNC_FLUSH in
-      lprintf "deflated %d/%d -> %d\n" used_in wbuf.len used_out;
+(*
+     lprintf "deflated %d/%d -> %d\n" used_in wbuf.len used_out;
       lprintf "[%s]\n" (String.escaped (String.sub compression_buffer 0 used_out));
+*)
       write sock compression_buffer 0 used_out;
       buf_used wbuf used_in;
       if used_in > 0 || used_out > 0 then
@@ -1491,7 +1560,7 @@ let deflate_timer _ =
   to_deflate := [];
   to_deflate_len := 0
 
-let to_deflate conn =
+let _to_deflate conn =
   if not (List.memq conn !to_deflate) then
     to_deflate := conn :: !to_deflate;
   if !to_deflate_len > 1000000 then
@@ -1499,13 +1568,17 @@ let to_deflate conn =
 
 let rec iter_inflate zs sock b rbuf =
   if b.len > 0 then begin
-      lprintf "iter_inflate %d\n" b.len;
-      lprintf "[%s]\n" (String.escaped (String.sub b.buf b.pos b.len));
+(*
+lprintf "iter_inflate %d\n" b.len;
+lprintf "[%s]\n" (String.escaped (String.sub b.buf b.pos b.len));
+*)
       let (_, used_in, used_out) = Zlib.inflate zs b.buf b.pos b.len
           compression_buffer 0 compression_buffer_len
           Zlib.Z_SYNC_FLUSH in
+(*
       lprintf "inflated %d/%d -> %d\n" used_in b.len used_out;
       lprintf "[%s]\n" (String.escaped (String.sub compression_buffer 0 used_out));
+*)
       buf_add sock rbuf compression_buffer 0 used_out;
       buf_used b used_in;
       if used_in > 0 || used_out > 0 then
@@ -1516,7 +1589,7 @@ let buf t =
   match t.compression with
     None -> t.rbuf
   | Some (zs, _, rbuf, _) ->
-      lprintf "CanBeCompressed.buf\n";
+(*      lprintf "CanBeCompressed.buf\n"; *)
       let b = buf t in
       if b.len > 0 then iter_inflate zs t b rbuf;
       rbuf
@@ -1527,7 +1600,7 @@ let write t s pos len =
   | Some (_,_, _,wbuf) ->
 
       to_deflate_len := !to_deflate_len + len;
-      to_deflate t;
+      _to_deflate t;
       buf_add t wbuf s pos len
 
 (*************************************************************************)
@@ -1544,7 +1617,7 @@ let _ =
       reset_connection_scheduler ();
       deflate_timer ();
   );
-  
+
   set_before_select_hook (fun _ ->
       schedule_connections ();
       List.iter (fun bc ->
@@ -1562,17 +1635,17 @@ let _ =
   );
 
 (*
-Some ideas: 
+Some ideas:
 
 * sort the connections so that we try to read and write as
   much as possible to/from connections on which we already have read/written
-  a lot. 
+  a lot.
 * postpone reads and writes for a few seconds so that we have more to
   read/write.
 
   dhcppc3:~#  ping computer.domain.org -f -s <packet_size> -c 1000
  where computer.domain.org is a nearby computer on a 100 Mbs link.
-  
+
 On my cable link, making <packet_size> vary from 1 to 2000 shows:
   * packets > 1450 are 100% lost
   * 300 > packets > 1 are 5% lost but exactly the same time (14.5 s)
@@ -1586,74 +1659,71 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
 , pipe 6, ipg/ewma 14.205/0.000 ms
 
   *)
-  
-  
+
+
   set_after_select_hook (fun _ ->
       List.iter (fun bc ->
           if bc.remaining_bytes > 0 then begin
-              
+
               bc.connections <- List.sort (fun t1 t2 ->
                   let w1 = t1.nread in
                   let w2 = t2.nread in
                   compare w1 w2
               ) bc.connections;
-              
+
               if !verbose_bandwidth > 2 && bc.nconnections > 0 then begin
                   lprintf "[BW3 %6d] %d read-waiting connections for %d allowed\n" (last_time ()) bc.nconnections bc.remaining_bytes;
                   List.iter (fun t ->
                       lprintf "[BW3 %6d]   %s:%d [buffered %d]\n" (last_time ()) t.name (sock_num t.sock_in) ((buf t).len);
                   ) bc.connections;
-                end;              
-                            
-              
+                end;
+
+
               List.iter (fun t ->
                   if bc.remaining_bytes > 0 then
                     let nconnections = maxi bc.nconnections 1 in
                     let can_read = maxi 1 (bc.remaining_bytes / nconnections) in
                     let can_read = maxi !ip_packet_size (can_read * t.read_power) in
-                    let old_nread = t.nread in
                     (try
 (*                    lprintf "allow to read %d\n" can_read; *)
                         can_read_handler t t.sock_in can_read
-                      with e -> 
+                      with e ->
 (*                          lprintf "Exception %s in can_read_handler %s:%d\n"
                             (Printexc2.to_string e)
                           t.name (sock_num t.sock_in) *) ()
                     );
-                    bc.remaining_bytes <- bc.remaining_bytes -
-                    t.nread + old_nread;
                     bc.nconnections <- bc.nconnections - t.read_power
                   else
                   if !verbose_bandwidth > 2 then begin
-                      lprintf "[BW3 %6d]    %s:%d could not read\n" (last_time ()) 
-                        t.name (sock_num t.sock_out)                   
+                      lprintf "[BW3 %6d]    %s:%d could not read\n" (last_time ())
+                        t.name (sock_num t.sock_out)
                     end
               ) bc.connections;
 (*              if bc.remaining_bytes > 0 then bc.allow_io := false; *)
             end;
-          
+
           if !verbose_bandwidth > 2 then begin
-              
-              if bc.remaining_bytes > 0 then 
+
+              if bc.remaining_bytes > 0 then
                 lprintf "[BW3 %6d] still %d bytes to read\n" (last_time ()) bc.remaining_bytes;
-              
+
               if bc.nconnections > 0 then
                 lprintf "[BW3 %6d] still %d read-waiting connections after loop\n" (last_time ()) bc.nconnections;
             end;
-          
-          
+
+
           bc.connections <- [];
           bc.nconnections <- 0;
       ) !read_bandwidth_controlers;
       List.iter (fun bc ->
           if bc.remaining_bytes > 0 then begin
-              
+
               bc.connections <- List.sort (fun t1 t2 ->
                   let w1 = remaining_to_write t1 in
                   let w2 = remaining_to_write t2 in
                   compare w1 w2
               ) bc.connections;
-              
+
               if !verbose_bandwidth > 2 && bc.nconnections > 0 then begin
                   lprintf "[BW3 %6d] %d write-waiting connections for %d allowed\n" (last_time ()) bc.nconnections bc.remaining_bytes;
                   List.iter (fun t ->
@@ -1676,24 +1746,24 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
                     bc.nconnections <- bc.nconnections - t.write_power;
                   else
                   if !verbose_bandwidth > 2 then begin
-                      lprintf "[BW3 %6d]    %s:%d could not write buffered %d bytes\n" (last_time ()) 
+                      lprintf "[BW3 %6d]    %s:%d could not write buffered %d bytes\n" (last_time ())
                         t.name (sock_num t.sock_out) (remaining_to_write t)
                     end
               ) bc.connections;
 (*          if bc.remaining_bytes > 0 then bc.allow_io := false; *)
             end;
           if !verbose_bandwidth > 2 then begin
-              
+
               if bc.remaining_bytes > 0 then begin
                   lprintf "[BW3 %6d] still %d bytes to write\n" (last_time ()) bc.remaining_bytes;
                   List.iter (fun t ->
                       let len = remaining_to_write t in
                       if len > 0 then
-                        lprintf "[BW3 %6d]    %s:%d could write %d\n" (last_time ()) 
+                        lprintf "[BW3 %6d]    %s:%d could write %d\n" (last_time ())
                           t.name (sock_num t.sock_out) len
                   ) bc.connections
                 end;
-              
+
               if bc.nconnections > 0 then
                 lprintf "[BW3 %6d] still %d write-waiting connections after loop\n" (last_time ()) bc.nconnections;
             end;
@@ -1701,11 +1771,11 @@ to an option 'packet_unit' (default 250). Ask Simon if it makes sense.
           bc.nconnections <- 0;
       ) !write_bandwidth_controlers
   )
-  
+
 let prevent_close t =
-  prevent_close t.sock_in;  
+  prevent_close t.sock_in;
   prevent_close t.sock_out
-  
+
 let must_write t bool = must_write t.sock_out bool
 let output_buffered t = t.wbuf.len
 
@@ -1728,13 +1798,13 @@ let join_stats titles values =
             old_value := value
           with _ ->
               Hashtbl.add net_stats title (ref value);
-              if !verbose_bandwidth>0 then begin
+              if !verbose_bandwidth > 0 then begin
                   lprintf "[BWS] INIT %s: %Ld\n"
                     title value
               end;
       ) fields_titles fields_values
   | _ -> assert false
-  
+
 let load_stats filename =
   try
     let ic = open_in filename in
@@ -1745,30 +1815,37 @@ let load_stats filename =
       let values = input_line ic in
       join_stats titles values;
       iter_first ic
-    
+
     in
     try
       iter_first ic
-    with 
+    with
       End_of_file -> close_in ic
-    | e -> 
+    | e ->
         lprintf "[BWS] Error %s reading %s\n" (Printexc2.to_string e) filename;
         close_in ic; proc_net_fs := false
-  with 
-  | e -> 
+  with
+  | e ->
       lprintf "[BWS] Error %s opening %s\n" (Printexc2.to_string e) filename;
       proc_net_fs := false
-      
+
 let proc_net_timer _ =
   if !proc_net_fs && !verbose_bandwidth > 0 then begin
       load_stats "/proc/net/netstat";
       load_stats "/proc/net/snmp";
     end
-    
-  
+
+
 let _ =
-  add_infinite_timer 1.0 proc_net_timer 
-  
-  
+  Heap.add_memstat "tcpBufferedSocket" (fun level buf ->
+      Printf.bprintf buf "  %d latencies\n" (Hashtbl.length latencies);
+      Printf.bprintf buf "  %d entries in net_stats\n" (Hashtbl.length net_stats);
+      Printf.bprintf buf "  String.length big_buffer: %d\n" (String.length big_buffer);
+      Printf.bprintf buf "  connection_managers: %d\n" (List.length !connection_managers);
+      Printf.bprintf buf "  read_bandwidth_controlers: %d\n" (List.length !read_bandwidth_controlers);
+      Printf.bprintf buf "  write_bandwidth_controlers: %d\n" (List.length !write_bandwidth_controlers);
+      Printf.bprintf buf "  to_deflate: %d\n" (List.length !to_deflate);
+  );
+  add_infinite_timer 1.0 proc_net_timer
+
 let packet_frame_size = ref 250
-  
\ No newline at end of file
