@@ -56,6 +56,7 @@ val create : token -> string -> Unix.file_descr -> handler -> t
 val create_simple : token -> string -> Unix.file_descr -> t
 val create_blocking : token -> string -> Unix.file_descr -> handler -> t
 val buf : t -> buf
+val setsock_iptos_throughput : t -> unit
 val set_reader : t -> (t -> int -> unit) -> unit
 val sock_used : t -> int -> unit
 val buf_create : int -> buf
@@ -78,7 +79,9 @@ val set_max_input_buffer : t -> int -> unit
 val set_max_output_buffer : t -> int -> unit  
 val can_write : t -> bool  
 val can_write_len : t -> int -> bool  
-val set_monitored : t -> unit
+  
+val set_monitored : t -> bool -> unit
+val monitored : t -> bool
   
 val close_after_write : t -> unit
 
@@ -90,8 +93,10 @@ val change_rate : bandwidth_controler -> int -> unit
 
   
 val my_ip : t -> Ip.t
+val peer_addr : t -> Ip.t * int
 val peer_ip : t -> Ip.t
-val host : t -> Ip.t * int
+val peer_port : t -> int
+(*val host : t -> Ip.t * int *)
   
 val stats :  Buffer.t -> t -> unit
 val buf_size : t -> int * int
@@ -167,4 +172,5 @@ val prevent_close : t -> unit
 val must_write : t -> bool -> unit
 val max_refill : t -> int
 val output_buffered : t -> int
-  
\ No newline at end of file
+  
+val get_latencies : bool ref -> string
